/*
 * VexdUM functions
 *
 * (c) 2002-2010 - AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _vexdum_included
  #endinput
#endif
#define _vexdum_included

#include <VexdUM_const>

/* Called once every server frame. */
forward server_frame();

/* Called when 2 entities touch. */
forward entity_touch(entity1, entity2);

/* Called on an entity's DispatchThink. */
forward entity_think(entity);

/* Called when an entity is spawned. */
forward entity_spawn(entity);

/* Called before the engine calculates player movement, etc. */
forward client_prethink(id);

/* Called after the engine has calculated player movement. */
forward client_postthink(id);

/* Called when a client types "kill" in console. */
forward client_kill(id);

/* Called when an emitting sound is played server-side. */
forward emitsound(entity, const sample[]);

/* Called when an emitting ambient sound is played server-side. */
forward emitambientsound(entity, const sample[]);

/* Called when a model spawns. */
forward set_model(entity, const model[]);

/* Called whatever an entity looks at. */
forward traceline(entity);

/* Called when a monster is hurt by VexdUM damage natives. */
forward monster_hurt(monster, attacker, damage);

/* Called when an entity gets a keyvalue set on it from the engine.
 * Use copy_keyvalue to get the keyvalue information. */
forward keyvalue(entity);

/* Called when a keyvalue is set on a player. */
forward setclientkeyvalue(id, key[], value[]);

/* Called when an alert message is sent. */
forward alertmessage(atype, message[]);

/* Called when gamedescription is called. */
forward gamedescription();

/* Forward AddToFullPack (pre/post).
 * See HLSDK for more informations about this forward.
 * Careful: Called many times (more than 1000 times per second). */
forward addtofullpack(es, e, iEnt, id, hostflags, player, pSet);
forward addtofullpack_post(es, e, iEnt, id, hostflags, player, pSet);

/* Sets a user model, model would be "vip" for the vip model...
 * By default it clears the model if there is no defined model (just set_user_model(id)). */
native set_user_model(id, const Model[] = "");

/* Sets model (TFC). */
native TFC_SetModel(id, const Model[], const Skin[]);

/* Clears model (TFC). */
native TFC_ClearModel(id);

/* Sets player's view to entity index. */
native attach_view(id, ent);

/* Gets player's speak flags. */
native get_speak(id, target = 0);

/* Sets player's speak flags.
 * Speak flags:
 *  SPEAK_NORMAL
 *  SPEAK_MUTED
 *  SPEAK_ALL
 *  SPEAK_LISTENALL */
native set_speak(id, iSpeakFlags, target = 0);

/* Sets map lightning, #OFF to disable. */
native set_lights(const Lightning[]);

/* Gets/Sets things in an entities Entvars Struct. See HLSDK or look at examples to determine what different variables do.
 * iVar is an EV_* from VexdUM_const.inc. */
native entity_get_int(ent, iVar);
native entity_set_int(ent, iVar, iVal);
native Float:entity_get_float(ent, iVar); 
native entity_set_float(ent, iVar, Float:fVal);
native entity_get_vector(ent, iVar, Float:vVal[3]);
native entity_set_vector(ent, iVar, const Float:vVal[3]);
native entity_get_edict(ent, iVar);
native entity_set_edict(ent, iVar, iEnt);
native entity_get_string(ent, iVar, szVal[], iLen); 
native entity_set_string(ent, iVar, const szVal[]);
native entity_get_byte(ent, iVar);
native entity_set_byte(ent, iVar, iVal);

/* Sets the model of an entity. */
native entity_set_model(ent, const szModel[]);

/* Proper origin setting, keeps updated with Half-Life engine. */
native entity_set_origin(ent, const Float:Orig[3]);

/* Sets the bounding box size of an entity. */
native entity_set_size(ent, const Float:vecMin[3], const Float:vecMax[3]);

/* Gets things in Globalvars Struct. See HLSDK or look at examples to determine what different variables do.
 * iVar is a GV_* from VexdUM_const.inc. */
native global_get_int(iVar);
native Float:global_get_float(iVar);
native global_get_vector(iVar, Float:vVal[3]);
native global_get_edict(iVar);
native global_get_string(iVar, szVal[], iLen);

/* Gets/Sets things in a players TraceLine Struct. See HLSDK or look at examples to determine what different variables do.
 * Can only be called in traceline(entity) forward function.
 * iVar is a TR_* from VexdUM_const.inc. */
native traceline_get_int(iVar);
native traceline_set_int(iVar, iVal);
native Float:traceline_get_float(iVar);
native traceline_set_float(iVar, Float:fVal);
native traceline_get_vector(iVar, Float:vVal[3]);
native traceline_set_vector(iVar, Float:vVal[3]);
native traceline_get_edict(iVar);
native traceline_set_edict(iVar, iEnt);

/* Some damage functions. */
native radius_damage(inflictor, Float:dmg, Float:orig[3], Float:rad, bit = DMG_BULLET, wpnName[] = "", hs = 0);
native take_damage(victim, attacker, Float:orig[3], Float:dmg, bit = DMG_BULLET, wpnName[] = "", hs = 0);
native client_death(victim, attacker, wpnName[] = "", hs = 0);

/* Will return the contents of a point (inside map? in sky? outside map? etc.).
 * Result is a CONTENTS_* from VexdUM_const.inc. */
native PointContents(const Float:fCheckAt[3]);

/* Dispatch a KeyValuePair, used for initalizing entities when a map spawns them.
 * Format: DispatchKeyValue("KeyName", "Value") - Sets keyvalue for the entity specified in the keyvalue() forward.
 * Format: DispatchKeyValue(ent, "KeyName", "Value") - Sets keyvalue for entity not specified in keyvalue() forward. */
native DispatchKeyValue(...);

/* Runs the GameDLL's DispatchSpawn for an entity, I think it's used with DispatchKeyValue. */
native DispatchSpawn(ent);

/* Creates an entity, will return the index of the created entity. ClassName must be valid. */
native create_entity(const szClassname[]);

/* Finds an entity in the world, will return -1 if nothing is found.
 * Type (optionnal):
 *   0 - "classname" (default)
 *   1 - "globalname"
 *   2 - "model"
 *   3 - "target"
 *   4 - "targetname"
 *   5 - "netname"
 *   6 - "message"
 *   7 - "noise"
 *   8 - "noise1"
 *   9 - "noise2"
 *  10 - "noise3"
 *  11 - "viewmodel"
 *  12 - "weaponmodel" */
native find_entity(ent, const szValue[], type = 0);

/* Finds an entity within a given origin and radius. */
native find_entity_sphere(ent, const Float:Orig[3], Float:Rad);

/* Removes an entity from the world. */
native remove_entity(ent);

/* Traces a line from Start(X, Y, Z) to End(X, Y, Z), will return the point hit in vReturn[3].
 * Will return an entindex if an entity is hit. */
native trace_line(ent, const Float:fStart[3], const Float:fEnd[3], Float:vRtn[3]);

/* Traces a hull. */
native trace_hull(const Float:fOrigin[3], hull, ignoredent = 0, ignoremonsters = 0);

/* Traces a line, and returns the normal to the plane hit in vReturn.
 * Returns 0 if theres no normal. */
native trace_normal(ent, const Float:fStart[3], const Float:fEnd[3], Float:vRtn[3]);

/* Traces what texture an entity is looking at or standing on.
 * Setting type:
 *  0 - Trace Forward
 *  1 - Trace Below */
native trace_texture(ent, szBuf[], iLen, set = 0);

/* Traces result. */
native trace_result(type, {Float,_}:...);

/* Precache event. */
native precache_event(type, const szEvent[]);

/* Playback a precached event. */
native playback_event(flags, invoker, eventindex, Float:delay, const Float:origin[3], const Float:angles[3], Float:fparam1, Float:fparam2, iparam1, iparam2, bparam1, bparam2);

/* Gets a integer from index's pvPrivateData at offset. */
native get_offset_int(ent, offset, linos = 5);

/* Sets a integer value in index's pvPrivateData at offset. */
native set_offset_int(ent, offset, value, linos = 5);

/* Gets a float from index's pvPrivateData at offset. */
native Float:get_offset_float(ent, offset, linos = 5);

/* Sets a float value in index's pvPrivateData at offset. */
native set_offset_float(ent, offset, Float:value, linos = 5);

/* Checks if index is an existing ent. */
native is_entity(ent);

/* Gets number of ents in a map. */
native get_num_ents();

/* Returns maxentities setting. */
native get_maxentities();

/* Gets whole buffer containing keys and their data. */
native get_info_keybuffer(id, szBuffer[], length);

/* Gets keyvalue from the entity. */
native get_keyvalue(ent, const szKey[], szValue[], length);

/* Copy keyvalue when keyvalue forward is called. */
native copy_keyvalue(szClassName[], iLen1, szKeyName[], iLen2, szValue[], iLen3);

/* Sets keyvalue on a player. */
native set_client_keyvalue(id, szKey[], szValue[]);

/* Gets decal index. */
native get_decal_index(const szDecalName[]);

/* Fake one entity touching another. */
native fake_touch(ent1, ent2);

/* CBaseEntity::Think() from the entity. */
native call_think(entity);

/* Calls a Half-Life engine DLL function.
 * type is a DLLFunc_* from VexdUM_const.inc. */
native dllfunc(type, {Float,_}:...);

/* Sets game description. Can only be called in gamedescription() forward function. */
native set_gamedescription(desc[]);

/* These functions are used with the entity_state data structure (AddToFullPack forward).
 * Get:
 *  0 - extra params - return integer
 *  1 - extra param - by ref float or vector or array
 * Set: Use anything
 * Use 0 for es_handle to specify the global entity_state handle. 
 * member is an ES_* from VexdUM_const.inc. */
native get_es(es_handle, EntityState:member, {Float,_}:...);
native set_es(es_handle, EntityState:member, {Float,_}:...);

/* Returns data for Metamod. */
native forward_return(type, {Float,_}:...);

/* Bot natives. */
native create_bot(name[]);
native remove_bot(id);
native control_bot(id, Float:vAngles[3], Float:fMove, Float:sMove, Float:uMove, Buttons, Impulse, mSec);

/* Some misc. natives. */
native dot_points(Float:vSrc[3], Float:vCheck[3], Float:vDir[3]);
native Float:dot_product(Float:vecA[3], Float:vecB[3]);
native normalize(Float:vVec[3], Float:vRet[3]);
native can_see(ent1, ent2);
native in_view_cone(ent, const Float:Orig[3]);
native drop_to_floor(ent);
native entity_use(eUsed, eOther);
native index_to_pointer(index);
native pointer_to_index(pointer);
native make_vectors(Float:vVec[3]);

/* Include the stock functions. */
#include <VexdUM_stock>
