/*
 * VexdUM stock functions
 *
 * (c) 2002-2010 - AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _vexdum_stock_included
  #endinput
#endif
#define _vexdum_stock_included

/* Gets the button(s) a user is pressing.
 * Button used have to be an IN_* from VexdUM_const.inc.
 * Example: get_user_button(id) & IN_ATTACK */
stock get_user_button(id) {
  return entity_get_int(id, EV_INT_button)
}

/* Gets the oldbutton(s) a user is pressing.
 * Button used have to be an IN_* from VexdUM_const.inc.
 * Example: get_user_oldbutton(id) & IN_DUCK */
stock get_user_oldbutton(id) {
  return entity_get_int(id, EV_INT_oldbuttons)
}

/* Returns the flag an entity is flagged with.
 * Flag used have to be a flag from VexdUM_const.inc. */
stock get_entity_flags(ent) {
  return entity_get_int(ent, EV_INT_flags)
}

/* Returns the distance between two entities. */
stock get_entity_distance(ent1, ent2) {
  new Float:orig1[3], Float:orig2[3]
  entity_get_vector(ent1, EV_VEC_origin, orig1)
  entity_get_vector(ent2, EV_VEC_origin, orig2)
  return floatround(vector_distance(orig1, orig2))
}

/* Gets the grenade is thrown by a user. */
stock get_grenade(id, grenadeid = -1) {
  new iGrenade = find_entity(grenadeid, "grenade")
  while(iGrenade > 0) {
    if(entity_get_edict(iGrenade, EV_ENT_owner) == id) {
      return iGrenade
    }
    iGrenade = find_entity(iGrenade, "grenade")
  }
  return -1
}

/* Gets the origin of a brush entity. */
stock get_brush_entity_origin(ent, Float:orig[3]) {
  new Float:Min[3], Float:Max[3]
  entity_get_vector(ent, EV_VEC_absmin, Min)
  entity_get_vector(ent, EV_VEC_absmax, Max)
  orig[0] = (Min[0] + Max[0]) * 0.5
  orig[1] = (Min[1] + Max[1]) * 0.5
  orig[2] = (Min[2] + Max[2]) * 0.5
  return 1
}

/* Removes all entities by name. */
stock remove_entities(const eName[]) {
  new iEntity = find_entity(-1, eName)
  while(iEntity > 0) {
    remove_entity(iEntity)
    iEntity = find_entity(-1, eName)
  }
  return 1
}

/* Get the contents of the point a player is aiming at. */
stock ViewContents(id) {
  new origin[3], Float:Orig[3]
  get_user_origin(id, origin, 3)
  Orig[0] = float(origin[0])
  Orig[1] = float(origin[1])
  Orig[2] = float(origin[2])
  return PointContents(Orig)
}

/* Checks the entity's flag (return true if valid, false otherwise).
 * Flag have to be an FL_* from VexdUM_const.inc. */
stock bool:CheckFlag(ent, flag) {
  switch(flag) {
    case FL_FLY: if(entity_get_int(ent, EV_INT_flags) & FL_FLY) return true
    case FL_SWIM: if(entity_get_int(ent, EV_INT_flags) & FL_SWIM) return true
    case FL_CONVEYOR: if(entity_get_int(ent, EV_INT_flags) & FL_CONVEYOR) return true
    case FL_CLIENT: if(entity_get_int(ent, EV_INT_flags) & FL_CLIENT) return true
    case FL_INWATER: if(entity_get_int(ent, EV_INT_flags) & FL_INWATER) return true
    case FL_MONSTER: if(entity_get_int(ent, EV_INT_flags) & FL_MONSTER) return true
    case FL_GODMODE: if(entity_get_int(ent, EV_INT_flags) & FL_GODMODE) return true
    case FL_NOTARGET: if(entity_get_int(ent, EV_INT_flags) & FL_NOTARGET) return true
    case FL_SKIPLOCALHOST: if(entity_get_int(ent, EV_INT_flags) & FL_SKIPLOCALHOST) return true
    case FL_ONGROUND: if(entity_get_int(ent, EV_INT_flags) & FL_ONGROUND) return true
    case FL_PARTIALGROUND: if(entity_get_int(ent, EV_INT_flags) & FL_PARTIALGROUND) return true
    case FL_WATERJUMP: if(entity_get_int(ent, EV_INT_flags) & FL_WATERJUMP) return true
    case FL_FROZEN: if(entity_get_int(ent, EV_INT_flags) & FL_FROZEN) return true
    case FL_FAKECLIENT: if(entity_get_int(ent, EV_INT_flags) & FL_FAKECLIENT) return true
    case FL_DUCKING: if(entity_get_int(ent, EV_INT_flags) & FL_DUCKING) return true
    case FL_FLOAT: if(entity_get_int(ent, EV_INT_flags) & FL_FLOAT) return true
    case FL_GRAPHED: if(entity_get_int(ent, EV_INT_flags) & FL_GRAPHED) return true
    case FL_IMMUNE_WATER: if(entity_get_int(ent, EV_INT_flags) & FL_IMMUNE_WATER) return true
    case FL_IMMUNE_SLIME: if(entity_get_int(ent, EV_INT_flags) & FL_IMMUNE_SLIME) return true
    case FL_IMMUNE_LAVA: if(entity_get_int(ent, EV_INT_flags) & FL_IMMUNE_LAVA) return true
    case FL_PROXY: if(entity_get_int(ent, EV_INT_flags) & FL_PROXY) return true
    case FL_ALWAYSTHINK: if(entity_get_int(ent, EV_INT_flags) & FL_ALWAYSTHINK) return true
    case FL_BASEVELOCITY: if(entity_get_int(ent, EV_INT_flags) & FL_BASEVELOCITY) return true
    case FL_MONSTERCLIP: if(entity_get_int(ent, EV_INT_flags) & FL_MONSTERCLIP) return true
    case FL_ONTRAIN: if(entity_get_int(ent, EV_INT_flags) & FL_ONTRAIN) return true
    case FL_WORLDBRUSH: if(entity_get_int(ent, EV_INT_flags) & FL_WORLDBRUSH) return true
    case FL_SPECTATOR: if(entity_get_int(ent, EV_INT_flags) & FL_SPECTATOR) return true
    case FL_CUSTOMENTITY: if(entity_get_int(ent, EV_INT_flags) & FL_CUSTOMENTITY) return true
    case FL_KILLME: if(entity_get_int(ent, EV_INT_flags) & FL_KILLME) return true
    case FL_DORMANT: if(entity_get_int(ent, EV_INT_flags) & FL_DORMANT) return true
  }
  return false
}

/* Returns the speed of an entity's velocity. */
stock get_speed(ent) {
  new Float:Vel[3]
  entity_get_vector(ent, EV_VEC_velocity, Vel)
  return floatround(vector_length(Vel))
}

/* Sets the rendering of an entity. */
stock set_rendering(index, fx = kRenderFxNone, r = 255, g = 255, b = 255, render = kRenderNormal, amount = 16) {
  entity_set_int(index, EV_INT_renderfx, fx)
  new Float:RenderColor[3]
  RenderColor[0] = float(r)
  RenderColor[1] = float(g)
  RenderColor[2] = float(b)
  entity_set_vector(index, EV_VEC_rendercolor, RenderColor)
  entity_set_int(index, EV_INT_rendermode, render)
  entity_set_float(index, EV_FL_renderamt, float(amount))
  return 1
}

/* Sets a flag on an entity.
 * Flag have to be an FL_* from VexdUM_const.inc. */
stock set_entity_flags(ent, flag, onoff) {
  if((entity_get_int(ent, EV_INT_flags) & flag) > 0 && onoff == 0) {
    entity_set_int(ent, EV_INT_flags, entity_get_int(ent, EV_INT_flags) - flag)
    return 1
  }
  else if(onoff == 1) {
    entity_set_int(ent, EV_INT_flags, entity_get_int(ent, EV_INT_flags) + flag)
    return 1
  }
  return 0
}

/* Gets a user model. Returns 0 if this is not a player or valid id.
 * Note: This function is currently disabled. */
/*stock get_user_model(id, model[], len) {
  new classname[32]
  entity_get_string(id, EV_SZ_classname, classname, charsmax(classname))
  if(!equal(classname, "player") || !(1 <= id <= get_maxplayers())) {
    return 0
  }
  //return get_keyvalue(id, "model", model, len) // you can use that to get model name
  entity_get_string(id, EV_SZ_model, model, len)
  if(model[0]) {
    replace(model, len, ".mdl", "")
    new string[2], pos = 0, slashcount = 0
    for(new i = 0; i < len; ++i) {
      if(model[i] == '/') ++slashcount
    }
    while(pos < len) {
      copy(string, 1, model[pos])
      if(model[pos] == '/') {
        replace(model[pos], len, string, "")
        if(slashcount > 1) {
          --slashcount
        }
        else {
          break
        }
      }
      else {
        replace(model[pos], len, string, "")
      }
      --len
    }
    return model[len]
  }
  return 0
}*/
