/*
 * Vexd Utility Module backwards compatibility
 *
 * (c) 2002-2010 - AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _Vexd_Utilities_included
  #endinput
#endif
#define _Vexd_Utilities_included

#include <VexdUM>

/* Sets player's view to entity index. */
stock AttachView(iIndex, iTargetIndex)
  return attach_view(iIndex, iTargetIndex)

/* Sets a user model, model would be "vip" for the vip model... */
stock CS_SetModel(id, const Model[])
  return set_user_model(id, Model)

/* Clears a user model. */
stock CS_ClearModel(id)
  return set_user_model(id)

/* Gets player's speak flags. */
stock GetSpeak(iIndex, target = 0)
  return get_speak(iIndex, target)

/* Sets player's speak flags.
 * Speak flags:
 *  SPEAK_NORMAL
 *  SPEAK_MUTED
 *  SPEAK_ALL
 *  SPEAK_LISTENALL */
stock SetSpeak(iIndex, iSpeakFlags)
  return set_speak(iIndex, iSpeakFlags)

/* Sets map lightning, #OFF to disable. */
stock SetLights(const Lighting[])
  return set_lights(Lighting)

/* Returns an integer from an entity's structure. */
stock Entvars_Get_Int(iIndex, iVariable)
  return entity_get_int(iIndex, iVariable)

/* Sets an integer value on an entity's structure. */
stock Entvars_Set_Int(iIndex, iVariable, iNewValue)
  return entity_set_int(iIndex, iVariable, iNewValue)

/* Returns a float from an entity's structure. */
stock Float:Entvars_Get_Float(iIndex, iVariable)
  return entity_get_float(iIndex, iVariable)

/* Sets a float value on an entity's structure. */
stock Entvars_Set_Float(iIndex, iVariable, Float:fNewValue)
  return entity_set_float(iIndex, iVariable, fNewValue)

/* Gets a vector from an entity's structure. */
stock Entvars_Get_Vector(iIndex, iVariable, Float:vRetVector[3])
  return entity_get_vector(iIndex, iVariable, vRetVector)

/* Sets a vector on an entity's structure. */
stock Entvars_Set_Vector(iIndex, iVariable, const Float:vNewVector[3])
  return entity_set_vector(iIndex, iVariable, vNewVector)

/* Returns the entity index of an entity's structure key. */
stock Entvars_Get_Edict(iIndex, iVariable)
  return entity_get_edict(iIndex, iVariable)

/* Sets an edict value on an entity's structure. */
stock Entvars_Set_Edict(iIndex, iVariable, iNewIndex)
  return entity_set_edict(iIndex, iVariable, iNewIndex)

/* Gets a string from an entity's structure. */
stock Entvars_Get_String(iIndex, iVariable, szReturnValue[], iReturnLen)
  return entity_get_string(iIndex, iVariable, szReturnValue, iReturnLen)

/* Sets a string on an entity's structure. */
stock Entvars_Set_String(iIndex, iVariable, const szNewValue[])
  return entity_set_string(iIndex, iVariable, szNewValue)

/* Returns a byte from an entity's structure. */
stock Entvars_Get_Byte(iIndex, iVariable)
  return entity_get_byte(iIndex, iVariable)

/* Sets a byte on an entity's structure. */
stock Entvars_Set_Byte(iIndex, iVariable, iNewValue)
  return entity_set_byte(iIndex, iVariable, iNewValue)

/* Sets the model of an entity. */
stock ENT_SetModel(iIndex, const szModel[])
  return entity_set_model(iIndex, szModel)

/* Proper origin setting, keeps updated with Half-Life engine. */
stock ENT_SetOrigin(iIndex, const Float:fNewOrigin[3])
  return entity_set_origin(iIndex, fNewOrigin)

/* Sets the bounding box size of an entity. */
stock ENT_SetSize(iIndex, const Float:vecMin[3], const Float:vecMax[3])
  return entity_set_size(iIndex, vecMin, vecMax)

/* Creates an entity, will return the index of the created entity. ClassName must be valid. */
stock CreateEntity(const szClassname[])
  return create_entity(szClassname)

/* Find an entity in the world, will return -1 if nothing is found.
 * Type (optionnal):
 *   0 - "classname" (default)
 *   1 - "globalname"
 *   2 - "model"
 *   3 - "target"
 *   4 - "targetname"
 *   5 - "netname"
 *   6 - "message"
 *   7 - "noise"
 *   8 - "noise1"
 *   9 - "noise2"
 *  10 - "noise3"
 *  11 - "viewmodel"
 *  12 - "weaponmodel" */
stock FindEntity(iIndex, const szValue[], type = 0)
  return find_entity(iIndex, szValue, type)

/* Remove an entity from the world. */
stock RemoveEntity(iIndex)
  return remove_entity(iIndex)

/* Trace a line from Start(X, Y, Z) to End(X, Y, Z), will return the point hit in vReturn[3].
 * Will return an entindex if an entity is hit. */
stock TraceLn(iIgnoreEnt, const Float:fStart[3], const Float:fEnd[3], Float:vReturn[3])
  return trace_line(iIgnoreEnt, fStart, fEnd, vReturn)

/* Traces a line, and returns the normal to the plane hit in vReturn.
 * Returns 0 if theres no normal. */
stock TraceNormal(iIgnoreEnt, const Float:fStart[3], const Float:fEnd[3], Float:vReturn[3])
  return trace_normal(iIgnoreEnt, fStart, fEnd, vReturn)

/* Changes a vector to an angle vector. */
stock VecToAngles(Float:fVector[3], Float:vReturn[3])
  return vector_to_angle(fVector, vReturn)

/* Gets the length of a vector (float[3]). */
stock Float:VecLength(Float:vVector[3])
  return vector_length(vVector)

/* Gets the distance between 2 vectors (float[3]). */
stock Float:VecDist(Float:vVector[3], Float:vVector2[3])
  return vector_distance(vVector, vVector2)

/* Returns the block flag on a message. */
stock GetMessageBlock(iMessage)
  return get_msg_block(iMessage)

/* Sets whether a message is blocked or not. */
stock MessageBlock(iMessage, iMessageFlags)
  return set_msg_block(iMessage, iMessageFlags)

/* Returns the gpGlobal time from Half-Life. */
stock Float:HLTime()
  return get_gametime()

/* Simulates two entities colliding/touching. */
stock FakeTouch(iToucher, iTouched)
  return fake_touch(iToucher, iTouched)

/* Hurts/Kills players in a sphere, like an explosion, Multiplier determines damage. */
stock RadiusDamage(Float:fExplodeAt[3], iDamageMultiplier, iRadiusMultiplier)
  return radius_damage(0, float(iDamageMultiplier), fExplodeAt, float(iRadiusMultiplier), DMG_BLAST)

#define vexd_pfntouch(%1,%2) entity_touch(%1,%2)
#define ServerFrame() server_frame()
