/******************************************************************************************

  Xtrafun_to_Vexd.inc v1.1

  Authors: KRoT@L/AMX Mod Team

  Use this library to convert plugins from Xtrafun to AMX 0.9.9/2006/2010 Vexd Utilities Module.
  Put this Xtrafun_to_Vexd.inc file in the include folder (amx/examples/include).
  Replace #include <xtrafun> with #include <Xtrafun_to_Vexd> in the .sma file.
  Then compile the plugin.

  You have to replace yourself find_entity(ent, category, value[])
  with find_entity(ent, value[]) if this function is used in the source code.
  Example: find_entity(ent, classname, "Bullet")
           => find_entity(ent, "Bullet")

******************************************************************************************/

#if defined _Xtrafun_to_Vexd_included
  #endinput
#endif
#define _Xtrafun_to_Vexd_included

#include <vector>
#include <VexdUM>

/* Gets a entity's velocity. */
stock get_entity_velocity(id, velocity[3]) {
  new Float:vel[3]
  entity_get_vector(id, EV_VEC_velocity, vel)
  FVecIVec(vel, velocity)
  return 1
}

/* Sets a entity's velocity. */
stock set_entity_velocity(id, velocity[3]) {
  new Float:vel[3]
  IVecFVec(velocity, vel)
  entity_set_vector(id, EV_VEC_velocity, vel)
  return 1
}

/* Gets an entity's origin. */
stock get_entity_origin(id, origin[3]) {
  new Float:orig[3]
  entity_get_vector(id, EV_VEC_origin, orig)
  FVecIVec(orig, origin)
  return 1
}

/* Sets an entity's origin. */
stock set_entity_origin(id, origin[3]) {
  new Float:orig[3]
  IVecFVec(origin, orig)
  entity_set_vector(id, EV_VEC_origin, orig)
  return 1
}

/* Gets a player's velocity. */
stock get_user_velocity(id, velocity[3]) {
  new Float:vel[3]
  entity_get_vector(id, EV_VEC_velocity, vel)
  FVecIVec(vel, velocity)
  return 1
}

/* Sets a player's velocity. */
stock set_user_velocity(id, velocity[3]) {
  new Float:vel[3]
  IVecFVec(velocity, vel)
  entity_set_vector(id, EV_VEC_velocity, vel)
  return 1
}

/* Gets the ID of a grenade. */
stock get_grenade_id(id, model[], len, grenadeid) {
  grenadeid = get_grenade(id, grenadeid)
  entity_get_string(grenadeid, EV_SZ_model, model, len)
  return grenadeid
}

/* Gets the index of the grenade belonging to index.
 * Model of grenade is returned in model[].
 * Specify the grenadeindex to start searching from,
 * or leave it at 0 to search from the start.
 * Returns grenade index.
 * Paths + models of grenades in Counter-Strike:
 *   HEGRENADE = "models/w_hegrenade.mdl"
 *   FLASHBANG = "models/w_flashbang.mdl"
 *   SMOKEGRENADE = "models/w_smokegrenade.mdl" */
stock get_grenade_index(index, model[], len, grenadeindex = 0) {
  if(grenadeindex == 0) grenadeindex = -1
  new entfind = grenadeindex
  new entowner = index
  for(;;) {
    entfind = find_entity(entfind, "grenade")
    if(entfind > 0 && is_entity(entfind)) {
      if(entity_get_edict(entfind, EV_ENT_owner) == entowner) {
        entity_get_string(entfind, EV_SZ_model, model, len)
        return entfind
      }
    }
    else {
      return 0
    }
  }
}

/* Returns the number of entities in the world. */
stock current_num_ents() {
  return get_num_ents()
}
