/*
 * AMX Mod misc. functions
 *
 * (c) 2002-2012 - AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _amxmisc_included
  #endinput
#endif
#define _amxmisc_included

/* Checks if the user has a specific admin flag (returns true of false).
 * Note: Use this function instead of get_user_flags(id)
 * if your command doesn't require any flags (as ADMIN_ALL). */
stock access(id, level) {
  new flags = get_user_flags(id)
  switch(level) {
    case ADMIN_ALL: return true
    case ADMIN_ADMIN: return ((flags & ADMIN_ADMIN) != 0)
  }
  return ((flags & level) == level)
}

/* Checks if a user has access for that command and if the number of arguments supplied by the user is enough.
 * commandMinNumberOfArguments should be the minimum number of arguments required + 1
 * because the command itself is counted as an argument. */
stock cmd_access(id, level, commandId, commandMinNumberOfArguments) {
  if(!access(id, level) && (id != (is_dedicated_server() ? 0 : 1))) {
#if defined _translator_included
    console_print(id, _T("You have no access to that command."))
#else
    console_print(id, "You have no access to that command.")
#endif
    return 0
  }
  if(read_argc() < commandMinNumberOfArguments) {
    new hcmd[32], hinfo[128], hflag
    get_concmd(commandId, hcmd, charsmax(hcmd), hflag, hinfo, charsmax(hinfo), level)
#if defined _translator_included
    if(is_translated(hinfo)) {
      console_print(id, _T("Usage: %s %s"), hcmd, _T(hinfo))
    }
    else {
      console_print(id, _T("Usage: %s %s"), hcmd, hinfo)
    }
#else
    console_print(id, "Usage: %s %s", hcmd, hinfo)
#endif
    return 0
  }
  return 1
}

/* Finds a player matching arg and flags parameters.
 * Flags:
 *  1 - obey immunity
 *  2 - allow yourself
 *  4 - must be alive
 *  8 - can't be a bot
 * Note: If you choose do not put any flag, immunity and yourself flags will be included by default. */
stock cmd_target(id, const arg[], flags = 3) {
  new player = find_player("bl", arg)
  if(player) {
    if(player != find_player("blj",arg)) {
#if defined _translator_included
      console_print(id, _T("There are more players matching to your argument."))
#else
      console_print(id, "There are more players matching to your argument.")
#endif
      return 0
    }
  }
  else if((player = find_player("c", arg)) == 0 && arg[0] == '#' && arg[1]) {
    player = find_player("k", strtonum(arg[1]))
  }
  if(!player) {
#if defined _translator_included
    console_print(id, _T("Player with that name or userid not found."))
#else
    console_print(id, "Player with that name or userid not found.")
#endif
    return 0
  }
  if(!(flags & 2) && (player == id)) {
#if defined _translator_included
    console_print(id, _T("That action can't be performed on yourself."))
#else
    console_print(id, "That action can't be performed on yourself.")
#endif
    return 0
  }
  if(flags & 1) {
    if((get_user_flags(player) & ADMIN_IMMUNITY) && (player != id) && !(get_user_flags(id) & ADMIN_SUPREME)) {
      new imname[32]
      get_user_name(player, imname, charsmax(imname))
#if defined _translator_included
      console_print(id, _T("Player ^"%s^" has immunity."), imname)
#else
      console_print(id, "Player ^"%s^" has immunity.", imname)
#endif
      return 0
    }
  }
  if(flags & 4) {
    if(!is_user_alive(player)) {
      new imname[32]
      get_user_name(player, imname, charsmax(imname))
#if defined _translator_included
      console_print(id, _T("That action can't be performed on dead player ^"%s^"."), imname)
#else
      console_print(id, "That action can't be performed on dead player ^"%s^".", imname)
#endif
      return 0
    }
  }
  if(flags & 8) {
    if(is_user_bot(player)) {
      new imname[32]
      get_user_name(player, imname, charsmax(imname))
#if defined _translator_included
      console_print(id, _T("That action can't be performed on bot ^"%s^"."), imname)
#else
      console_print(id, "That action can't be performed on bot ^"%s^".", imname)
#endif
      return 0
    }
  }
  return player
}

/* Checks if the user has any admin flags (returns true or false). */
stock is_user_admin(id) {
  if(id == 0) return true
  new flags = get_user_flags(id)
  return (flags > 0 && (flags & ADMIN_USER) == 0)
}

/* Checks if the user is a real admin (returns true or false).
 * Uses the cvar "amx_realadmin_flags" to know the flags that represent a real admin.
 * Set menu parameter to 1 only if you wish to use this function into AMX menus. */
stock is_user_realadmin(id, menu = 0) {
  if(id == 0) return true
  static cvar_realadmin_flags
  if(!cvar_realadmin_flags) {
    cvar_realadmin_flags = get_cvar_pointer("amx_realadmin_flags")
    if(!cvar_realadmin_flags) {
      cvar_realadmin_flags = register_cvar("amx_realadmin_flags", "cdefghijlmnopqrstuvw")
    }
  }
  new realadminflags[27]
  get_cvarptr_string(cvar_realadmin_flags, realadminflags, charsmax(realadminflags))
  new flags = get_user_flags(id)
  return ((flags & ADMIN_USER) == 0 &&
  (flags & (menu ? (read_flags(realadminflags)|ADMIN_IMMUNITY) : read_flags(realadminflags)) != 0))
}

/* Checks if the user is a master admin (returns true or false).
 * Note: master admin = admin with full access (except "b" flag). */
stock is_user_masteradmin(id) {
  if(id == 0) return true
  // Note: read_flags("acdefghijklmnopqrstuvwy") = 25165821
  return ((get_user_flags(id) & 25165821) == 25165821)
}

/* Checks if the user has any of the provided flags (returns true or false). */
stock has_flag(id, const flags[]) {
  if(id == 0) return true
  return (get_user_flags(id) & read_flags(flags))
}

/* Checks if the user has all the provided flags (returns true or false). */
stock has_all_flags(id, const flags[]) {
  if(id == 0) return true
  new allFlags = read_flags(flags)
  return ((get_user_flags(id) & allFlags) == allFlags)
}

/* You can replace is_user_realadmin with is_user_admin if you prefer
 * for both the show_activity and show_activity_color functions (below). */
#define ___is_user_admin is_user_realadmin

/* Standard admin activity (normal chat print). */
stock show_activity(id, const name[], {Float,_}: ... ) {
  static cvar_amx_show_activity
  if(!cvar_amx_show_activity) {
    cvar_amx_show_activity = get_cvar_pointer("amx_show_activity")
    if(!cvar_amx_show_activity) {
      cvar_amx_show_activity = register_cvar("amx_show_activity", "2")
    }
  }

  new show_activity_value = get_cvarptr_num(cvar_amx_show_activity)
  if(show_activity_value <= 0 || show_activity_value > 5)
    return false

  new players[32], playersnum, player
  get_players(players, playersnum, "c")
  if(playersnum == 0)
    return false

  new buffer[128]

#if !defined _translator_included
  format_args(buffer, charsmax(buffer), 2)
#endif
  if(id == 0) {
    switch(show_activity_value) {
      case 5, 4: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print(player, print_chat, "%s: %s", _T("SERVER"), buffer)
#else
            client_print(player, print_chat, "SERVER: %s", buffer)
#endif
          }
        }
      }
      case 3, 2, 1: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print(player, print_chat, "%s: %s", _T("SERVER"), buffer)
#else
          client_print(player, print_chat, "SERVER: %s", buffer)
#endif
        }
      }
    }
  }
  else {
    new usertype[7]
    copy(usertype, charsmax(usertype), ___is_user_admin(id) ? "ADMIN" : "PLAYER")
    switch(show_activity_value) {
      case 5: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print(player, print_chat, "%s %s: %s", _T(usertype), name, buffer)
#else
            client_print(player, print_chat, "%s %s: %s", usertype, name, buffer)
#endif
          }
        }
      }
      case 4: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print(player, print_chat, "%s: %s", _T(usertype),  buffer)
#else
            client_print(player, print_chat, "%s: %s", usertype, buffer)
#endif
          }
        }
      }
      case 3: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print(player, print_chat, "%s %s: %s", _T(usertype), name, buffer)
#else
            client_print(player, print_chat, "%s %s: %s", usertype, name, buffer)
#endif
          }
          else {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print(player, print_chat, "%s: %s", _T(usertype), buffer)
#else
            client_print(player, print_chat, "%s: %s", usertype, buffer)
#endif
          }
        }
      }
      case 2: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print(player, print_chat, "%s %s: %s", _T(usertype), name, buffer)
#else
          client_print(player, print_chat, "%s %s: %s", usertype, name, buffer)
#endif
        }
      }
      case 1: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print(player, print_chat, "%s: %s", _T(usertype), buffer)
#else
          client_print(player, print_chat, "%s: %s", usertype, buffer)
#endif
        }
      }
    }
  }
  return true
}

/* Colored admin activity ("SERVER", "ADMIN" or "PLAYER" in green, id's name in team color, message in normal color).
 * Note: This function only works with Counter-Strike 1.6 and Condition Zero. */
stock show_activity_color(id, const name[], {Float,_}: ... ) {
  static cvar_amx_show_activity
  if(!cvar_amx_show_activity) {
    cvar_amx_show_activity = get_cvar_pointer("amx_show_activity")
    if(!cvar_amx_show_activity) {
      cvar_amx_show_activity = register_cvar("amx_show_activity", "2")
    }
  }

  new show_activity_value = get_cvarptr_num(cvar_amx_show_activity)
  if(show_activity_value <= 0 || show_activity_value > 5)
    return false

  new players[32], playersnum, player
  get_players(players, playersnum, "c")
  if(playersnum == 0)
    return false

  new buffer[128]

#if !defined _translator_included
  format_args(buffer, charsmax(buffer), 2)
#endif
  if(id == 0) {
    switch(show_activity_value) {
      case 5, 4: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print_color(0, player, "^4%s^1: %s", _T("SERVER"), buffer)
#else
            client_print_color(0, player, "^4SERVER^1: %s", buffer)
#endif
          }
        }
      }
      case 3, 2, 1: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print_color(0, player, "^4%s^1: %s", _T("SERVER"), buffer)
#else
          client_print_color(0, player, "^4SERVER^1: %s", buffer)
#endif
        }
      }
    }
  }
  else {
    new usertype[7]
    copy(usertype, charsmax(usertype), ___is_user_admin(id) ? "ADMIN" : "PLAYER")
    switch(show_activity_value) {
      case 5: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print_color(id, player, "^4%s ^3%s^1: %s", _T(usertype), name, buffer)
#else
            client_print_color(id, player, "^4%s ^3%s^1: %s", usertype, name, buffer)
#endif
          }
        }
      }
      case 4: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print_color(id, player, "^4%s^1: %s", _T(usertype),  buffer)
#else
            client_print_color(id, player, "^4%s^1: %s", usertype, buffer)
#endif
          }
        }
      }
      case 3: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
          if(___is_user_admin(player)) {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print_color(id, player, "^4%s ^3%s^1: %s", _T(usertype), name, buffer)
#else
            client_print_color(id, player, "^4%s ^3%s^1: %s", usertype, name, buffer)
#endif
          }
          else {
#if defined _translator_included
            format_args(buffer, charsmax(buffer), 2, player)
            client_print_color(id, player, "^4%s^1: %s", _T(usertype), buffer)
#else
            client_print_color(id, player, "^4%s^1: %s", usertype, buffer)
#endif
          }
        }
      }
      case 2: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print_color(id, player, "^4%s ^3%s^1: %s", _T(usertype), name, buffer)
#else
          client_print_color(id, player, "^4%s ^3%s^1: %s", usertype, name, buffer)
#endif
        }
      }
      case 1: {
        for(--playersnum; playersnum >= 0; playersnum--) {
          player = players[playersnum]
#if defined _translator_included
          format_args(buffer, charsmax(buffer), 2, player)
          client_print_color(id, player, "^4%s^1: %s", _T(usertype), buffer)
#else
          client_print_color(id, player, "^4%s^1: %s", usertype, buffer)
#endif
        }
      }
    }
  }
  return true
}

/* Sends colored message to player. Set destid to 0 to send text globally.
 * Translation can be used, but do not specify the destid in the _T() function.
 * If you want to display a colored message to all players with the own
 * team color of each player, just set id to -1 and destid to 0.
 * Note: This function only works with Counter-Strike 1.6 and Condition Zero.
 *
 * Color values (add one of them before the part of message):
 *  !def! is the default chat color value (or use: ^1)
 *  !green! is the green color value (or use: ^4)
 *  !team! is the id's team color value (may be red, blue, grey, or normal if the id is the server) */
stock client_print_color(id, destid, {Float,_}:...) {
  static msg[192]
  static msgSayText
  msg[0] = 0x01
  if(id < 0 && destid != 0) id = 0
  new bool:ownDestidTeam = (id < 0) ? true : false
#if !defined _translator_included
  format_args(msg[1], sizeof(msg) - 2, 2)
  while(replace(msg[1], sizeof(msg) - 2, "!def!", "^1")) {}
  while(replace(msg[1], sizeof(msg) - 2, "!green!", "^4")) {}
  while(replace(msg[1], sizeof(msg) - 2, "!team!", (id == 0) ? "^1" : "^3")) {}
#endif
  if(!msgSayText) {
    msgSayText = get_user_msgid("SayText")
  }
  if(destid == 0) {
    new players[32], num, player
    new sender = (ownDestidTeam == false) ? ((id == 0) ? 1 : id) : 1
    get_players(players, num, "c")
    for(new i = 0; i < num; ++i) {
      player = players[i]
#if defined _translator_included
      format_args(msg[1], sizeof(msg) - 2, 2, player)
      while(replace(msg[1], sizeof(msg) - 2, "!def!", "^1")) {}
      while(replace(msg[1], sizeof(msg) - 2, "!green!", "^4")) {}
      while(replace(msg[1], sizeof(msg) - 2, "!team!", (id == 0) ? "^1" : "^3")) {}
#endif
      message_begin(MSG_ONE_UNRELIABLE, msgSayText, _, player)
      write_byte(ownDestidTeam ? player : sender)
      write_string(msg)
      message_end()
    }
  }
  else {
#if defined _translator_included
    format_args(msg[1], sizeof(msg) - 2, 2, destid)
    while(replace(msg[1], sizeof(msg) - 2, "!def!", "^1")) {}
    while(replace(msg[1], sizeof(msg) - 2, "!green!", "^4")) {}
    while(replace(msg[1], sizeof(msg) - 2, "!team!", (id == 0) ? "^1" : "^3")) {}
#endif
    message_begin(MSG_ONE_UNRELIABLE, msgSayText, _, destid)
    write_byte((id == 0) ? 1 : id)
    write_string(msg)
    message_end()
  }
}

/* Checks if a player is valid or not, by id (returns true or false).
 * If maxplayers var is set to 1 the function will check the real
 * maxplayers value on the server by the get_maxplayers() function. */
stock is_playerid_valid(id, maxplayers = 0) {
  if(maxplayers == 1) {
    static maxpl
    if(!maxpl) {
      maxpl = get_maxplayers()
    }
    return (1 <= id <= maxpl)
  }
  return (1 <= id <= 32)
}

/* Returns the value of the amx_version cvar.
 * If the cvar doesn't exist, function returns 0. */
stock get_amx_version(version[], len) {
  static cvar_amx_version = -1
  if(cvar_amx_version == -1) {
    cvar_amx_version = get_cvar_pointer("amx_version")
  }
  if(cvar_amx_version == 0) {
    return 0
  }
  get_cvarptr_string(cvar_amx_version, version, len)
  if(!version[0]) {
    copy(version, len, "undefined")
  }
  return version[0]
}

/* Checks the game mod running to know if AMX can add colors to menus (returns true or false). */
stock colored_menus() {
  new mod_name[12]
  get_modname(mod_name, charsmax(mod_name))
  return (equal(mod_name, "cstrike") || equal(mod_name, "czero") || equal(mod_name, "dod"))
}

/* Checks is the provided game mod is running or not (returns true or false). */
stock is_running(const arg[]) {
  new mod_name[12]
  get_modname(mod_name, charsmax(mod_name))
  return equal(mod_name, arg)
}

/* Checks the current map (returns true or false). */
stock is_map_running(const mapname[]) {
  new currentmap[32]
  get_mapname(currentmap, charsmax(currentmap))
  return equali(currentmap, mapname)
}

/* Builds a path from a local info.
 * Note: Use this function instead of format/formatex natives to declare a file location. */
stock build_path(path[], len, {Float,_}:... ) {
  format_args(path, len, 2)
  new pathlen = strlen(path)
  new basedir[32]
  if(containi(path, "$basedir") != -1) {
    get_localinfo("amx_basedir", basedir, charsmax(basedir))
    if(!basedir[0]) copy(basedir, charsmax(basedir), "addons/amx")
    if((pathlen+strlen(basedir)-strlen("$basedir")) < len) {
      replace(path, len, "$basedir", basedir)
    }
  }
  new dir[64]
  if(containi(path, "$configdir") != -1) {
    get_localinfo("amx_configdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/config", basedir)
    if((pathlen+strlen(basedir)-strlen("$configdir")) < len) {
      replace(path, len, "$configdir", dir)
    }
  }
  if(containi(path, "$mapsconfigdir") != -1) {
    get_localinfo("amx_mapsconfigdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/config/maps", basedir)
    if((pathlen+strlen(basedir)-strlen("$mapsconfigdir")) < len) {
      replace(path, len, "$mapsconfigdir", dir)
    }
  }
  if(containi(path, "$langdir") != -1) {
    get_localinfo("amx_langdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/lang", basedir)
    if((pathlen+strlen(basedir)-strlen("$langdir")) < len) {
      replace(path, len, "$langdir", dir)
    }
  }
  if(containi(path, "$modulesdir") != -1) {
    get_localinfo("amx_modulesdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/modules", basedir)
    if((pathlen+strlen(basedir)-strlen("$modulesdir")) < len) {
      replace(path, len, "$modulesdir", dir)
    }
  }
  if(containi(path, "$pluginsdir") != -1) {
    get_localinfo("amx_pluginsdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/plugins", basedir)
    if((pathlen+strlen(basedir)-strlen("$pluginsdir")) < len) {
      replace(path, len, "$pluginsdir", dir)
    }
  }
  if(containi(path, "$mapspluginsdir") != -1) {
    get_localinfo("amx_mapspluginsdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/config/maps_plugins", basedir)
    if((pathlen+strlen(basedir)-strlen("$mapspluginsdir")) < len) {
      replace(path, len, "$mapspluginsdir", dir)
    }
  }
  if(containi(path, "$logdir") != -1) {
    get_localinfo("amx_logdir", dir, charsmax(dir))
    if(!dir[0]) formatex(dir, charsmax(dir), "%s/logs", basedir)
    if((pathlen+strlen(basedir)-strlen("$logdir")) < len) {
      replace(path, len, "$logdir", dir)
    }
  }
  return 1
}

/* Returns AMX base directory path. */
stock get_basedir(name[], len)
  return get_localinfo("amx_basedir", name, len)

/* Gets the current logfile name by Year, Month and Day.
 * Logs are registered in "adminYearMonthDay.log" format. */
stock get_logfile(name[], len)
  return get_time("admin%Y%m%d.log", name, len)

/* Gets the server language name. */
stock get_server_language(const lang[], size)
  get_user_language(0, lang, size)

/* Sets the server language by language name. */
stock set_server_language(const lang[])
  set_user_language(0, lang)

/* Gets the server language id. */
stock get_server_languagenum()
  return get_user_languagenum(0)

/* Sets the server language by language id. */
stock set_server_languagenum(lang)
  set_user_languagenum(0, lang)
