/*******************************************************************************

  amxmodx_to_amx.inc (AMXX -> AMX library)

  Version: 1.3.2

  Authors: KRoT@L (v1.0.0 -> v1.2.0)
           AMX Mod Team (v1.3.0 RC1 -> v1.3.2)

  You need AMX Mod 2010.1 or higher.

  Credits for the original AMX Mod X natives/stocks/defines go to AMX Mod X Dev Team.
  The authors of AMX Mod X are:
    David "BAILOPAN" Anderson, Pavol "PM OnoTo" Marko,
    Felix "SniperBeamer" Geyer, Jonny "Got His Gun" Bergstrom,
    Lukasz "SidLuke" Wlasinski, Christian "Basic-Master" Hammacher,
    Borja "faluco" Ferrer, Scott "Damaged Soul" Ehlert.

*******************************************************************************/

/*******************************************************************************
* INSTRUCTIONS
********************************************************************************

  Put this file inside your amx/examples/include folder (if it's not already inside).

--------------------------------------------------------------------------------

  Replace all the #include <...> lines with just this line:
    #include <amxmodx_to_amx>

  For example, replace this:

    #include <amxmodx>
    #include <amxmisc>
    #include <engine>
    #include <fun>
    #include <fakemeta>
    ...

  with just:

    #include <amxmodx_to_amx>

--------------------------------------------------------------------------------

  If the plugin uses csstats, add:
    #include <csstats>
  BEFORE #include <amxmodx_to_amx>

  If the plugin uses dodxmod, add:
    #include <dodxmod>
  BEFORE #include <amxmodx_to_amx>

--------------------------------------------------------------------------------

  Also, if after your compilation you get some errors with unknown pev/set_pev functions,
  you have to replace yourself these functions.

  How to do so? Read these instructions below:
  These pev_* or set_pev_* functions can be replaced by entity_get_* or entity_set_* VexdUM functions.
  So you need to replace them with the help of the VexdUM_const.inc file (if you are a newbie).

  ---- Examples ----
  First example:
    pev(id, pev_classname, classname, 31)
  1. Copy the "_classname" part (skip the "pev" part) and looking for it into the include file.
  2. You should find EV_SZ_classname enum, if yes, see the top of this enum list, and look that:
    Entvars String
    To use with entity_get_string/entity_set_string

  3. Now you can replace the pev function with that one:
    entity_get_string(id, EV_SZ_classname, classname, 31)

  Second example:
    set_pev(id, pev_rendermode, kRenderTransAlpha)
  1. Copy the "_rendermode" part (skip the "pev" part) and looking for it into the include file.
  2. You should find EV_INT_rendermode enum, if yes, see the top of this enum list, and look that:
    Entvars Integer
    To use with entity_get_int/entity_set_int

  3. Now you can replace the set_pev function with that one:
    entity_set_int(id, EV_INT_rendermode, kRenderTransAlpha)
  ---- End of the examples ----

  Made the same things for others pev_* or set_pev_* functions, then read the VexdUM.inc file
  to see a list of the available entity_get_* and entity_set_* natives.

  But careful! If the pev function (pev_* iVar) returns to a floating point function of VexdUM,
  you should change the part of code and do something like this (after "Should be modified to"):
    new Float:fov
    pev(index, pev_fov, fov);
  Should be modified to:
    new Float:fov = entity_get_float(index, EV_FL_fov)

  Just because the entity_get_float native doesn't allow the possibility to have a third parameter.
  Otherwise, if you don't do this, you would get a warning message and the function won't work.

  Also, when you are using this conversion include file, sometimes you could get an error such as:
    myplugin.sma(line) : error 021: symbol already defined: "symbol name"

  If you get this error (the most probable), that means the "symbol name" function is 
  already defined into this file. To fix it looking for this function into the 
  plugin filename (.sma file) and remove it then try to recompile the plugin.

*******************************************************************************/

/*******************************************************************************
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*******************************************************************************/

#if defined _amxmodx_to_amx_included
  #endinput
#endif
#define _amxmodx_to_amx_included

/*******************************************************************************
* INCLUDES
*******************************************************************************/

#include <amxmod>
#include <amxmisc>
#include <cstrike>
#include <fun>
#include <geoip>
#include <sockets>
#include <Vexd_Utilities>
#include <xs>

/*******************************************************************************
* GLOBAL DEFINES/CONSTANTS
*******************************************************************************/

/* AMXCONST.INC */
#define LANG_SERVER  0
#define LANG_PLAYER -1

#define INVALID_PLUGIN_ID 0

/* AMXMISC.INC */
#define CMDTARGET_OBEY_IMMUNITY 1
#define CMDTARGET_ALLOW_SELF    2
#define CMDTARGET_ONLY_ALIVE    4
#define CMDTARGET_NO_BOTS       8

/* DODCONST.INC */
#define STAMINA_SET   0
#define STAMINA_RESET 1

#define FUSE_SET   0
#define FUSE_RESET 1

#define PS_NOPRONE     0
#define PS_PRONE       1
#define PS_PRONEDEPLOY 2
#define PS_DEPLOY      3

#define MI_ALLIES_TEAM  0
#define MI_ALLIES_PARAS 1
#define MI_AXIS_PARAS   2

/* FAKEMETA_CONST.INC */
#define FMRES_IGNORED   PLUGIN_CONTINUE
#define FMRES_HANDLED   PLUGIN_CONTINUE
#define FMRES_OVERRIDE  PLUGIN_CONTINUE
#define FMRES_SUPERCEDE PLUGIN_HANDLED

#define FM_NULLENT -1

/* FAKEMETA_UTIL.INC */
#define OFFSET_LONGJUMP 356

/* HAM_CONST.INC */
#define HAM_IGNORED   PLUGIN_CONTINUE
#define HAM_HANDLED   PLUGIN_CONTINUE
#define HAM_OVERRIDE  PLUGIN_CONTINUE
#define HAM_SUPERCEDE PLUGIN_HANDLED

/* SVN_VERSION.INC */
#define AMXX_VERSION     2010
#define AMXX_VERSION_NUM 2010
#define AMXX_VERSION_STR AMX_VERSION

/*******************************************************************************
* AMXMISC.INC
*******************************************************************************/

stock cstrike_running() {
  new mod_name[10];
  get_modname(mod_name, charsmax(mod_name));
  return (equal(mod_name, "cstrike") || equal(mod_name, "czero"));
}

stock get_configsdir(name[], len) {
  return build_path(name, len, "$configdir");
}

stock get_datadir(name[], len) {
  return build_path(name, len, "$modulesdir");
}

stock get_customdir(name[], len) {
  return build_path(name, len, "$configdir");
}

stock register_menu(title[], keys, function[], outside = 0) {
  register_menucmd(register_menuid(title, outside), keys, function);
}

stock AddMenuItem(const MENU_TEXT[], const MENU_CMD[], const MENU_ACCESS, const MENU_PLUGIN[]) {
  new szAccess[16];
  get_flags(MENU_ACCESS, szAccess, charsmax(szAccess));
  server_cmd("amx_addmenuitem ^"amxmodmenu^" ^"%s^" ^"%s^" ^"%s^" ^"b^" ^"%s^"",MENU_PLUGIN,MENU_TEXT,MENU_CMD,szAccess);
}

stock constraint_offset(low, high, seed, offset) {
  new numElements = high - low + 1;
  offset += seed - low;
  if(offset >= 0)
    return low + (offset % numElements);
  else
    return high - (abs(offset) % numElements) + 1;
  return 0;
}

/*******************************************************************************
* AMXMODX.INC
*******************************************************************************/

stock user_has_weapon(index, weapon, setweapon = -1) {
  if(setweapon == 0 || setweapon == 1) {
    return set_user_weapon(index, weapon, setweapon);
  }
  return has_user_weapon(index, weapon);
}

stock change_task(id = 0, Float:newTime = 1.0, outside = 0) {
  return change_tasktime(id, newTime, outside);
}

stock is_module_loaded(const name[]) {
  return is_module_running(name);
}

stock is_plugin_loaded(const name[]) {
  return is_plugin_running(name);
}

stock require_module(const module[]) {
  #pragma unused module
  return 0;
}

stock is_amd64_server() {
  return 0;
}

stock find_plugin_byfile(const filename[], ...) {
  return is_plugin_running(filename);
}

stock log_error(error, const fmt[], ...) {
  new str[512];
  format_args(str, charsmax(str), 1);
  return report_error(str);
}

stock abort(error, const fmt[] = "", {Float,_}:...) {
  new str[512];
  format_args(str, charsmax(str), 1);
  return report_error(str);
}

stock module_exists(const logtag[]) {
  return is_module_running(logtag);
}

stock CreateHudSyncObj(...) {
  return 0;
}

stock ShowSyncHudMsg(target, syncObj, const fmt[], ...) {
  #pragma unused syncObj, fmt
  new str[192];
  format_args(str, charsmax(str), 2);
  return show_hudmessage(target, str);
}

stock ClearSyncHud(target, ...) {
  set_hudmessage(0, 0, 0, 0.0, 0.0, 0, 0.0, 0.01, 0.0, 0.0, 4);
  return show_hudmessage(target, "");
}

stock get_pcvar_flags(pcvar) {
  return get_cvarptr_flags(pcvar);
}

stock set_pcvar_flags(pcvar, flags) {
  return set_cvarptr_flags(pcvar, flags);
}

stock get_pcvar_num(pcvar) {
  return get_cvarptr_num(pcvar);
}

stock set_pcvar_num(pcvar, num) {
  return set_cvarptr_num(pcvar, num);
}

stock Float:get_pcvar_float(pcvar) {
  return get_cvarptr_float(pcvar);
}

stock set_pcvar_float(pcvar, Float:num) {
  return set_cvarptr_float(pcvar, num);
}

stock get_pcvar_string(pcvar, string[], maxlen) {
  return get_cvarptr_string(pcvar, string, maxlen);
}

stock set_pcvar_string(pcvar, const string[]) {
  return set_cvarptr_string(pcvar, string);
}

/*******************************************************************************
* CSX.INC
*******************************************************************************/

#define client_damage(%1,%2,%3,%4,%5,%6) damage_info(%1,%2,%3,%4,%5,%6)
#define client_death(%1,%2,%3,%4,%5) death_info(%1,%2,%3,%4,%5)
#define bomb_explode(%1,%2) bomb_exploded(%1,%2)
#define bomb_defusing(defuser) bomb_defusing(defuser,hasDefuseKit)
#define bomb_defused(defuser) bomb_defused(defuser,hasDefuseKit)

/*******************************************************************************
* DODFUN.INC
*******************************************************************************/

stock dod_set_stamina(index, set = 0, minvalue = 0, maxvalue = 100) {
  if(set == 0) {
    return set_user_stamina(index, minvalue, maxvalue);
  }
  return reset_user_stamina(index);
}

stock dod_set_fuse(index, set = 0, Float:newFuse = 5.0, Type = (1<<0)) {
  if(set == 0) {
    return nade_set_fuse(index, newFuse, Type);
  }
  return nade_reset_fuse(index);
}

/*******************************************************************************
* DODX.INC
*******************************************************************************/

#define client_score(%1,%2,%3) score_info(%1,%2,%3)

enum {
  XMF_DAMAGE = 0,
  XMF_DEATH,
  XMF_SCORE,
};

stock register_statsfwd(ftype) {
  return ftype;
}

stock dod_get_map_info(info) {
  switch(info) {
    case 0: return detect_allies_team();
    case 1: return detect_allies_paras();
    case 2: return detect_axis_paras();
  }
  return 0;
}

stock xmod_is_melee_wpn(wpnindex) {
  return dod_is_melee(wpnindex);
}

stock xmod_get_wpnname(wpnindex, name[], len) {
  return dod_get_wpnname(wpnindex, name, len);
}

stock xmod_get_wpnlogname(wpnindex, name[], len) {
  return dod_get_wpnlogname(wpnindex, name, len);
}

/*******************************************************************************
* ENGINE.INC
*******************************************************************************/

stock force_use(pPlayer, pEntity) {
  return entity_use(pEntity, pPlayer);
}

stock Float:get_global_float(variable) {
  return global_get_float(variable);
}

stock get_global_int(variable) {
  return global_get_int(variable);
}

stock get_global_string(variable, string[], maxlen) {
  return global_get_string(variable, string, maxlen);
}

stock get_global_vector(variable, Float:vector[3]) {
  return global_get_vector(variable, vector);
}

stock get_global_edict(variable) {
  return global_get_edict(variable);
}

stock Float:entity_range(ida, idb) {
  new Float:orig1[3], Float:orig2[3];
  entity_get_vector(ida, EV_VEC_origin, orig1);
  entity_get_vector(idb, EV_VEC_origin, orig2);
  return vector_distance(orig1, orig2);
}

stock find_ent_by_class(iIndex, const szClass[]) {
  if(iIndex == 0) iIndex = -1;
  new ent = find_entity(iIndex, szClass);
  return (ent > 0) ? ent : 0;
}

stock find_ent_by_owner(iIndex, const szClass[], iOwner, iJghgType = 0) {
  if(iIndex == 0) iIndex = -1;
  switch(iJghgType) {
    case 1: {
      while((iIndex = find_entity(iIndex, szClass, 3)) > 0) { // target
        if(entity_get_edict(iIndex, EV_ENT_owner) == iOwner) {
          return iIndex;
        }
      }
    }
    case 2: {
      while((iIndex = find_entity(iIndex, szClass, 4)) > 0) { // targetname
        if(entity_get_edict(iIndex, EV_ENT_owner) == iOwner) {
          return iIndex;
        }
      }
    }
    default: {
      while((iIndex = find_entity(iIndex, szClass)) > 0) { // classname
        if(entity_get_edict(iIndex, EV_ENT_owner) == iOwner) {
          return iIndex;
        }
      }
    }
  }
  return 0;
}

stock find_ent_by_target(iIndex, const szTarget[]) {
  if(iIndex == 0) iIndex = -1;
  new ent = find_entity(iIndex, szTarget, 3);
  return (ent > 0) ? ent : 0;
}

stock find_ent_by_tname(iIndex, const szTargetName[]) {
  if(iIndex == 0) iIndex = -1;
  new ent = find_entity(iIndex, szTargetName, 4);
  return (ent > 0) ? ent : 0;
}

stock find_ent_by_model(iIndex, const szClass[], const szModel[]) {
  if(iIndex == 0) iIndex = -1;
  new model[64];
  while((iIndex = find_entity(iIndex, szClass)) > 0) {
    entity_get_string(iIndex, EV_SZ_model, model, charsmax(model));
    if(equal(szModel, model)) {
      return iIndex;
    }
  }
  return 0;
}

stock find_ent_in_sphere(start_from_ent, const Float:origin[3], Float:radius) {
  if(start_from_ent == 0) start_from_ent = -1;
  new ent = find_entity_sphere(start_from_ent, origin, radius);
  return (ent > 0) ? ent : 0;
}

stock is_valid_ent(iIndex) {
  return is_entity(iIndex);
}

stock entity_count() {
  return get_num_ents();
}

stock point_contents(const Float:fCheckAt[3]) {
  return PointContents(fCheckAt);
}

stock Float:halflife_time() {
  return get_gametime();
}

stock find_sphere_class(aroundent, const _lookforclassname[], Float:radius, entlist[], maxents, Float:origin[3] = {0.0, 0.0, 0.0}) {
  if(is_entity(aroundent)) {
    entity_get_vector(aroundent, EV_VEC_origin, origin);
  }
  new entnum = 0;
  new classname[64];
  new ent = find_entity_sphere(-1, origin, radius);
  while(ent > 0 && entnum < maxents) {
    entity_get_string(ent, EV_SZ_classname, classname, 63);
    if(equal(classname, _lookforclassname)) {
      entlist[entnum++] = ent;
    }
    ent = find_entity_sphere(ent, origin, radius);
  }
  return entnum;
}

stock is_in_viewcone(entity, const Float:origin[3]) {
  return in_view_cone(entity, origin);
}

stock is_visible(entity, target) {
  return can_see(entity, target);
}

#define client_PreThink(%1) client_prethink(%1)
#define client_PostThink(%1) client_postthink(%1)
#define pfn_touch(%1,%2) entity_touch(%1,%2)
#define pfn_think(%1) entity_think(%1)
#define pfn_keyvalue(%1) keyvalue(%1)
#define pfn_spawn(%1) entity_spawn(%1)

/*******************************************************************************
* ENGINE_STOCKS.INC
*******************************************************************************/

stock fakedamage(idvictim, const szClassname[], Float:takedmgdamage, damagetype) {
  new entity = create_entity("trigger_hurt");
  if(entity > 0) {
    DispatchKeyValue(entity, "classname", "trigger_hurt");
    new szDamage[16];
    format(szDamage, charsmax(szDamage), "%f", takedmgdamage * 2);
    DispatchKeyValue(entity, "dmg", szDamage);
    format(szDamage, charsmax(szDamage), "%i", damagetype);
    DispatchKeyValue(entity, "damagetype", szDamage);
    DispatchKeyValue(entity, "origin", "8192 8192 8192");
    DispatchSpawn(entity);
    entity_set_string(entity, EV_SZ_classname, szClassname);
    fake_touch(entity, idvictim);
    remove_entity(entity);
    return 1;
  }
  return 0;
}

stock find_ent(iStart, const szClassname[]) {
  if(iStart == 0) iStart = -1;
  new ent = find_entity(iStart, szClassname);
  return (ent > 0) ? ent : 0;
}

stock remove_entity_name(const eName[]) {
  return remove_entities(eName);
}

stock set_entity_visibility(entity, visible = 1) {
  if(visible == 1) {
    entity_set_int(entity, EV_INT_effects, entity_get_int(entity, EV_INT_effects) & ~EF_NODRAW);
  }
  else {
    entity_set_int(entity, EV_INT_effects, entity_get_int(entity, EV_INT_effects) | EF_NODRAW);
  }
  return 1;
}

stock get_entity_visibility(entity) {
  return (!(entity_get_int(entity, EV_INT_effects) & EF_NODRAW));
}

stock set_user_velocity(entity, const Float:vec[3]) {
  return entity_set_vector(entity, EV_VEC_velocity, vec);
}

stock get_user_velocity(entity, Float:vec[3]) {
  return entity_get_vector(entity, EV_VEC_velocity, vec);
}

stock set_size(index, const Float:mins[3], const Float:maxs[3]) {
  return entity_set_size(index, mins, maxs);
}

stock IsInWorld(ent) {
   new Float:origin[3];
   entity_get_vector(ent, EV_VEC_origin, origin);
   if(origin[0] >= 4096.0) return 0;
   if(origin[1] >= 4096.0) return 0;
   if(origin[2] >= 4096.0) return 0;
   if(origin[0] <= -4096.0) return 0;
   if(origin[1] <= -4096.0) return 0;
   if(origin[2] <= -4096.0) return 0;
   new Float:velocity[3];
   entity_get_vector(ent, EV_VEC_velocity, velocity);
   if(velocity[0] >= 2000) return 0;
   if(velocity[1] >= 2000) return 0;
   if(velocity[2] >= 2000) return 0;
   if(velocity[0] <= -2000) return 0;
   if(velocity[1] <= -2000) return 0;
   if(velocity[2] <= -2000) return 0;
   return 1;
}

/*******************************************************************************
* ESF_STOCK.INC / ESF_UTIL.INC (other .inc files)
*******************************************************************************/

#define esf_set_ki(%1,%2) entity_set_float(%1, EV_FL_fuser4, float(%2)) 
#define esf_get_powerlevel(%1) get_offset_int(%1, 460)
#define esf_set_powerlevel(%1,%2) set_offset_int(%1, 460, %2)
#define esf_get_acpl(%1) get_offset_int(%1, 463)
#define esf_set_acpl(%1,%2) set_offset_int(%1, 463, %2)
#define esf_get_maxhealth(%1) get_offset_int(%1, 142)
#define esf_get_speed(%1) get_offset_int(%1, 462)
#define esf_set_speed(%1,%2) set_offset_int(%1, 462, %2)
#define esf_get_swoopspeed(%1) entity_get_float(%1, EV_FL_fuser1)
#define esf_set_swoopspeed(%1,%2) entity_set_float(%1, EV_FL_fuser1, float(%2)) 
#define esf_get_fly(%1) get_offset_int(%1, 195)
#define esf_get_turbo(%1) get_offset_int(%1, 196)
#define esf_get_powerup(%1) get_offset_int(%1, 199)
#define esf_get_swooping(%1) get_offset_int(%1, 317)
#define esf_get_attack_charge(%1) get_offset_int(%1, 200)
#define esf_get_advancedmelee(%1) get_offset_int(%1, 298)
#define esf_get_block(%1) get_offset_int(%1, 198)
#define esf_get_throw(%1) get_offset_int(%1, 230)
#define esf_get_user_deaths(%1) get_offset_int(%1, 604, 5)

stock esf_get_ki(index) {
  if(!is_user_alive(index)) {
    return 0;
  }
  new Float:value = entity_get_float(index, EV_FL_fuser4);
  return floatround(value);
}

stock esf_set_maxhealth(index, health) {
  set_offset_int(index, 142, health);
  message_begin(MSG_ONE_UNRELIABLE, get_user_msgid("MaxHealth"), {0, 0, 0}, index);
  write_byte(health);
  message_end();
  return 1;
}

stock esf_get_team(index) {
  new szTeamName[2];
  get_user_team(index, szTeamName, charsmax(szTeamName));

  switch(szTeamName[0]) {
    case 'G': return 1;
    case 'E': return 2;
  }
  return 0;
}

enum {
  ESF_CLASS_NONE = 0,
  ESF_CLASS_BUU,
  ESF_CLASS_GOKU,
  ESF_CLASS_GOHAN,
  ESF_CLASS_KRILLIN,
  ESF_CLASS_FRIEZA,
  ESF_CLASS_PICCOLO,
  ESF_CLASS_TRUNKS,
  ESF_CLASS_VEGETA,
  ESF_CLASS_CELL
};

#define esf_get_player_class(%1) entity_get_int(%1, EV_INT_playerclass)

/*******************************************************************************
* FAKEMETA.INC
*******************************************************************************/

stock pev_valid(entindex) {
  return is_entity(entindex);
}

stock get_pdata_int(_index, _Offset, _linuxdiff = 5) {
  return get_offset_int(_index, _Offset, _linuxdiff);
}

stock set_pdata_int(_index, _Offset, _Value, _linuxdiff = 5) {
  return set_offset_int(_index, _Offset, _Value, _linuxdiff);
}

stock Float:get_pdata_float(_index, _Offset, _linuxdiff = 5) {
  return get_offset_float(_index, _Offset, _linuxdiff);
}

stock set_pdata_float(_index, _Offset, Float:_Value, _linuxdiff = 5) {
  return set_offset_float(_index, _Offset, _Value, _linuxdiff);
}

/*******************************************************************************
* FAKEMETA_STOCKS.INC
*******************************************************************************/

stock EF_PrecacheModel(const string[]) {
  return precache_model(string);
}

stock EF_PrecacheSound(const string[]) {
  return precache_sound(string);
}

stock EF_SetModel(id, const string[]) {
  return set_user_model(id, string);
}

stock EF_SetSize(entity, const Float:min[3], const Float:max[3]) {
  return entity_set_size(entity, min, max);
}

stock EF_VecToAngles(Float:vecIn[3], Float:vecOut[3]) {
  return vector_to_angle(vecIn, vecOut);
}

stock EF_FindEntityByString(entity, const szValue[], const typeName[]) {
  if(entity == 0) entity = -1;
  new type;
  if(equali(typeName, "globalname")) type = 1;
  else if(equali(typeName, "model")) type = 2;
  else if(equali(typeName, "target")) type = 3;
  else if(equali(typeName, "targetname")) type = 4;
  else if(equali(typeName, "netname")) type = 5;
  else if(equali(typeName, "message")) type = 6;
  else if(equali(typeName, "noise")) type = 7;
  else if(equali(typeName, "noise1")) type = 8;
  else if(equali(typeName, "noise2")) type = 9;
  else if(equali(typeName, "noise3")) type = 10;
  else if(equali(typeName, "viewmodel")) type = 11;
  else if(equali(typeName, "weaponmodel")) type = 12;
  else type = 0;

  new ent = find_entity(entity, szValue, type);
  return (ent > 0) ? ent : 0;
}

stock EF_FindEntityInSphere(entity, const Float:origin[3], Float:radius) {
  return find_entity_sphere(entity, origin, radius);
}

stock EF_MakeVectors(Float:vVector[3]) {
  return make_vectors(vVector);
}

stock EF_RemoveEntity(entity) {
  return remove_entity(entity);
}

stock EF_CreateNamedEntity(const classname[]) {
  return create_entity(classname);
}

stock EF_DropToFloor(entity) {
  return drop_to_floor(entity);
}

stock EF_EmitSound(entity, channel, sample[], Float:volume, Float:attenuation, fFlags, pitch) {
  return emit_sound(entity, channel, sample, volume, attenuation, fFlags, pitch);
}

stock EF_LightStyle(style, const val[]) {
 return set_lights(val);
}

stock EF_DecalIndex(const szName[]) {
  return get_decal_index(szName);
}

stock EF_PointContents(const Float:fVector[3]) {
  return PointContents(fVector);
}

stock EF_SetClientMaxspeed(entity, Float:newMaxspeed) {
  return entity_set_float(entity, EV_FL_maxspeed, newMaxspeed);
}

stock EF_NumberOfEntities() {
  return get_num_ents();
}

stock EF_PrecacheGeneric(const string[]) {
  return precache_generic(string);
}

stock EF_PrecacheEvent(type, const string[]) {
  return precache_event(type, string);
}

stock EF_PlaybackEvent(flags, invoker, eventindex, Float:delay, const Float:origin[3], const Float:angles[3], Float:fparam1, Float:fparam2, iparam1, iparam2, bparam1, bparam2) {
  return playback_event(flags, invoker, eventindex, delay, origin[3], angles[3], fparam1, fparam2, iparam1, iparam2, bparam1, bparam2);
}

stock EF_GetClientListening(receiver, sender) {
  return get_client_listen(receiver, sender);
}

stock EF_SetClientListening(receiver, sender, bool:listen) {
  return set_client_listen(receiver, sender, (listen == true) ? 1 : 0);
}

stock EF_MessageBegin(msg_dest, msg_type, const Float:origin[3], ed) {
  return message_begin(msg_dest, msg_type, origin, ed);
}

stock EF_WriteCoord(Float:value) {
  return write_coord(floatround(value));
}

stock EF_WriteAngle(Float:value) {
  return write_angle(floatround(value));
}

stock EF_InfoKeyValue(infobuffer, const key[], returnValue[], maxLength) {
  return get_keyvalue(infobuffer, key, returnValue, maxLength);
}

stock EF_ClientPrintf(entity, printType, const message[]) {
  return engclient_print(entity, printType, message);
}

stock EF_ServerPrint(const message[]) {
  return server_print(message);
}

stock DF_GameInit() {
  return dllfunc(DLLFunc_GameInit);
}

stock DF_Spawn(const entity) {
  return dllfunc(DLLFunc_Spawn, entity);
}

stock DF_Think(const entity) {
  return dllfunc(DLLFunc_Think, entity);
}

stock DF_Use(const entUsed, entUser) {
  return dllfunc(DLLFunc_Use, entUsed, entUser);
}

stock DF_Touch(const entTouched, const entToucher) {
  return dllfunc(DLLFunc_Touch, entTouched, entToucher);
}

stock DF_Blocked(const entBlocked, const entOther) {
  return dllfunc(DLLFunc_Blocked, entBlocked, entOther);
}

stock DF_SetAbsBox(const entity) {
  return dllfunc(DLLFunc_SetAbsBox, entity);
}

stock DF_ClientConnect(const entity, const name[], const adress[], RejectReason[128]) {
  return dllfunc(DLLFunc_ClientConnect, entity, name, adress, RejectReason);
}

stock DF_ClientDisconnect(const entity) {
  return dllfunc(DLLFunc_ClientDisconnect, entity);
}

stock DF_ClientKill(const entity) {
  return dllfunc(DLLFunc_ClientKill, entity);
}

stock DF_ClientPutInServer(const entity) {
  return dllfunc(DLLFunc_ClientPutInServer, entity);
}

stock DF_ClientCommand(const entity) {
  return dllfunc(DLLFunc_ClientCommand, entity);
}

stock DF_ServerDeactivate() {
  return dllfunc(DLLFunc_ServerDeactivate);
}

stock DF_PlayerPreThink(const entity) {
  return dllfunc(DLLFunc_PlayerPreThink, entity);
}

stock DF_PlayerPostThink(const entity) {
  return dllfunc(DLLFunc_PlayerPostThink, entity);
}

stock DF_StartFrame() {
  return dllfunc(DLLFunc_StartFrame);
}

stock DF_ParmsNewLevel() {
  return dllfunc(DLLFunc_ParmsNewLevel);
}

stock DF_ParmsChangeLevel() {
  return dllfunc(DLLFunc_ParmsChangeLevel);
}

stock DF_GetGameDescription() {
  return dllfunc(DLLFunc_GetGameDescription);
}

stock DF_SpectatorConnect(const entity) {
  return dllfunc(DLLFunc_SpectatorConnect, entity);
}

stock DF_SpectatorDisconnect(const entity) {
  return dllfunc(DLLFunc_SpectatorDisconnect, entity);
}

stock DF_SpectatorThink(const entity) {
  return dllfunc(DLLFunc_SpectatorThink, entity);
}

stock DF_Sys_Error(const error[]) {
  return dllfunc(DLLFunc_Sys_Error, error);
}

stock DF_PM_FindTextureType(name[]) {
  return dllfunc(DLLFunc_PM_FindTextureType, name);
}

stock DF_RegisterEncoders() {
  return dllfunc(DLLFunc_RegisterEncoders);
}

stock DF_GetHullBounds(hullnumber, Float:mins[3], Float:maxs[3]) {
  return dllfunc(DLLFunc_GetHullBounds, hullnumber, mins, maxs);
}

stock DF_CreateInstBaselines() {
  return dllfunc(DLLFunc_CreateInstBaselines);
}

stock DF_pfnAllowLagCompensation() {
  return dllfunc(DLLFunc_pfnAllowLagCompensation);
}

stock DF_MetaFunc_CallGameEntity(const string[], const entity) {
  return dllfunc(MetaFunc_CallGameEntity, string, entity);
}

stock DF_ClientUserInfoChanged(const idPlayer) {
  return dllfunc(DLLFunc_ClientUserInfoChanged, idPlayer);
}

stock DF_UpdateClientData(const entity, sendweapons, const cd/* = 0*/) {
  return dllfunc(DLLFunc_UpdateClientData, entity, sendweapons, cd);
}

stock DF_AddToFullPack(const state/* = 0*/, e, ent, host, hostflags, player, set) {
  return dllfunc(DLLFunc_AddToFullPack, state, e, ent, host, hostflags, player, set);
}

stock DF_CmdStart(const player, const cmd/* = 0*/, randomSeed) {
  return dllfunc(DLLFunc_CmdStart, player, cmd, randomSeed);
}

stock DF_CmdEnd(const player) {
  return dllfunc(DLLFunc_CmdEnd, player);
}

stock DF_CreateBaseline(player, eIndex, baseline, playerModelIndex, Float:playerMins[3], Float:playerMaxs[3]) {
  return dllfunc(DLLFunc_CreateBaseline, player, eIndex, baseline, playerModelIndex, playerMins, playerMaxs);
}

/*******************************************************************************
* FAKEMETA_UTIL.INC
*******************************************************************************/

/* Engine functions */
#define fm_precache_generic(%1) precache_generic(%1)
#define fm_precache_event(%1,%2) precache_event(%1,%2)
#define fm_drop_to_floor(%1) drop_to_floor(%1)
#define fm_force_use(%1,%2) dllfunc(DLLFunc_Use,%1,%2)
#define fm_entity_set_size(%1,%2,%3) entity_set_size(%1,%2,%3)
#define fm_get_decal_index(%1) get_decal_index(%1)
#define fm_entity_range(%1,%2) entity_range(%1,%2)
#define fm_create_entity(%1) create_entity(%1)
#define fm_find_ent_by_class(%1,%2) find_ent_by_class(%1,%2)

stock fm_find_ent_by_owner(index, const classname[], owner, jghgtype = 0) {
  return find_ent_by_owner(index, classname, owner, jghgtype);
}

#define fm_find_ent_by_target(%1,%2) find_ent_by_target(%1,%2)
#define fm_find_ent_by_tname(%1,%2) find_ent_by_tname(%1,%2)
#define fm_find_ent_by_model(%1,%2,%3) find_ent_by_model(%1,%2,%3)
#define fm_find_ent_in_sphere(%1,%2,%3) find_ent_in_sphere(%1,%2,%3)
#define fm_call_think(%1) dllfunc(DLLFunc_Think,%1)
#define fm_is_valid_ent(%1) is_entity(%1)

stock fm_entity_set_origin(index, const Float:origin[3]) {
  new Float:mins[3], Float:maxs[3];
  entity_get_vector(index, EV_VEC_mins, mins);
  entity_get_vector(index, EV_VEC_maxs, maxs);
  entity_set_size(index, mins, maxs);
  return entity_set_origin(index, origin);
}

#define fm_entity_set_model(%1,%2) entity_set_model(%1,%2)
#define fm_remove_entity(%1) remove_entity(%1)
#define fm_entity_count() get_num_ents()
#define fm_fake_touch(%1,%2) dllfunc(DLLFunc_Touch,%1,%2)
#define fm_DispatchSpawn(%1) dllfunc(DLLFunc_Spawn,%1)
#define fm_point_contents(%1) PointContents(%1)

stock fm_trace_line(ignoreent, const Float:start[3], const Float:end[3], Float:ret[3]) {
  return trace_line(ignoreent, start, end, ret);
}

stock fm_trace_hull(const Float:origin[3], hull, ignoredent = 0, ignoremonsters = 0) {
  return trace_hull(origin, hull, ignoredent, ignoremonsters);
}

stock fm_trace_normal(ignoreent, const Float:start[3], const Float:end[3], Float:ret[3]) {
  return trace_normal(ignoreent, start, end, ret);
}

stock fm_get_grenade_id(id, model[], len, grenadeid = 0) {
  new ent = find_ent_by_owner(grenadeid, "grenade", id);
  if(ent && len > 0) {
    entity_get_string(ent, EV_SZ_model, model, len);
  }
  return ent;
}

#define fm_halflife_time() get_gametime()
#define fm_attach_view(%1,%2) attach_view(%1,%2)

stock fm_playback_event(flags, invoker, eventindex, Float:delay, const Float:origin[3], const Float:angles[3], Float:fparam1, Float:fparam2, iparam1, iparam2, bparam1, bparam2) {
  return playback_event(flags, invoker, eventindex, delay, origin, angles, fparam1, fparam2, iparam1, iparam2, bparam1, bparam2);
}

/* HLSDK functions */
stock bool:fm_is_in_viewcone(index, const Float:point[3]) {
  return in_view_cone(index, point) ? true : false;
}

/* Engine_stocks functions */
#define fm_fakedamage(%1,%2,%3,%4) fakedamage(%1,%2,%3,%4)
#define fm_find_ent(%1,%2) find_entity(%1,%2)
#define fm_get_user_button(%1) get_user_button(%1)
#define fm_get_user_oldbutton(%1) get_user_oldbutton(%1)
#define fm_get_entity_flags(%1) get_entity_flags(%1)
#define fm_get_entity_distance(%1,%2) get_entity_distance(%1,%2)
#define fm_get_grenade(%1) fm_get_grenade_id(%1, "", 0)
#define fm_get_brush_entity_origin(%1,%2) get_brush_entity_origin(%1,%2)
#define fm_remove_entity_name(%1) remove_entities(%1)
#define fm_ViewContents(%1) ViewContents(%1)
#define fm_get_speed(%1) get_speed(%1)

stock fm_set_rendering(entity, fx = kRenderFxNone, r = 255, g = 255, b = 255, render = kRenderNormal, amount = 16) {
  return set_rendering(entity, fx, r, g, b, render, amount);
}

stock fm_set_entity_flags(index, flag, onoff) {
  return set_entity_flags(index, flag, onoff);
}

stock fm_set_entity_visibility(index, visible = 1) {
  return set_entity_visibility(index, visible);
}

stock fm_get_entity_visibility(index) {
  return (!(entity_get_int(index, EV_INT_effects) & EF_NODRAW));
}

stock fm_set_user_velocity(entity, const Float:vector[3]) {
  return entity_set_vector(entity, EV_VEC_velocity, vector);
}

stock fm_get_user_velocity(entity, Float:vector[3]) {
  return entity_get_vector(entity, EV_VEC_velocity, vector);
}

/* Fun functions */
#define fm_get_client_listen(%1,%2) get_client_listen(%1,%2)
#define fm_set_client_listen(%1,%2,%3) set_client_listen(%1,%2,%3)

stock fm_get_user_godmode(index) {
  return (entity_get_float(index, EV_FL_takedamage) == DAMAGE_NO);
}

stock fm_set_user_godmode(index, godmode = 0) {
  return entity_set_float(index, EV_FL_takedamage, (godmode == 1) ? DAMAGE_NO : DAMAGE_AIM);
}

stock fm_set_user_armor(index, armor) {
  return entity_set_float(index, EV_FL_armorvalue, float(armor));
}

stock fm_set_user_health(index, health) {
  return (health > 0) ? entity_set_float(index, EV_FL_health, float(health)) : dllfunc(DLLFunc_ClientKill, index);
}

stock fm_set_user_origin(index, origin[3]) {
  new Float:orig[3];
  IVecFVec(origin, orig);
  return entity_set_origin(index, orig);
}

stock fm_set_user_rendering(index, fx = kRenderFxNone, r = 255, g = 255, b = 255, render = kRenderNormal, amount = 16) {
  return set_user_rendering(index, fx, r, g, b, render, amount);
}

stock fm_give_item(index, const item[]) {
  return give_item(index, item);
}

stock fm_set_user_maxspeed(index, Float:speed = -1.0) {
  return entity_set_float(index, EV_FL_maxspeed, speed);
}

stock Float:fm_get_user_maxspeed(index) {
  return entity_get_float(index, EV_FL_maxspeed);
}

stock fm_set_user_gravity(index, Float:gravity = 1.0) {
  return entity_set_float(index, EV_FL_gravity, gravity);
}

stock Float:fm_get_user_gravity(index) {
  return entity_get_float(index, EV_FL_gravity);
}

stock fm_set_user_noclip(index, noclip = 0) {
  return entity_set_int(index, EV_INT_movetype, (noclip == 1) ? MOVETYPE_NOCLIP : MOVETYPE_WALK);
}

stock fm_get_user_noclip(index) {
  return (entity_get_int(index, EV_INT_movetype) == MOVETYPE_NOCLIP);
}

stock fm_strip_user_weapons(index) {
  return strip_user_weapons(index);
}

stock fm_set_user_frags(index, frags) {
  return entity_set_float(index, EV_FL_frags, float(frags));
}

/* Cstrike functions */
stock fm_cs_user_spawn(index) {
  return user_spawn(index);
}

/* Custom functions */
stock fm_set_kvd(entity, const key[], const value[], const classname[] = "") {
  if(classname[0]) {
    DispatchKeyValue(entity, "classname", classname);
  }
  else {
    new class[32];
    entity_get_string(entity, EV_SZ_classname, class, charsmax(class));
    DispatchKeyValue(entity, "classname", class);
  }
  DispatchKeyValue(entity, key, value);
  return 1;
}

stock Float:fm_distance_to_box(const Float:point[3], const Float:mins[3], const Float:maxs[3]) {
  new Float:dist[3];
  for(new i = 0; i < 3; ++i) {
    if(point[i] > maxs[i]) {
      dist[i] = point[i] - maxs[i];
    }
    else if(mins[i] > point[i]) {
      dist[i] = mins[i] - point[i];
    }
  }
  return vector_length(dist);
}

stock Float:fm_boxes_distance(const Float:mins1[3], const Float:maxs1[3], const Float:mins2[3], const Float:maxs2[3]) {
  new Float:dist[3];
  for(new i = 0; i < 3; ++i) {
    if(mins1[i] > maxs2[i]) {
      dist[i] = mins1[i] - maxs2[i];
    }
    else if(mins2[i] > maxs1[i]) {
      dist[i] = mins2[i] - maxs1[i];
    }
  }
  return vector_length(dist);
}

stock Float:fm_distance_to_boxent(entity, boxent) {
  new Float:point[3];
  entity_get_vector(entity, EV_VEC_origin, point);
  new Float:mins[3], Float:maxs[3];
  entity_get_vector(boxent, EV_VEC_absmin, mins);
  entity_get_vector(boxent, EV_VEC_absmax, maxs);
  return fm_distance_to_box(point, mins, maxs);
}

stock Float:fm_boxents_distance(boxent1, boxent2) {
  new Float:mins1[3], Float:maxs1[3];
  entity_get_vector(boxent1, EV_VEC_absmin, mins1);
  entity_get_vector(boxent1, EV_VEC_absmax, maxs1);
  new Float:mins2[3], Float:maxs2[3];
  entity_get_vector(boxent2, EV_VEC_absmin, mins2);
  entity_get_vector(boxent2, EV_VEC_absmax, maxs2);
  return fm_boxes_distance(mins1, maxs1, mins2, maxs2);
}

stock fm_kill_entity(index) {
  return entity_set_int(index, EV_INT_flags, entity_get_int(index, EV_INT_flags) | FL_KILLME);
}

stock fm_get_user_weapon_entity(id, wid = 0) {
  new weapon = wid, clip, ammo;
  if(!weapon && !(weapon = get_user_weapon(id, clip, ammo))) {
    return 0;
  }
  new class[32];
  get_weaponname(weapon, class, charsmax(class));
  return find_ent_by_owner(-1, class, id);
}

stock bool:fm_strip_user_gun(index, wid = 0, const wname[] = "") {
  new ent_class[32];
  if(!wid && wname[0]) {
    copy(ent_class, charsmax(ent_class), wname);
  }
  else {
    new weapon = wid, clip, ammo;
    if(!weapon && !(weapon = get_user_weapon(index, clip, ammo))) {
      return false;
    }
    get_weaponname(weapon, ent_class, charsmax(ent_class));
  }
  new ent_weap = find_ent_by_owner(-1, ent_class, index);
  if(!ent_weap) {
    return false;
  }
  engclient_cmd(index, "drop", ent_class);
  new ent_box = entity_get_edict(ent_weap, EV_ENT_owner);
  if(!ent_box || ent_box == index) {
    return false;
  }
  call_think(ent_box);
  return true;
}

stock bool:fm_transfer_user_gun(index1, index2, wid = 0, const wname[] = "") {
  new ent_class[32];
  if(!wid && wname[0]) {
    copy(ent_class, charsmax(ent_class), wname);
  }
  else {
    new weapon = wid, clip, ammo;
    if(!weapon && !(weapon = get_user_weapon(index1, clip, ammo))) {
      return false;
    }
    get_weaponname(weapon, ent_class, charsmax(ent_class));
  }
  new ent_weap = find_ent_by_owner(-1, ent_class, index1);
  if(!ent_weap) {
    return false;
  }
  engclient_cmd(index1, "drop", ent_class);
  new ent_box = entity_get_edict(ent_weap, EV_ENT_owner);
  if(!ent_box || ent_box == index1) {
    return false;
  }
  entity_set_int(ent_box, EV_INT_flags, entity_get_int(ent_box, EV_INT_flags) | FL_ONGROUND);
  dllfunc(DLLFunc_Touch, ent_box, index2);
  if(entity_get_edict(ent_weap, EV_ENT_owner) != index2) {
    return false;
  }
  return true;
}

stock bool:fm_is_ent_visible(index, entity, ...) {
  return can_see(index, entity) ? true : false;
}

stock bool:fm_get_user_longjump(index) {
  return (get_offset_int(index, OFFSET_LONGJUMP)) ? true : false;
}

stock fm_set_user_longjump(index, bool:longjump = true, bool:tempicon = true) {
  if(longjump == fm_get_user_longjump(index)) {
    return 0;
  }
  if(longjump) {
    give_item(index, "item_longjump");
    emit_sound(index, CHAN_ITEM, "items/gunpickup2.wav", VOL_NORM, ATTN_NORM, 0, PITCH_NORM);
  }
  else if(is_playerid_valid(index, 1)) {
    report_error("fm_set_user_longjump: Unable to remove ^"item_longjump^" entity on player %d, feature doesn't work.", index);
    return 0;
  }
  return 1;
}

stock bool:fm_get_user_suit(index) {
  return (entity_get_int(index, EV_INT_weapons) & (1<<HLW_SUIT)) ? true : false;
}

stock fm_set_user_suit(index, bool:suit = true, bool:sound = true) {
  new weapons = entity_get_int(index, EV_INT_weapons);
  if(!suit) {
    entity_set_int(index, EV_INT_weapons, weapons & ~(1<<HLW_SUIT));
  }
  else if(!(weapons & (1<<HLW_SUIT))) {
    entity_set_int(index, EV_INT_weapons, weapons | (1<<HLW_SUIT));
    if(sound) {
      emit_sound(index, CHAN_VOICE, "items/tr_kevlar.wav", VOL_NORM, ATTN_NORM, 0, PITCH_NORM);
    }
  }
}

stock fm_cs_remove_decals(index = 0) {
  static eventindex_decal_reset;
  if(!eventindex_decal_reset) {
    eventindex_decal_reset = precache_event(1, "events/decal_reset.sc");
  }
  new flags = FEV_RELIABLE;
  if(!index) {
    flags |= FEV_GLOBAL;
  }
  playback_event(flags, index, eventindex_decal_reset, 0.0, Float:{0.0, 0.0, 0.0}, Float:{0.0, 0.0, 0.0}, 0.0, 0.0, 0, 0, 0, 0);
}

stock bool:fm_is_ent_classname(index, const classname[]) {
  if(!is_entity(index)) {
    return false;
  }
  new class[32];
  entity_get_string(index, EV_SZ_classname, class, charsmax(class));
  if(equali(class, classname)) {
    return true;
  }
  return false;
}

stock fm_user_kill(index, flag = 0) {
  return user_kill(index, flag);
}

stock Float:fm_get_view_angle_diff(index, const Float:point[3]) {
  new Float:vec[3], Float:ofs[3], Float:aim[3];
  entity_get_vector(index, EV_VEC_origin, vec);
  entity_get_vector(index, EV_VEC_view_ofs, ofs);
  xs_vec_add(vec, ofs, vec);
  xs_vec_sub(point, vec, vec);
  xs_vec_normalize(vec, vec);
  entity_get_vector(index, EV_VEC_v_angle, aim);
  make_vectors(aim);
  global_get_vector(GV_VEC_v_forward, aim);
  return xs_vec_angle(vec, aim);
}

stock fm_get_weaponbox_type(entity) {
  static max_clients, max_entities;
  if(!max_clients) {
    max_clients = global_get_int(GV_INT_maxClients);
  }
  if(!max_entities) {
    max_entities = global_get_int(GV_INT_maxEntities);
  }
  for(new i = max_clients + 1; i < max_entities; ++i) {
    if(is_entity(i) && entity == entity_get_edict(i, EV_ENT_owner)) {
      new wname[32];
      entity_get_string(i, EV_SZ_classname, wname, charsmax(wname));
      return get_weaponid(wname);
    }
  }
  return 0;
}

stock fm_remove_weaponbox(entity) {
  return call_think(entity);
}

/*******************************************************************************
* FAKEMETA - BONUS FUNCTIONS
*******************************************************************************/

#define fm_get_user_team(%1) get_offset_int(%1, OFFSET_TEAM)
#define fm_set_user_team(%1,%2) set_offset_int(%1, OFFSET_TEAM, %2)

stock fm_set_user_team_extra(index, team) {
  set_offset_int(index, OFFSET_TEAM, team);
  static gmsgTeamInfo;
  if(!gmsgTeamInfo) {
    gmsgTeamInfo = get_user_msgid("TeamInfo");
  }
  static const TeamInfo[][] = {
    "UNASSIGNED",
    "TERRORIST",
    "CT",
    "SPECTATOR"
  };
  message_begin(MSG_ALL, gmsgTeamInfo);
  write_byte(index);
  write_string(TeamInfo[team]);
  message_end();
  return 1;
}

#define fm_get_user_deaths(%1) get_offset_int(%1, OFFSET_CSDEATHS)
#define fm_set_user_deaths(%1,%2) set_offset_int(%1, OFFSET_CSDEATHS, %2)

#define fm_get_user_money(%1) get_offset_int(%1, OFFSET_CSMONEY)

stock fm_set_user_money(index, amount, flash = 1) {
  static gmsgMoney;
  if(!gmsgMoney) {
    gmsgMoney = get_user_msgid("Money");
  }
  set_offset_int(index, OFFSET_CSMONEY, amount);
  message_begin(MSG_ONE_UNRELIABLE, gmsgMoney, _, index);
  write_long(amount);
  write_byte(flash ? 1 : 0);
  message_end();
  return 1;
}

stock fm_set_user_nvg(index, nvgoggles = 1) {
  new current = get_offset_int(index, OFFSET_NVGOGGLES);
  if(!(current & HAS_NVGOGGLES) && nvgoggles) {
    current |= HAS_NVGOGGLES;
  }
  else if((current & HAS_NVGOGGLES) && !nvgoggles) {
    current &= ~HAS_NVGOGGLES;
  }
  else {
    return 0;
  }
  set_offset_int(index, OFFSET_NVGOGGLES, current);
  return 1;
}

/*******************************************************************************
* FILE.INC
*******************************************************************************/

stock filesize(const filename[512], {Float,_}:...) {
  new str[512];
  format_args(str, charsmax(str), 0);
  return file_size(str);
}

stock unlink(const filename[512], {Float,_}:...) {
  new str[512];
  format_args(str, charsmax(str), 0);
  return delete_file(str);
}

/*******************************************************************************
* FLOAT.INC
*******************************************************************************/

stock Float:floatabs(Float:value) {
  return fabs(value);
}

stock Float:floatsin(Float:value, anglemode:mode = radian) {
  return sin(value, mode);
}

stock Float:floatcos(Float:value, anglemode:mode = radian) {
  return cos(value, mode);
}

stock Float:floattan(Float:value, anglemode:mode = radian) {
  return tan(value, mode);
}

stock Float:floatatan(Float:angle, radix) {
  return atan(angle, anglemode:radix);
}

stock Float:floatacos(Float:angle, radix) {
  return acos(angle, anglemode:radix);
}

stock Float:floatasin(Float:angle, radix) {
  return asin(angle, anglemode:radix);
}

stock Float:floatatan2(Float:x, Float:y, radix) {
  return atan2(x, y, anglemode:radix);
}

stock Float:floatsqroot(Float:value) {
  return fsqroot(value);
}

stock Float:floatpower(Float:value, Float:exponent) {
  return fpower(value, exponent);
}

stock Float:floatlog(Float:value, Float:base = 10.0) {
  return flog(value, base);
}

stock Float:floatsinh(Float:angle, anglemode:mode = radian) {
  return sinh(angle, mode);
}

stock Float:floatcosh(Float:angle, anglemode:mode = radian) {
  return cosh(angle, mode);
}

stock Float:floattanh(Float:angle, anglemode:mode = radian) {
  return tanh(angle, mode);
}

/*******************************************************************************
* FUN.INC
*******************************************************************************/

stock spawn(index) {
  return user_spawn(index);
}

/*******************************************************************************
* LANG.INC
*******************************************************************************/

stock get_langsnum() {
  return get_languagesnum();
}

stock get_lang(id, name[3]) {
  return get_language(id, name, 2);
}

stock lang_exists(const name[]) {
  return is_language_known(name);
}

/*******************************************************************************
* MESSAGE_STOCKS.INC
*******************************************************************************/

stock dod_make_deathmsg(killer, victim, weaponNUM) {
  message_begin(MSG_ALL, get_user_msgid("DeathMsg"), {0, 0, 0}, 0);
  write_byte(killer);
  write_byte(victim);
  write_byte(weaponNUM);
  message_end();
  return 1;
}

stock user_silentkill(index) {
  static msgid = 0;
  new msgblock;
  if(!msgid) {
    msgid = get_user_msgid("DeathMsg");
  }
  msgblock = get_msg_block(msgid);
  set_msg_block(msgid, BLOCK_ONCE);
  user_kill(index, 1);
  set_msg_block(msgid, msgblock);
  return 1;
}

stock make_deathmsg(killer, victim, headshot, weapon[]) {
  message_begin(MSG_ALL, get_user_msgid("DeathMsg"), {0, 0, 0}, 0);
  write_byte(killer);
  write_byte(victim);
  new mod_name[10];
  get_modname(mod_name, charsmax(mod_name));
  if(equal(mod_name, "cstrike") || equal(mod_name, "czero") || equal(mod_name, "csv15") || equal(mod_name, "cs13")) {
    write_byte(headshot);
  }
  write_string(weapon);
  message_end();
  return 1;
}

/*******************************************************************************
* MESSAGES.INC
*******************************************************************************/

#define set_msg_arg_int(%1,%2,%3) set_msg_arg_int(%1,%3)
#define set_msg_arg_float(%1,%2,%3) set_msg_arg_float(%1,%3)
#define get_msg_argtype(%1) get_msg_arg_type(%1)

/*******************************************************************************
* NS2AMX.INC
*******************************************************************************/

stock get_private_i(index, offset, linuxdiff = 5) {
  return get_offset_int(index, offset, linuxdiff);
}

stock set_private_i(index, offset, value, linuxdiff = 5) {
  return set_offset_int(index, offset, value, linuxdiff);
}

stock Float:get_private_f(index, offset, linuxdiff = 5) {
  return get_offset_float(index, offset, linuxdiff);
}

stock set_private_f(index, offset, Float:value, linuxdiff = 5) {
  return set_offset_float(index, offset, value, linuxdiff);
}

stock gpgobals_time() {
  return floatround(get_gametime());
}

stock Float:get_range(ida, idb) {
  return entity_range(ida, idb);
}

stock supercede() {
  return 0;
}

stock register_clientkill() {
  return 0;
}

stock register_changelvl() {
  return 0;
}

stock register_msgblock(const msgName[]) {
  return set_msg_block(get_user_msgid(msgName), BLOCK_SET);
}

stock register_playback(event, cmd[]) {
  return 0;
}

stock has_weapon(index, weapon, setweapon = -1) {
  return user_has_weapon(index, weapon, setweapon);
}

stock gpglobals_v(type, Float:ret[3]) {
  switch(type) {
    case 1: return global_get_vector(GV_VEC_v_forward, ret);
    case 2: return global_get_vector(GV_VEC_v_right, ret);
    case 3: return global_get_vector(GV_VEC_v_up, ret);
  }
  return 0;
}

stock Float:pev_f(_index, _field) {
  return entity_get_float(_index, _field);
}

stock set_pev_f(_index, _field, Float:_val) {
  return entity_set_float(_index, _field, _val);
}

stock msg_args() {
  return read_datanum();
}

stock msg_dest() {
  return 0;
}

stock msg_type() {
  return 0;
}

stock msg_name() {
  return 0;
}

stock msg_data(value, ...) {
  #pragma unused value
  return 0;
}

stock get_filename(szFile[], len = -1) {
  new tmp[1];
  new res = get_plugin(0, szFile, len, tmp, 0, tmp, 0, tmp, 0, tmp, 0);
  return res;
}

stock ns2amx_version() {
  return 0;
}

stock set_kvhandled() {
  return 0;
}

stock ns2amx_moveto(idMoved, idDest) {
  new Float:origin[3];
  entity_get_vector(idDest, EV_VEC_origin, origin);
  entity_set_origin(idMoved, origin);
  return 1;
}

stock ns2amx_inrange(ida, idb, range) {
  if(entity_range(ida, idb) <= range) {
    return 1;
  }
  return 0;
}

/*******************************************************************************
* STRING.INC
*******************************************************************************/

stock Float:str_to_float(const string[]) {
  return floatstr(string);
}

stock strcat(dest[], const source[], maxlength) {
  return add(dest, maxlength, source);
}

stock strfind(const string[], const sub[], ignorecase = 0, pos = 0) {
  if(pos >= strlen(string)) return -1;
  if(ignorecase) {
    return containi(string[pos], sub);
  }
  return contain(string[pos], sub);
}

stock strcmp(const string1[], const string2[], ignorecase = 0) {
  if(ignorecase) {
    return (equali(string1, string2)) ? 0 : -1;
  }
  return (equal(string1, string2)) ? 0 : -1;
}

stock ucfirst(string[]) {
  string[0] = toupper(string[0]);
  return 1;
}

stock remove_filepath(szFilePath[], szFile[], pMax) {
  new len = strlen(szFilePath);
  while((--len >= 0) && (szFilePath[len] != '/') && (szFilePath[len] != '\')) { }
  copy(szFile, pMax, szFilePath[len + 1]);
  return 1;
}

/*******************************************************************************
* TIME.INC
*******************************************************************************/

/* Time unit types for get_time_length() */
enum {
  timeunit_seconds = 0,
  timeunit_minutes,
  timeunit_hours,
  timeunit_days,
  timeunit_weeks,
};

// Seconds are in each time unit
#define SECONDS_IN_MINUTE 60
#define SECONDS_IN_HOUR   3600
#define SECONDS_IN_DAY    86400
#define SECONDS_IN_WEEK   604800

/* Stock by Brad */
stock get_time_length(id, unitCnt, type, output[], outputLen) {
// IMPORTANT: 	You must add register_dictionary("time.txt") in plugin_init()
// id:          The player whose language the length should be translated to (or 0 for server language).
// unitCnt:     The number of time units you want translated into verbose text.
// type:        The type of unit (i.e. seconds, minutes, hours, days, weeks) that you are passing in.
// output:      The variable you want the verbose text to be placed in.
// outputLen:	The length of the output variable.
// NOTE: Translation feature disabled on AMX Mod conversion.

  if (unitCnt > 0) {
    // determine the number of each time unit there are
    new weekCnt = 0, dayCnt = 0, hourCnt = 0, minuteCnt = 0, secondCnt = 0;

    switch(type) {
      case timeunit_seconds: secondCnt = unitCnt;
      case timeunit_minutes: secondCnt = unitCnt * SECONDS_IN_MINUTE;
      case timeunit_hours:   secondCnt = unitCnt * SECONDS_IN_HOUR;
      case timeunit_days:    secondCnt = unitCnt * SECONDS_IN_DAY;
      case timeunit_weeks:   secondCnt = unitCnt * SECONDS_IN_WEEK;
    }

    weekCnt = secondCnt / SECONDS_IN_WEEK;
    secondCnt -= (weekCnt * SECONDS_IN_WEEK);

    dayCnt = secondCnt / SECONDS_IN_DAY;
    secondCnt -= (dayCnt * SECONDS_IN_DAY);

    hourCnt = secondCnt / SECONDS_IN_HOUR;
    secondCnt -= (hourCnt * SECONDS_IN_HOUR);

    minuteCnt = secondCnt / SECONDS_IN_MINUTE;
    secondCnt -= (minuteCnt * SECONDS_IN_MINUTE);

    // translate the unit counts into verbose text
    new maxElementIdx = -1;
    new timeElement[5][33];

    if(weekCnt > 0)
      format(timeElement[++maxElementIdx], 32, "%i %s", weekCnt, (weekCnt == 1) ? "week" : "weeks");
    if(dayCnt > 0)
      format(timeElement[++maxElementIdx], 32, "%i %s", dayCnt, (dayCnt == 1) ? "day" : "days");
    if(hourCnt > 0)
      format(timeElement[++maxElementIdx], 32, "%i %s", hourCnt, (hourCnt == 1) ? "hour" : "hours");
    if(minuteCnt > 0)
      format(timeElement[++maxElementIdx], 32, "%i %s", minuteCnt, (minuteCnt == 1) ? "minute" : "minutes");
    if(secondCnt > 0)
      format(timeElement[++maxElementIdx], 32, "%i %s", secondCnt, (secondCnt == 1) ? "second" : "seconds");

    switch(maxElementIdx) {
      case 0: format(output, outputLen, "%s", timeElement[0]);
      case 1: format(output, outputLen, "%s and %s", timeElement[0], timeElement[1]);
      case 2: format(output, outputLen, "%s, %s and %s", timeElement[0], timeElement[1], timeElement[2]);
      case 3: format(output, outputLen, "%s, %s, %s and %s", timeElement[0], timeElement[1], timeElement[2], timeElement[3]);
      case 4: format(output, outputLen, "%s, %s, %s, %s and %s", timeElement[0], timeElement[1], timeElement[2], timeElement[3], timeElement[4]);
    }
  }
}

/*******************************************************************************
* TSX.INC
*******************************************************************************/

stock ts_weaponspawn(const weaponid[], const duration[], const extraclip[], const spawnflags[], const Float:Origin[3]) {
  new ent = create_entity("ts_groundweapon");
  DispatchKeyValue(ent, "tsweaponid", weaponid);
  DispatchKeyValue(ent, "wduration", duration);
  DispatchKeyValue(ent, "wextraclip", extraclip);
  DispatchKeyValue(ent, "spawnflags", spawnflags);
  DispatchSpawn(ent);
  entity_set_origin(ent, Origin);
  return 1;
}

/*******************************************************************************
* VECTOR.INC
*******************************************************************************/

stock angle_vector(Float:angle[3], FRU, Float:ret[3]) {
  return angle_to_vector(angle, FRU, ret);
}

stock velocity_by_aim(iIndex, iVelocity, Float:vRetValue[3]) {
  return VelocityByAim(iIndex, iVelocity, vRetValue);
}

/*******************************************************************************
* XTRAFUN.INC
*******************************************************************************/

stock get_entity_velocity(id, velocity[3]) {
  new Float:vel[3];
  entity_get_vector(id, EV_VEC_velocity, vel);
  FVecIVec(vel, velocity);
  return 1;
}

stock set_entity_velocity(id, velocity[3]) {
  new Float:vel[3];
  IVecFVec(velocity, vel);
  entity_set_vector(id, EV_VEC_velocity, vel);
  return 1;
}

stock get_entity_origin(id, origin[3]) {
  new Float:orig[3];
  entity_get_vector(id, EV_VEC_origin, orig);
  FVecIVec(orig, origin);
  return 1;
}

stock set_entity_origin(id, origin[3]) {
  new Float:orig[3];
  IVecFVec(origin, orig);
  entity_set_vector(id, EV_VEC_origin, orig);
  return 1;
}

stock get_grenade_id(id, model[], len, grenadeid) {
  grenadeid = get_grenade(id, grenadeid);
  entity_get_string(grenadeid, EV_SZ_model, model, len);
  return grenadeid;
}

stock get_grenade_index(index, model[], len, grenadeindex = 0) {
  if(grenadeindex == 0) grenadeindex = -1;
  new entfind = grenadeindex;
  new entowner = index;
  for(;;) {
    entfind = find_entity(entfind, "grenade");
    if(entfind > 0 && is_entity(entfind)) {
      if(entity_get_edict(entfind, EV_ENT_owner) == entowner) {
        entity_get_string(entfind, EV_SZ_model, model, len);
        return entfind;
      }
    }
    else {
      return 0;
    }
  }
}

stock current_num_ents() {
  return get_num_ents();
}
