/*
 * Message natives
 *
 * (c) 2010 - AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _message_included
  #endinput
#endif
#define _message_included

#include <message_const>

/* These functions are used to generate client messages.
 * You may generate menu, smoke, shockwaves, thunderlights,
 * intermission and many other messages.
 * See HL SDK or message_const.inc for more examples. */
native message_begin(dest, msg_type, const origin[3] = {0, 0, 0}, player = 0);
native message_end();
native write_byte(x);
native write_char(x);
native write_short(x);
native write_long(x);
native write_entity(x);
native write_angle(x);
native write_coord(x);
native write_string(const x[]);

/* These are the same as above, except that the messages sent
 * are also sent to all other plugins and Metamod plugins.
 * This means that if you send one of these messages, other plugins will
 * be notified, which was previously impossible.
 * BE CAREFUL! Using these incorrectly, or not for their intended purpose,
 * could cause infinite recursion or something just as bad.
 * NOTE: These natives are experimental. */
native emessage_begin(dest, msg_type, const origin[3] = {0, 0, 0}, player = 0);
native emessage_end();
native ewrite_angle(x);
native ewrite_byte(x);
native ewrite_char(x);
native ewrite_coord(x);
native ewrite_entity(x);
native ewrite_long(x);
native ewrite_short(x);
native ewrite_string(const x[]);

/* Hooks a message to a single function.
 * This is like an advanced register_event.
 * The function you create should have the following arguments passed to it:
 *  msg_id, msg_dest, and msg_entity (id).
 * Just like that: my_message_function(msg_id, msg_dest, id)
 * You should NEVER send a message or use a command which sends a message
 * while in a register_message hook. */
native register_message(msg_type, const function[]);

/* Unregisters a message hook previously created with register_message.
 * You must pass the proper message id, and return value from the message
 * to unregister the message successfully. */
native unregister_message(msg_type, handle);

/* Returns the value of an argument in a message. */
native get_msg_arg_int(argpos);

/* Sets the value of an argument message. */
native set_msg_arg_int(argpos, value);

/* Returns the value of an argument in a message as a float. */
native Float:get_msg_arg_float(argpos);

/* Sets a floating value in a message argument. */
native set_msg_arg_float(argpos, Float:value);

/* Returns a string from a message argument. */
native get_msg_arg_string(argpos, result[], len);

/* Sets a string in a message argument. */
native set_msg_arg_string(argpos, const value[]);

/* Gets/Sets the origin of the current message. */
native get_msg_origin(Float:origin[3]);
native set_msg_origin(Float:origin[3]);

/* Sets the destination entity of a
 * MSG_ONE or MSG_ONE_UNRELIABLE message to another one. */
native set_msg_entity(entity);

/* Returns the type of an message's argument. */
native get_msg_arg_type(argpos);

/* Returns the number of arguments passed to the current message. */
native get_msg_args();

/* Gets/Sets what engine messages are blocked.
 * msg_flags (for set_msg_block() only):
 *  BLOCK_NOT  
 *  BLOCK_ONCE  
 *  BLOCK_SET */
native get_msg_block(msg_type);
native set_msg_block(msg_type, msg_flags);
