/*
 * Strings manipulation functions
 *
 * (c) 2002-2010 - OLO/AMX Mod Team
 * This file is provided as is (no warranties).
 *
 */

#if defined _string_included
  #endinput
#endif
#define _string_included

/* Decrease of -1 the value of an array.
 * Example:
 *  new myarray[32]
 *  copy(myarray, charsmax(myarray), "Hello")
 * Note: charsmax(myarray) will "return" 31. */
#define charsmax(%1) sizeof(%1) - 1

/* Checks if source contains string. On success function
 * returns position in source, on failure returns -1. */
native contain(const source[], const string[]);

/* Checks if source contains string with case ignoring. On success function
 * returns position in source, on failure returns -1. */
native containi(const source[], const string[]);

/* Replaces given string with another in given text. */
native replace(text[], len, const what[], const with[]);

/* Adds one string to another. Last parameter different from 0, specifies
 * how many chars we want to add. Function returns number of all merged chars. */
native add(dest[], len, const src[], max = 0);

/* Fills string with given format and parameters.
 * Function returns number of copied chars.
 * Example: format(dest, "Hello %s. You are %d years old", "Tom", 17) */
native format(output[], len, const format[], {Float,_}:...);

/* Same as format(), except does not perform a "copy back" check.
 * This means formatex() is faster, but DOES NOT ALLOW this type
 * of call:
 *  formatex(buffer, len, "%s", buffer)
 *  formatex(buffer, len, buffer, buffer)
 *  formatex(buffer, len, "%s", buffer[5])
 * This is because the output is directly stored into "buffer",
 * rather than copied back at the end. */
native formatex(output[], len, const format[], {Float,_}:...);

/* Gets parameters from function as formatted string. */
native format_args(output[], len, pos = 0, destid = -1, forcelang = -1);

/* Converts number to string. */
native num_to_str(num, string[], len);
native numtostr(num, string[], len);

/* Converts string to number. */
native str_to_num(const string[]);
native strtonum(const string[]);

/* Converts all chars in string to lower case. */
native strtolower(string[]);

/* Converts all chars in string to upper case. */
native strtoupper(string[]);

/* Checks if two strings equal. If len var is set
 * then there are only c chars compared. */
native equal(const a[], const b[], c = 0);

/* Checks if two strings equal with case ignoring.
 * If len var is set then there are only c chars compared. */
native equali(const a[], const b[], c = 0);

/* Copies one string to another.
 * By len var you may specify max. number of chars to copy. */
native copy(dest[], len, const src[]);

/* Copies one string to another until char ch is found.
 * By len var you may specify max. number of chars to copy. */
native copyc(dest[], len, const src[], ch);

/* Sets string with given character. */
native setc(src[], len, ch);

/* Gets parameters from text.
 * Example: To split text: "^"This is^" the best year",
 * call function like this: parse(text,arg1,len1,arg2,len2,arg3,len3,arg4,len4)
 * and you will get: "This is", "the", "best", "year"
 * Function returns number of parsed parameters. */
native parse(const text[], ...);

/* Trims whitespace characters from the beginning and end of a string. */
native trim(string[]);

/* Returns true when value is digit. */
native isdigit(ch);

/* Returns true when value is letter. */
native isalpha(ch);

/* Returns true when value is space. */
native isspace(ch);

/* Returns true when value is digit or letter. */
native isalnum(ch);

/* Returns true when value is a quote ("). */
stock isquote(ch) {
  if(ch == '"') {
    return true
  }
  return false
}

/* Tests if given string contains only digits.
 * Note: By AMX Mod X Dev Team. */
stock bool:is_str_num(string[]) {
  new i = 0
  while(string[i] && isdigit(string[i]))
    ++i
  return (string[i] == 0 && i != 0)
}

/* Strips spaces and quotes from the beginning and end of a string.
 * This is an improved version of trim(), same thing except you can
 * choose to let an existing quote (") at each extremities of the string.
 * Example:
 *  Original sentence -> "" "My mother says hi to you "" (result = 31)
 *  With trim_imp() and both bools on true -> "My mother says hi to you" (result = 26)
 *
 * If you let both bSkipFirst and bSkipLast booleans to false, existing quotes
 * around the text will be removed (not like in the example).
 * This function returns to the new length of the string (may include quotes too).
 * Note: By AMX Mod Team (updated in 2011-07-21). */
stock trim_imp(szString[], bool:bSkipFirst = false, bool:bSkipLast = false) {
  new iEnd = 0
  while(szString[iEnd] != 0) { iEnd++; }
  if(iEnd == 0) {
    return 0
  }

  new iPos = 0, iFlags = 0

  do {
    switch(szString[iPos]) {
      case ' ': iPos++
      case '"': {
        iPos++
        iFlags |= (1<<0)
      }
      default: break
    }
  }
  while(iPos < iEnd)

  if(iPos == iEnd) {
    szString[0] = 0
    return 0
  }

  while(iEnd > iPos) {
    switch(szString[iEnd - 1]) {
      case ' ': szString[--iEnd] = 0
      case '"': {
        szString[--iEnd] = 0
        iFlags |= (1<<1)
      }
      default: break
    }
  }

  if(iPos == 0) {
    if(bSkipLast && (iFlags & (1<<1)) ) {
      szString[iEnd++] = '"'
    }
    return iEnd
  }

  new iStart = 0, iDiff = iPos

  if(bSkipFirst && (iFlags & (1<<0)) ) {
    szString[iStart++] = '"'
    iEnd++
  }
  while(iPos < iEnd) {
    szString[iStart++] = szString[iPos++]
  }
  if(bSkipLast && (iFlags & (1<<1)) ) {
    szString[iStart++] = '"'
    iEnd++
  }
  szString[iEnd - iDiff] = 0
  return iStart
}

/* Breaks a string into two halves, by token.
 * Note: C++ version by AMX Mod X Dev Team (fixed in 2011-05-28). */
stock strtok(const string[], left[], leftLen, right[], rightLen, token = ' ', Trim = 0) {
  new left_pos = 0
  new right_pos = 0
  new i = 0
  new bool:done_flag = false
  new len = strlen(string)
  for(i = 0; i < len; ++i) {
    if(!done_flag && string[i] == token) {
      done_flag = true
      ++i
    }
    if(done_flag) {
      if(right_pos < rightLen) {
        right[right_pos++] = string[i]
      }
    }
    else {
      if(left_pos < leftLen) {
        left[left_pos++] = string[i]
      }
    }
  }
  right[right_pos] = 0
  left[left_pos] = 0
  if(Trim) {
    trim(left)
  }
  return 1
}

/* Gets parameters from text one at a time.
 * Note: C++ version by AMX Mod X Dev Team. */
stock strbreak(const text[], Left[], leftLen, Right[], rightLen) {
  new len = strlen(text)
  parse(text, Left, leftLen, Right, rightLen)
  new len1 = strlen(Left), len2 = strlen(Right)
  while(len1 < len && !equal(text[len1], Right, len2)) { ++len1; }
  if(len1 < len) {
    copy(Right, rightLen, text[len1])
  }
  return 1
}

/* It is basically strbreak but you have a delimiter that is more than one character in length.
 * You pass the Input string, the Left output, the max length of the left output,
 * the right output , the max right length, and then the delimiter string.
 * Note: By Suicid3. */
stock split(szInput[], szLeft[], pL_Max, szRight[], pR_Max, szDelim[]) {
  new iEnd = contain(szInput, szDelim)
  new iStart = iEnd + strlen(szDelim)
  if(iEnd == -1) {
    iStart = copy(szLeft, pL_Max, szInput)
    copy(szRight, pR_Max, szInput[iStart])
    return 1
  }
  if(pL_Max >= iEnd) {
    copy(szLeft, iEnd, szInput)
  }
  else {
    copy(szLeft, pL_Max, szInput)
  }
  copy(szRight, pR_Max, szInput[iStart])
  return 1
}

/* Replaces a contained string. Returns number of replacements.
 * Note: By jtp10181. */
stock replace_all(string[], len, what[], with[]) {
  new pos = 0
  if((pos = contain(string, what)) == -1) { // By AMX Mod X Dev Team
    return 0
  }
  new with_len = strlen(with)
  new total_len = strlen(string)
  new diff_len = strlen(what) - with_len
  if(total_len - diff_len > len) {
    return 0
  }
  new num = 0, temp_test = 0
  while(replace(string[pos], len-pos, what, with) != 0) {
    total_len -= diff_len
    if(total_len - diff_len > len) {
      break
    }
    pos += with_len
    if(pos >= total_len) {
      break
    }
    if((temp_test = contain(string[pos], what)) == -1) { // By AMX Mod X Dev Team
      break
    }
    pos += temp_test
    ++num
  }
  return num
}
