/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * A part of the code is from Ludwig Van's admin chat deluxe plugin.
 * Credit for fxsay and flicksay goes to NiLuJe.
 *
 */

/*
 * Commands:
 * say_team @<message> - displays message to admins
 * Note: Usefull command for send a message to admins when you are not one.
 *
 * say <@[@|@]|#[#|#]|$[$|$]>[color] <message> - displays chat/hud message
 * For people who don't know these shortcuts:
 *  say @@@message = amx_scrollsay message
 *  say @@@ymessage = amx_scrollsayy message
 *  say @@message = amx_csay message
 *  say @@ymessage = amx_csayy message
 *  say @message = amx_tsay message
 *  say @ymessage = amx_tsayy message
 *  say ###message = amx_say message
 *  say ###ymessage = amx_sayy message
 *  say ##message = amx_chat message
 *  say #message = amx_psay message
 *  say #ymessage = amx_psayy message
 *  say $$$message = amx_fsay message
 *  say $$$ymessage = amx_fsayy message
 *  say $$message = amx_flicksay message
 *  say $$ymessage = amx_flicksayy message
 *  say $message = amx_fxsay message
 *  say $ymessage = amx_fxsayy message
 *
 * amx_say <message> - sends message to all players
 * amx_sayy <message> - sends message to all players - anonymous
 * amx_chat <message> - sends message to admins
 * amx_psay <name|#userid|authid> <message> - sends private message
 * amx_psayy <name|#userid|authid> <message> - sends private message - anonymous
 * amx_tsay [color] <message> - sends left side hud message to all players
 * amx_tsayy [color] <message> - sends left side hud message to all players - anonymous
 * amx_csay [color] <message> - sends center hud message to all players
 * amx_csayy [color] <message> - sends center hud message to all players - anonymous
 * amx_fxsay [color] <message> - sends fx hud message to all players
 * amx_fxsayy [color] <message> - sends fx hud message to all players - anonymous
 * amx_cfxsay [color] <message> - sends center fx hud message to all players
 * amx_cfxsayy [color] <message> - sends center fx hud message to all players - anonymous
 * amx_flicksay [color] <message> - sends flickering hud message to all players
 * amx_flicksayy [color] <message> - sends flickering hud message to all players - anonymous
 * amx_cflicksay [color] <message> - sends center flickering hud message to all players
 * amx_cflicksayy [color] <message> - sends center flickering hud message to all players - anonymous
 * amx_fsay <x position 0-99> <y position 0-99> [color] <message> - sends hud message to all players
 * amx_fsayy <x position 0-99> <y position 0-99> [color] <message> - sends hud message to all players - anonymous
 * amx_scrollsay [color] <message> - sends scroll message to all players
 * amx_scrollsayy [color] <message> - sends scroll message to all players - anonymous
 *
 * Note: Chat commands with both "y" won't show the admin name depending of the
 * both amx_chat_anonymous and amx_show_activity cvars settings.
 *
 * Cvars:
 * amx_chat_anonymous <0|1|2> - 0: disabled
 *                              1: enabled, but only if amx_show_activity is not set to 2
 *                              2: always enabled
 * Note: This cvar has no effect on the simple chat commands (without both "y").
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_SAYTEAM_ADMINS ADMIN_ALL   // say_team @message - sends message to the admins
#define FLAG_AMX_SAYCHAT        ADMIN_CHAT  // say <@[@|@]|#[#|#]|$[$|$]>[color] <message>
#define FLAG_AMX_SAY            ADMIN_CHAT
#define FLAG_AMX_CHAT           ADMIN_CHAT
#define FLAG_AMX_PSAY           ADMIN_CHAT
#define FLAG_AMX_TSAY           ADMIN_CHAT
#define FLAG_AMX_CSAY           ADMIN_CHAT
#define FLAG_AMX_FXSAY          ADMIN_CHAT
#define FLAG_AMX_CFXSAY         ADMIN_CHAT
#define FLAG_AMX_FLICKSAY       ADMIN_CHAT
#define FLAG_AMX_CFLICKSAY      ADMIN_CHAT
#define FLAG_AMX_FSAY           ADMIN_CHAT
#define FLAG_AMX_SCROLLSAY      ADMIN_CHAT

const Float:HOLDTIME = 8.0 // number of seconds a hud message lasts

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const MAX_CLR = 21

new const g_Colors[MAX_CLR][] = {"!","?","white","red","green","blue","yellow","magenta","cyan","orange","amx","amxmod","w","r","g","b","y","m","c","o","a"}
new const g_Values[MAX_CLR][] = {
  {255,255,255}, // !
  {255,255,255}, // ?
  {255,255,255}, // white
  {255,0,0}, // red
  {0,255,0}, // green
  {0,0,255}, // blue
  {255,255,0}, // yellow
  {255,0,255}, // magenta
  {0,255,255}, // cyan
  {255,100,0}, // orange
  {0,100,255}, // AMX Mod original info color!
  {0,100,255}, // AMX Mod original info color!
  {255,255,255}, // white
  {255,0,0}, // red
  {0,255,0}, // green
  {0,0,255}, // blue
  {255,255,0}, //yellow
  {255,0,255}, // magenta
  {0,255,255}, // cyan
  {255,100,0}, // orange
  {0,100,255} // AMX Mod original info color!
}

const Float:SCROLL_SPEED = 0.3

new sc_msgChannel
new sc_start_pos
new sc_end_pos
new sc_scroll_msg[384]
new sc_display_msg[384]
new Float:sc_x_pos
new sc_a
new sc_timer
new sc_length

new g_msgChannel

new g_coloredChat

new g_cvar_amx_chat_anonymous
new g_cvar_amx_show_activity

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("adminchat")
  register_plugin(_T("Admin Chat"),AMX_VERSION,"AMX Mod Team")
  register_clcmd("say_team","cmdSayAdmin",FLAG_AMX_SAYTEAM_ADMINS,_T("@<message> - displays message to admins"))
  register_clcmd("say","cmdSayChat",FLAG_AMX_SAYCHAT,_T("<@[@|@]|#[#|#]|$[$|$]>[color] <message> - displays chat/hud message"))
  register_concmd("amx_say","cmdSay",FLAG_AMX_SAY,_T("<message> - sends message to all players"))
  register_concmd("amx_sayy","cmdSay",FLAG_AMX_SAY,_T("<message> - sends message to all players - anonymous"))
  register_concmd("amx_chat","cmdChat",FLAG_AMX_CHAT,_T("<message> - sends message to admins"))
  register_concmd("amx_psay","cmdPsay",FLAG_AMX_PSAY,_T("<name|#userid|authid> <message> - sends private message"))
  register_concmd("amx_psayy","cmdPsay",FLAG_AMX_PSAY,_T("<name|#userid|authid> <message> - sends private message - anonymous"))
  register_concmd("amx_tsay","cmdTsay",FLAG_AMX_TSAY,_T("[color] <message> - sends left side hud message to all players"))
  register_concmd("amx_tsayy","cmdTsay",FLAG_AMX_TSAY,_T("[color] <message> - sends left side hud message to all players - anonymous"))
  register_concmd("amx_csay","cmdTsay",FLAG_AMX_CSAY,_T("[color] <message> - sends center hud message to all players"))
  register_concmd("amx_csayy","cmdTsay",FLAG_AMX_CSAY,_T("[color] <message> - sends center hud message to all players - anonymous"))
  register_concmd("amx_fxsay","cmdTsay",FLAG_AMX_FXSAY,_T("[color] <message> - sends fx hud message to all players"))
  register_concmd("amx_fxsayy","cmdTsay",FLAG_AMX_FXSAY,_T("[color] <message> - sends fx hud message to all players - anonymous"))
  register_concmd("amx_cfxsay","cmdTsay",FLAG_AMX_CFXSAY,_T("[color] <message> - sends center fx hud message to all players"))
  register_concmd("amx_cfxsayy","cmdTsay",FLAG_AMX_CFXSAY,_T("[color] <message> - sends center fx hud message to all players - anonymous"))
  register_concmd("amx_flicksay","cmdTsay",FLAG_AMX_FLICKSAY,_T("[color] <message> - sends flickering hud message to all players"))
  register_concmd("amx_flicksayy","cmdTsay",FLAG_AMX_FLICKSAY,_T("[color] <message> - sends flickering hud message to all players - anonymous"))
  register_concmd("amx_cflicksay","cmdTsay",FLAG_AMX_CFLICKSAY,_T("[color] <message> - sends center flickering hud message to all players"))
  register_concmd("amx_cflicksayy","cmdTsay",FLAG_AMX_CFLICKSAY,_T("[color] <message> - sends center flickering hud message to all players - anonymous"))
  register_concmd("amx_fsay","cmdFsay",FLAG_AMX_FSAY,_T("<x position 0-99> <y position 0-99> [color] <message> - sends hud message to all players"))
  register_concmd("amx_fsayy","cmdFsay",FLAG_AMX_FSAY,_T("<x position 0-99> <y position 0-99> [color] <message> - sends hud message to all players - anonymous"))
  register_concmd("amx_scrollsay","cmdScrollsay",FLAG_AMX_SCROLLSAY,_T("[color] <message> - sends scroll message to all players"))
  register_concmd("amx_scrollsayy","cmdScrollsay",FLAG_AMX_SCROLLSAY,_T("[color] <message> - sends scroll message to all players - anonymous"))
  g_cvar_amx_chat_anonymous = register_cvar("amx_chat_anonymous","2")
  g_cvar_amx_show_activity = get_cvar_pointer("amx_show_activity")
  #if !defined NO_STEAM
  g_coloredChat = (is_running("cstrike") || is_running("czero"))
  #else
  g_coloredChat = false
  #endif
}

public cmdSayChat(id) { 
  if(!access(id, FLAG_AMX_SAYCHAT))
    return PLUGIN_CONTINUE 

  new message[192]
  read_args(message, 191)
  replace_all(message, 191, "^"", "")

  if(equal(message, "@@@", 3)) {
    if(message[3] == 'y') {
      client_cmd(id, "amx_scrollsayy %s", message[4])
    }
    else {
      client_cmd(id, "amx_scrollsay %s", message[3])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "@@", 2)) {
    if(message[2] == 'y') {
      client_cmd(id, "amx_csayy %s", message[3])
    }
    else {
      client_cmd(id, "amx_csay %s", message[2])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "@", 1)) {
    if(message[1] == 'y') {
      client_cmd(id, "amx_tsayy %s", message[2])
    }
    else {
      client_cmd(id, "amx_tsay %s", message[1])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "###", 3)) {
    if(message[3] == 'y') {
      client_cmd(id, "amx_sayy %s", message[4])
    }
    else {
      client_cmd(id, "amx_say %s", message[3])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "##", 2)) {
    client_cmd(id, "amx_chat %s", message[2])
    return PLUGIN_HANDLED
  }
  else if(equal(message, "#", 1)) {
    if(message[1] == 'y') {
      client_cmd(id, "amx_psayy %s", message[2])
    }
    else {
      client_cmd(id, "amx_psay %s", message[1])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "$$$", 3)) {
    if(message[3] == 'y') {
      client_cmd(id, "amx_fsayy %s", message[4])
    }
    else {
      client_cmd(id, "amx_fsay %s", message[3])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "$$", 2)) {
    if(message[2] == 'y') {
      client_cmd(id, "amx_flicksayy %s", message[3])
    }
    else {
      client_cmd(id, "amx_flicksay %s", message[2])
    }
    return PLUGIN_HANDLED
  }
  else if(equal(message, "$", 1)) {
    if(message[1] == 'y') {
      client_cmd(id, "amx_fxsayy %s", message[2])
    }
    else {
      client_cmd(id, "amx_fxsay %s", message[1])
    }
    return PLUGIN_HANDLED
  }
  return PLUGIN_CONTINUE
}

public cmdSayAdmin(id) {
  if(!access(id, FLAG_AMX_SAYTEAM_ADMINS))
    return PLUGIN_CONTINUE

  new said[3]
  read_argv(1, said, 2)
  if(said[0] != '@' || (said[0] == '@' && !said[1])) return PLUGIN_CONTINUE
  new message[192], name[32], authid[32], ipaddress[24], userid
  new players[32], inum, player
  read_args(message, 191)
  replace_all(message, 191, "^"", "")
  get_user_authid(id, authid, 31)
  get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)
  log_amx("Chat: ^"%s<%d><%s><%s>^" chat ^"%s^" (from command: say_team @<message>)",name,userid,authid,ipaddress,message[1])
  log_message("^"%s<%d><%s><%s>^" triggered ^"amx_chat^" (text ^"%s^") (from command: say_team @<message>)",name,userid,authid,ipaddress,message[1])
  new isAdmin = is_user_realadmin(id) ? 1 : 0
  get_players(players, inum, "c")
  for(new i = 0; i < inum; ++i) {
    player = players[i]
    if(player == id || is_user_realadmin(player)) {
      switch(isAdmin) {
        case 1: {
          if(g_coloredChat) client_print_color(id, player, "(^4ADMIN^1) ^3%s ^1:  %s", name, message[1])
          else client_print(player, print_chat, _T("(ADMIN) %s :  %s"), name, message[1])
        }
        default: {
          if(g_coloredChat) client_print_color(id, player, _T("(^4PLAYER^1) ^3%s ^1:  %s"), name, message[1])
          else client_print(player, print_chat, _T("(PLAYER) %s :  %s"), name, message[1])
        }
      }
    }
  }
  return PLUGIN_HANDLED
}

public cmdChat(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new message[192], name[32], authid[32], ipaddress[24], players[32], inum, player, userid
  read_args(message, 191)
  replace_all(message, 191, "^"", "")
  get_user_authid(id, authid, 31)
  if(id == 0) name = _T("SERVER")
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)
  get_players(players, inum, "c")
  log_amx("Chat: ^"%s<%d><%s><%s>^" chat ^"%s^"",name,userid,authid,ipaddress,message)
  log_message("^"%s<%d><%s><%s>^" triggered ^"amx_chat^" (text ^"%s^")",name,userid,authid,ipaddress,message)
  console_print(id, message)
  for(new i = 0; i < inum; ++i) {
    player = players[i]
    if(is_user_realadmin(player)) {
      if(g_coloredChat) client_print_color(id, player, "(^4ADMINS^1) !team!%s ^1:  %s", name, message)
      else client_print(player, print_chat, _T("(ADMINS) %s :  %s"), name, message)
    }
  }
  return PLUGIN_HANDLED
}

public cmdSay(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new cmd[16]
  read_argv(0, cmd, 15)
  new message[192], name[32], authid[32], ipaddress[24], userid
  read_args(message, 191)
  replace_all(message, 191, "^"", "")
  get_user_authid(id, authid, 31)
  if(id == 0) name = _T("SERVER")
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)

  new amx_chat_anonymous = get_cvarptr_num(g_cvar_amx_chat_anonymous)
  if(containi(cmd, "yy") == -1 || amx_chat_anonymous == 0 || (amx_chat_anonymous == 1 && get_cvarptr_num(g_cvar_amx_show_activity) == 2)) {
    if(g_coloredChat) client_print_color(id, 0, _T("(^4ALL^1) !team!%s ^1:  %s"), name, message)
    else client_print(0, print_chat, _T("(ALL) %s :  %s"), name, message)
  }
  else {
    if(g_coloredChat) client_print_color(id, 0, _T("(^4ALL^1) ADMIN :  %s"), message)
    else client_print(0, print_chat, _T("(ALL) ADMIN :  %s"), message)
  }
  console_print(id, _T("(ALL) %s :  %s"), name, message)
  log_amx("Chat: ^"%s<%d><%s><%s>^" say ^"%s^"",name,userid,authid,ipaddress,message)
  log_message("^"%s<%d><%s><%s>^" triggered ^"amx_say^" (text ^"%s^")",name,userid,authid,ipaddress,message)
  return PLUGIN_HANDLED
}

public cmdPsay(id,level,cid) {
  if(!cmd_access(id,level,cid,3))
    return PLUGIN_HANDLED

  new cmd[16]
  read_argv(0, cmd, 15)
  new name[32]
  read_argv(1, name, 31)
  new priv = cmd_target(id, name, 8)
  if(!priv) return PLUGIN_HANDLED
  new length = strlen(name) + 1
  new message[192], name2[32], authid[32], authid2[32], ipaddress[24], ipaddress2[24], userid, userid2
  get_user_authid(id, authid, 31)
  get_user_ip(id, ipaddress, 23, 1)
  if(id == 0) name2 = _T("SERVER")
  else get_user_name(id, name2, 31)
  userid = get_user_userid(id)
  read_args(message, 191)
  if(message[0] == '"' && message[length] == '"') { // HLSW fix
    message[0] = ' '
    message[length] = ' '
    length += 2
  }
  replace_all(message[length], 191-length, "^"", "")
  get_user_name(priv, name, 31)

  new amx_chat_anonymous = get_cvarptr_num(g_cvar_amx_chat_anonymous)
  if(containi(cmd, "yy") == -1 || amx_chat_anonymous == 0 || (amx_chat_anonymous == 1 && get_cvarptr_num(g_cvar_amx_show_activity) == 2)) {
    if(id && id != priv) {
      if(g_coloredChat) client_print_color(id, id, "(^4%s^1) ^3%s ^1:  %s", name, name2, message[length])
      else client_print(id, print_chat, "(%s) %s :  %s", name, name2, message[length])
    }
    if(g_coloredChat) client_print_color(id, priv, "(^4%s^1) !team!%s ^1:  %s", name, name2, message[length])
    else client_print(priv, print_chat, "(%s) %s :  %s", name, name2, message[length])
  }
  else {
    if(id && id != priv) {
      if(g_coloredChat) client_print_color(id, id, "(^4%s^1) ^3%s ^1:  %s", name, name2, message[length])
      else client_print(id, print_chat, "(%s) %s :  %s", name, name2, message[length])
    }
    if(g_coloredChat) client_print_color(id, priv, "(^4%s^1) ADMIN :  %s", name, message[length])
    else client_print(priv, print_chat, _T("(%s) ADMIN :  %s"), name, message[length])
  }

  console_print(id, "(%s) %s :  %s", name, name2, message[length])
  get_user_authid(priv, authid2, 31)
  get_user_ip(priv, ipaddress2, 23, 1)
  userid2 = get_user_userid(priv)
  log_amx("Chat: ^"%s<%d><%s><%s>^" psay ^"%s<%d><%s><%s>^" ^"%s^"",
    name2,userid,authid,ipaddress,name,userid2,authid2,ipaddress2,message[length])
  log_message("^"%s<%d><%s><%s>^" triggered ^"amx_psay^" against ^"%s<%d><%s><%s>^" (text ^"%s^")",
    name2,userid,authid,ipaddress,name,userid2,authid2,ipaddress2,message[length])
  return PLUGIN_HANDLED
}

public cmdTsay(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new cmd[16], color[12], message[192], name[32], authid[32], ipaddress[24], userid = 0
  read_argv(0, cmd, 15)
  read_args(message, 191)
  parse(message, color, 11)
  new found = 0, a = 0
  if(equal(color, "!", 1)) {
    a = 0
    found = 1
    new sred[4], sgreen[4], sblue[4], ired, igreen, iblue
    if(strlen(color) < 2) {
      g_Values[0][0] = 255
      g_Values[0][1] = 255
      g_Values[0][2] = 255
    }
    else {
      copy(sred, 3, color[1])
      copy(sgreen, 3, color[4])
      copy(sblue, 3, color[7])
      ired = str_to_num(sred)
      igreen = str_to_num(sgreen)
      iblue = str_to_num(sblue)
      if(ired < 0 || ired > 255)
        ired = 255
      if(igreen < 0 || igreen > 255)
        igreen = 255
      if(iblue < 0 || iblue > 255)
        iblue = 255
      g_Values[0][0] = ired
      g_Values[0][1] = igreen
      g_Values[0][2] = iblue
    }
  }
  else if(equal(color, "?")) {
    a = 1
    found = 1
    g_Values[1][0] = random_num(32,255)
    g_Values[1][1] = random_num(32,255)
    g_Values[1][2] = random_num(32,255)
  }
  else {
    for(new i = 2; i < MAX_CLR; ++i) {
      if(equali(color, _T(g_Colors[i], id)) || equali(color, g_Colors[i])) {
        a = i
        found = 1
        break
      }
    }
  }
  if(!found) a = 2
  
  new length = found ? (strlen(color) + 1) : 0
  if(++g_msgChannel > 6 || g_msgChannel < 3)
    g_msgChannel = 3
  get_user_authid(id, authid, 31)
  if(id == 0) name = _T("SERVER")
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)

  if(equali(cmd, "tsay", 4) || equali(cmd[4], "tsay", 4))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], 0.05, 0.55+float(g_msgChannel)/35.0, 0, 6.0, HOLDTIME, 0.5, 0.15, g_msgChannel)
  else if(equali(cmd, "csay", 4) || equali(cmd[4], "csay", 4))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], -1.0, 0.1+float(g_msgChannel)/35.0, 0, 6.0, HOLDTIME, 0.5, 0.15, g_msgChannel)
  else if(equali(cmd, "fxsay", 5) || equali(cmd[4], "fxsay", 5))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], 0.05, 0.55+float(g_msgChannel)/35.0, 2, 0.25, HOLDTIME, 0.05, 1.0, g_msgChannel)
  else if(equali(cmd, "cfxsay", 6) || equali(cmd[4], "cfxsay", 6))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], -1.0, 0.1+float(g_msgChannel)/35.0, 2, 0.25, HOLDTIME, 0.05, 1.0, g_msgChannel)
  else if(equali(cmd, "flicksay", 8) || equali(cmd[4], "flicksay", 8))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], 0.05, 0.55+float(g_msgChannel)/35.0, 1, 6.0, HOLDTIME, 0.5, 0.2, g_msgChannel)
  else if(equali(cmd, "cflicksay", 9) || equali(cmd[4], "cflicksay", 9))
    set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], -1.0, 0.1+float(g_msgChannel)/35.0, 1, 6.0, HOLDTIME, 0.5, 0.2, g_msgChannel)

  replace_all(message[length], 191-length, "^"", "")
  replace_all(message[length], 191-length, "^^n", "^n")
  new amx_chat_anonymous = get_cvarptr_num(g_cvar_amx_chat_anonymous)
  if(containi(cmd, "yy") == -1 || amx_chat_anonymous == 0 || (amx_chat_anonymous == 1 && get_cvarptr_num(g_cvar_amx_show_activity) == 2)) {
    show_hudmessage(0, "%s :  %s", name, message[length])
    replace_all(message[length], 191-length, "^n", " ")
    client_print(0, print_notify, "%s :  %s", name, message[length])
  }
  else {
    show_hudmessage(0, message[length])
    replace_all(message[length], 191-length, "^n", " ")
    client_print(0, print_notify, message[length])
  }

  log_amx("Chat: ^"%s<%d><%s><%s>^" %s ^"%s^"",name,userid,authid,ipaddress,cmd,message[length])
  log_message("^"%s<%d><%s><%s>^" triggered ^"%s^" (text ^"%s^") (color ^"%s^")",
    name,userid,authid,ipaddress,cmd,message[length],g_Colors[a])
  return PLUGIN_HANDLED
}

public cmdFsay(id,level,cid) {
  if(!cmd_access(id,level,cid,4))
    return PLUGIN_HANDLED

  new cmd[16], x_pos[8], y_pos[8], color[12], message[192], name[32], authid[32], ipaddress[24], userid = 0
  read_argv(0, cmd, 15)
  read_args(message, 191)
  parse(message, x_pos, 7, y_pos, 7, color, 11)
  new len = strlen(x_pos) + 1 + strlen(y_pos) + 1
  new found = 0, a = 0
  if(equal(color, "!", 1)) {
    a = 0
    found = 1
    new sred[4], sgreen[4], sblue[4], ired, igreen, iblue
    if(strlen(color) < 2) {
      g_Values[0][0] = 255
      g_Values[0][1] = 255
      g_Values[0][2] = 255
    }
    else {
      copy(sred, 3, color[1])
      copy(sgreen, 3, color[4])
      copy(sblue, 3, color[7])
      ired = str_to_num(sred)
      igreen = str_to_num(sgreen)
      iblue = str_to_num(sblue)
      if(ired < 0 || ired > 255)
        ired = 255
      if(igreen < 0 || igreen > 255)
        igreen = 255
      if(iblue < 0 || iblue > 255)
        iblue = 255
      g_Values[0][0] = ired
      g_Values[0][1] = igreen
      g_Values[0][2] = iblue
    }
  }
  else if(equal(color, "?")) {
    a = 1
    found = 1
    g_Values[1][0] = random_num(32,255)
    g_Values[1][1] = random_num(32,255)
    g_Values[1][2] = random_num(32,255)
  }
  else {
    for(new i = 2; i < MAX_CLR; ++i) {
      if(equali(color, _T(g_Colors[i], id)) || equali(color, g_Colors[i])) {
        a = i
        found = 1
        break
      }
    }
  }
  if(!found) a = 2
 
  if(equal(x_pos, "-", 1))
    copy(x_pos, 7, "-100")
  new iXpos = str_to_num(x_pos)
  new iYpos = str_to_num(y_pos)
  if(iXpos > 99) iXpos = 99
  if(iYpos < 0)  iYpos = 0
  if(iYpos > 99) iYpos = 99
  new Float:xpos = (iXpos * 0.01)
  new Float:ypos = (iYpos * 0.01)
  new length = found ? (len + strlen(color) + 1) : len
  if(++g_msgChannel > 6 || g_msgChannel < 3)
    g_msgChannel = 3
  get_user_authid(id, authid, 31)
  if(id == 0) name = _T("SERVER")
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)
  set_hudmessage(g_Values[a][0], g_Values[a][1], g_Values[a][2], xpos, ypos, 0, 6.0, HOLDTIME, 0.5, 0.15, g_msgChannel)

  replace_all(message[length], 191-length, "^"", "")
  replace_all(message[length], 191-length, "^^n", "^n")
  new amx_chat_anonymous = get_cvarptr_num(g_cvar_amx_chat_anonymous)
  if(containi(cmd, "yy") == -1 || amx_chat_anonymous == 0 || (amx_chat_anonymous == 1 && get_cvarptr_num(g_cvar_amx_show_activity) == 2)) {
    show_hudmessage(0, "%s :  %s", name, message[length])
    replace_all(message[length], 191-length, "^n", " ")
    client_print(0, print_notify, "%s :  %s", name, message[length])
  }
  else {
    show_hudmessage(0, message[length])
    replace_all(message[length], 191-length, "^n", " ")
    client_print(0, print_notify, message[length])
  }

  log_amx("Chat: ^"%s<%d><%s><%s>^" %s ^"%s^"",name,userid,authid,ipaddress,cmd,message[length])
  log_message("^"%s<%d><%s><%s>^" triggered ^"%s^" (text ^"%s^") (color ^"%s^")",
    name,userid,authid,ipaddress,cmd,message[length],g_Colors[a])
  return PLUGIN_HANDLED
}

public cmdScrollsay(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  if(sc_timer > 0) {
    console_print(id, _T("There is already a scrolling message in progress."))
    return PLUGIN_HANDLED
  }
  new cmd[16], color[12], name[32], authid[32], ipaddress[24], userid = 0
  read_argv(0, cmd, 15)
  read_args(sc_scroll_msg, 383)
  parse(sc_scroll_msg, color, 11)
  new found = 0
  sc_a = 0
  if(equal(color, "!", 1)) {
    sc_a = 0
    found = 1
    new sred[4], sgreen[4], sblue[4], ired, igreen, iblue
    if(strlen(color) < 2) {
      g_Values[0][0] = 255
      g_Values[0][1] = 255
      g_Values[0][2] = 255
    }
    else {
      copy(sred, 3, color[1])
      copy(sgreen, 3, color[4])
      copy(sblue, 3, color[7])
      ired = str_to_num(sred)
      igreen = str_to_num(sgreen)
      iblue = str_to_num(sblue)
      if(ired < 0 || ired > 255)
        ired = 255
      if(igreen < 0 || igreen > 255)
        igreen = 255
      if(iblue < 0 || iblue > 255)
        iblue = 255
      g_Values[0][0] = ired
      g_Values[0][1] = igreen
      g_Values[0][2] = iblue
    }
  }
  else if(equal(color, "?")) {
    sc_a = 1
    found = 1
    g_Values[1][0] = random_num(32,255)
    g_Values[1][1] = random_num(32,255)
    g_Values[1][2] = random_num(32,255)
  }
  else {
    for(new i = 2; i < MAX_CLR; ++i) {
      if(equali(color, _T(g_Colors[i], id)) || equali(color, g_Colors[i])) {
        sc_a = i
        found = 1
        break
      }
    }
  }
  if(!found) sc_a = 2

  new length = found ? (strlen(color) + 1) : 0
  if(++g_msgChannel > 6 || g_msgChannel < 3)
    g_msgChannel = 3
  sc_msgChannel = g_msgChannel
  sc_end_pos = 1
  sc_start_pos = 0
  sc_x_pos = 0.65
  get_user_authid(id, authid, 31)
  if(id == 0) name = _T("SERVER")
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  userid = get_user_userid(id)

  replace_all(sc_scroll_msg[length], 383-length, "^"", "")
  new amx_chat_anonymous = get_cvarptr_num(g_cvar_amx_chat_anonymous)
  if(containi(cmd, "yy") == -1 || amx_chat_anonymous == 0 || (amx_chat_anonymous == 1 && get_cvarptr_num(g_cvar_amx_show_activity) == 2)) {
    format(sc_scroll_msg, 383, "%s :  %s", name, sc_scroll_msg[length])
    client_print(0, print_notify, "%s :  %s", name, sc_scroll_msg)
  }
  else {
    format(sc_scroll_msg, 383, sc_scroll_msg[length])
    client_print(0, print_notify, sc_scroll_msg)
  }

  sc_length = strlen(sc_scroll_msg)
  sc_timer = sc_length + 48
  set_task(SCROLL_SPEED, "scroll_msg", 456, "", 0, "a", sc_length + 48)

  log_amx("Chat: ^"%s<%d><%s><%s>^" %s ^"%s^"",name,userid,authid,ipaddress,cmd,sc_scroll_msg)
  log_message("^"%s<%d><%s><%s>^" triggered ^"%s^" (text ^"%s^") (color ^"%s^")",
    name,userid,authid,ipaddress,cmd,sc_scroll_msg,g_Colors[sc_a])
  return PLUGIN_HANDLED
}

public scroll_msg() {
  sc_timer -= 1
  new b = sc_start_pos, i = 0
  while(b < sc_end_pos) {
    sc_display_msg[i++] = sc_scroll_msg[b++]
  }

  sc_display_msg[i] = 0

  if(sc_end_pos < sc_length)
    ++sc_end_pos

  if(sc_x_pos > 0.35)
    sc_x_pos -= 0.0063
  else {
    ++sc_start_pos
    sc_x_pos = 0.35
  }
  set_hudmessage(g_Values[sc_a][0], g_Values[sc_a][1], g_Values[sc_a][2], sc_x_pos, 0.90, 0, SCROLL_SPEED, SCROLL_SPEED, 0.05, 0.05, sc_msgChannel)
  show_hudmessage(0, sc_display_msg)
}
