/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Commands:
 * amx_kick <name|#userid|authid> [reason] - kicks
 * amx_ban <name|#userid|authid|ip> [ip] <minutes> [reason] - bans
 * amx_unban <authid/ip> - unbans
 * amx_slay <name|#userid|authid> - slays
 * amx_slap <name|#userid|authid> [power] - slaps
 * amx_map <map> - changelevel
 * amx_extendmap <number of minutes> - extends map
 * amx_cvar <cvar> [value] - displays/changes a cvar value
 * amx_cfg <filename> - executes a config
 * amx_pause - pause or unpause the game
 * amx_leave <tag> [tag] [tag] [tag] - kicks non specified players
 * amx_name <name|#userid|authid> <new name> - changes player's name
 * amx_clcmd <name|#userid|authid> <command line> - executes a client command
 * amx_who - displays who is on server
 * amx_rcon <command line> - executes a server command
 * amx_showrcon <command line> - executes a server command
 * amx_showip [name|#userid|authid] - displays IP list
 *
 * Cvar:
 * amx_show_clcmds <0|1|2> - 0: disabled (don't show any client command)
 *                           1: show all client commands except for protected commands
 *                           2: show all client commands
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_KICK      ADMIN_KICK
#define FLAG_AMX_BAN       ADMIN_BAN
#define FLAG_AMX_UNBAN     ADMIN_UNBAN
#define FLAG_AMX_SLAY      ADMIN_SLAY
#define FLAG_AMX_SLAP      ADMIN_SLAY
#define FLAG_AMX_MAP       ADMIN_MAP
#define FLAG_AMX_EXTENDMAP ADMIN_CVAR
#define FLAG_AMX_CVAR      ADMIN_CVAR
#define FLAG_AMX_CFG       ADMIN_CFG
#define FLAG_AMX_PAUSE     ADMIN_CVAR
#define FLAG_AMX_LEAVE     ADMIN_KICK
#define FLAG_AMX_NAME      ADMIN_SLAY
#define FLAG_AMX_CLCMD     ADMIN_RCON
#define FLAG_AMX_WHO       ADMIN_ALL // Note: only admins can see the other admins
#define FLAG_AMX_RCON      ADMIN_RCON
#define FLAG_AMX_SHOWRCON  ADMIN_RCON
#define FLAG_AMX_SHOWIP    ADMIN_BAN

const ADMIN_TEMPBAN_MAXTIME = 120 // maximum number of minutes an admin can ban a player
                                  // if the admin does not have the ADMIN_PERMBAN flag

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>
#include <mysql>

const MAXRCONCVARS = 32

new g_cvarRcon[MAXRCONCVARS][32]
new g_cvarRconNum
new g_pauseCon
new Float:g_pausAble
new bool:g_Paused
new bool:g_CanPause
new const g_addCvar[] = "amx_cvar add %s"

new g_mysqlRunning

new g_cvar_amx_show_clcmds

new g_cvar_pausable
new g_cvar_rcon_password
new g_cvar_mp_timelimit
new g_cvar_sv_lan

public disable_natives_check() {
}

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("admincmd")
  register_plugin(_T("Admin Commands"),AMX_VERSION,"AMX Mod Team")
  g_mysqlRunning = (is_module_running("MySQL Access"))
  register_concmd("amx_kick","cmdKick",FLAG_AMX_KICK,_T("<name|#userid|authid> [reason] - kicks"))
  register_concmd("amx_ban","cmdBan",FLAG_AMX_BAN,_T("<name|#userid|authid|ip> [ip] <minutes> [reason] - bans"))
  register_concmd("amx_unban","cmdUnban",FLAG_AMX_UNBAN,_T("<authid/ip> - unbans"))
  register_concmd("amx_slay","cmdSlay",FLAG_AMX_SLAY,_T("<name|#userid|authid> - slays"))
  register_concmd("amx_slap","cmdSlap",FLAG_AMX_SLAP,_T("<name|#userid|authid> [power] - slaps"))
  register_concmd("amx_map","cmdMap",FLAG_AMX_MAP,_T("<map> - changelevel"))
  register_concmd("amx_extendmap","cmdExtendMap",FLAG_AMX_EXTENDMAP,_T("<number of minutes> - extends map"))
  register_concmd("amx_cvar","cmdCvar",FLAG_AMX_CVAR,_T("<cvar> [value] - displays/changes a cvar value"))
  register_concmd("amx_cfg","cmdCfg",FLAG_AMX_CFG,_T("<filename> - executes a config"))
  register_concmd("amx_pause","cmdPause",FLAG_AMX_PAUSE,_T("- pause or unpause the game"))
  register_concmd("amx_leave","cmdLeave",FLAG_AMX_LEAVE,_T("<tag> [tag] [tag] [tag] - kicks non specified players"))
  register_concmd("amx_name","cmdName",FLAG_AMX_NAME,_T("<name|#userid|authid> <new name> - changes player's name"))
  register_concmd("amx_clcmd","cmdClCmd",FLAG_AMX_CLCMD,_T("<name|#userid|authid> <command line> - executes a client command"))
  register_concmd("amx_who","cmdWho",FLAG_AMX_WHO,_T("- displays who is on server"))
  register_clcmd("amx_rcon","cmdRcon",FLAG_AMX_RCON,_T("<command line> - executes a server command"))
  register_clcmd("amx_showrcon","cmdShowRcon",FLAG_AMX_SHOWRCON,_T("<command line> - executes a server command"))
  register_concmd("amx_showip","cmdShowIp",FLAG_AMX_SHOWIP,_T("[name|#userid|authid] - displays IP list"))
  register_clcmd("pauseAck","cmdLBack")
  g_cvar_amx_show_clcmds = register_cvar("amx_show_clcmds","1")
}

public plugin_cfg() {
  g_cvar_pausable = get_cvar_pointer("pausable")
  g_cvar_rcon_password = get_cvar_pointer("rcon_password")
  g_cvar_mp_timelimit = get_cvar_pointer("mp_timelimit")
  g_cvar_sv_lan = get_cvar_pointer("sv_lan")

  // Cvars which can be changed only with rcon access
  server_cmd(g_addCvar, "amx_version")
  server_cmd(g_addCvar, "rcon_password")
  server_cmd(g_addCvar, "amx_mode")
  server_cmd(g_addCvar, "amx_password_field")
  server_cmd(g_addCvar, "amx_default_access")
  server_cmd(g_addCvar, "amx_realadmin_flags")
  server_cmd(g_addCvar, "amx_show_activity")
  server_cmd(g_addCvar, "amx_show_clcmds")
  server_cmd(g_addCvar, "amx_reservation")
  server_cmd(g_addCvar, "amx_log_to_hl_logs")
  server_cmd(g_addCvar, "amx_debug")
  server_cmd(g_addCvar, "amx_langdebug")
  server_cmd(g_addCvar, "csstats_rank")
  server_cmd(g_addCvar, "csstats_rankbots")
  server_cmd(g_addCvar, "csstats_pause")
  server_cmd(g_addCvar, "csstats_maxsize")
  server_cmd(g_addCvar, "csstats_reset")
  server_cmd(g_addCvar, "dodstats_rank")
  server_cmd(g_addCvar, "dodstats_rankbots")
  server_cmd(g_addCvar, "dodstats_pause")
  server_cmd(g_addCvar, "dodstats_maxsize")
  server_cmd(g_addCvar, "dodstats_reset")
}

bool:bIsProtectedCmd(const szCommand[]) {
  // Format: {command length (number of chars), "command name"}
  // You can modify or add the commands as you want.
  static const muCommandsList[][] = {
    {9, "developer"},
    {6, "unbind"},
    {6, "volume"},
    {10, "brightness"},
    {11, "sensitivity"},
    {9, "crosshair"},
    {5, "gamma"},
    {4, "hide"},
    {4, "draw"},
    {3, "cl_"},
    {4, "con_"},
    {4, "fps_"},
    {3, "gl_"},
    {4, "hud_"},
    {2, "m_"},
    {2, "r_"},
    {2, "s_"},
    {6, "voice_"},
    {4, "rate"},
    {2, "cd"},
    {1, ";"},
    //{4, "bind"},
    //{5, "alias"},
    {4, "exec"},
    {8, "writecfg"},
    {5, "retry"},
    {9, "reconnect"},
    {10, "disconnect"},
    {4, "quit"},
    {4, "exit"},
    {1, "+"},
    {1, "-"}
  }

  for(new i = 0; i < sizeof(muCommandsList); i++) {
    if(equali(szCommand, muCommandsList[i][1], muCommandsList[i][0]))
      return true
  }
  return false
}

public cmdKick(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 3)
  if(!player) return PLUGIN_HANDLED
  new authid[32], authid2[32], name[32], name2[32], ipaddress[24], ipaddress2[24], userid2, reason[70]
  get_user_authid(id, authid, 31)
  get_user_authid(player, authid2, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_name(player, name2, 31)
  get_user_ip(id, ipaddress, 23, 1)
  get_user_ip(player, ipaddress2, 23, 1)
  userid2 = get_user_userid(player)
  read_args(reason, 69)
  new length = strlen(arg)
  while(reason[++length] == ' ') {}
  replace_all(reason[length], 69-length, "^"", "")
  log_amx("Kick: ^"%s<%d><%s><%s>^" kick ^"%s<%d><%s><%s>^" (reason ^"%s^")",
    name,get_user_userid(id),authid,ipaddress,name2,userid2,authid2,ipaddress2,reason[length])
  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("kick %s."), name2)
  #else
  show_activity_color(id, name, _T("kick %s."), name2)
  #endif
  if(reason[length] && !is_user_bot(player)) {
    #if !defined NO_STEAM
    server_cmd("kick #%d %s", userid2, reason[length])
    #else
    client_cmd(player, "echo ^"Kicked: %s^";disconnect", reason[length])
    #endif
  }
  else {
    #if !defined NO_STEAM
    server_cmd("kick #%d", userid2)
    #else
    if(is_user_bot(player)) {
      server_cmd("kick #%d", userid2)
    }
    else {
      client_cmd(player, "echo ^"Kicked.^";disconnect")
    }
    #endif
  }
  console_print(id, _T("Player ^"%s^" kicked."), name2)
  return PLUGIN_HANDLED
}

hasImmunity(const authid[], isSteamid) {
  if(is_plugin_running("Admin Base") > 0) {
    new szFilename[64]
    new szText[256], szName[32], szPassword[32], szFlags[32], szAccess[32]
    build_path(szFilename, 63, "$configdir/users.ini")
    new a, pos = 0
    new parsedNum
    while((pos = read_file(szFilename, pos, szText, 255, a))) {
      if(!a || szText[0] == ';' || szText[0] == '/' || szText[0] == '#') continue
      parsedNum = parse(szText, szName, 31, szPassword, 31, szAccess, 31, szFlags, 31)
      if(parsedNum < 4) continue
      if(equal(authid, szName) && containi(szAccess, "a")!=-1 &&
      ((isSteamid == 1 && containi(szFlags, "c")!=-1) || (isSteamid == 0 && containi(szFlags, "d")!=-1))) {
        return 1
      }
    }
    return 0
  }
  else if(g_mysqlRunning && is_plugin_running("Admin Base for MySQL") > 0) {
    new host[64], user[32], pass[32], db[32], error[128], query[256]
    get_cvar_string("amx_mysql_host", host, 63)
    get_cvar_string("amx_mysql_user", user, 31)
    get_cvar_string("amx_mysql_pass", pass, 31)
    get_cvar_string("amx_mysql_db", db, 31)
    new mysql = mysql_connect(host, user, pass, db, error, 127)
    if(mysql < 1) {
      return 0
    }
    if(isSteamid == 1) {
      formatex(query, 255, "SELECT auth FROM admins WHERE auth='%s' AND access LIKE '%%a%%' AND flags LIKE '%%c%%'", authid)
    }
    else {
      formatex(query, 255, "SELECT auth FROM admins WHERE auth='%s' AND access LIKE '%%a%%' AND flags LIKE '%%d%%'", authid)
    }
    if(mysql_query(mysql, query) < 1) {
      mysql_close(mysql)
      return 0
    }
    if(mysql_nextrow(mysql) > 0) {
      mysql_close(mysql)
      return 1
    }
    mysql_close(mysql)
    return 0
  }
  return 0
}

getBanArgs(start,&minutes,reason[],rlen) {
  new str[64], argc = read_argc()
  if(argc < start)
    minutes = 0
  else {
    read_argv(start, str, 63)
    minutes = str_to_num(str)
  }
  reason[0] = '^0'
  if(argc > start) {
    new len = 0
    for(new i = start+1; i < argc && len < rlen; ++i) {
      read_argv(i, str, 63)
      len += formatex(reason[len], rlen, len ? " %s" : "%s", str)
    }
  }
}

public cmdBan(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32], len, i = 0, found = 0, myname[32], myauthid[32], myipaddress[24], kicked = 0
  new player, minutes = 0, reason[128], freason[128]
  new players[32], pnum, startarg
  new flags = get_user_flags(id)
  new isMaster = has_all_flags(id, "ly") ? 1 : 0

  get_players(players, pnum, "c")
  if(id == 0) myname = "SERVER"
  else get_user_name(id, myname, 31)
  get_user_authid(id, myauthid, 31)
  get_user_ip(id, myipaddress, 23, 1)

  len = read_argv(1, arg, 31)
  if(arg[0] != '#')
    player = find_player("bl", arg)
  else
    player = find_player("k", str_to_num(arg[1]))

  if(player) {
    if(arg[0] != '#' && player != find_player("blj", arg))
      console_print(id, _T("There are more than one player matching to your argument."))
    else {
      new authid[32], ipaddress[24], arg2[32]
      get_user_authid(player, authid, 31)
      get_user_ip(player, ipaddress, 23, 1)

      read_argv(2, arg2, 31)
      if(equali(arg2, "ip"))
        startarg = 3
      else
        startarg = 2

      if(startarg == 2 && equal(authid,"HLTV")) {
        console_print(id, _T("Sorry, you can't ban HLTV proxies! Use ^"sv_proxies^" (server Cvar) instead."))
        return PLUGIN_HANDLED
      }

      get_user_name(player, arg, 31)

      if((player != id) && (isMaster == 0) && (get_user_flags(player) & ADMIN_IMMUNITY)) {
        console_print(id, _T("Player ^"%s^" has immunity."), arg)
        return PLUGIN_HANDLED
      }

      getBanArgs(startarg, minutes, reason, 127)
      if((minutes == 0 || minutes > ADMIN_TEMPBAN_MAXTIME) && ((flags & ADMIN_PERMBAN) == 0)) {
        console_print(id, _T("You can only ban for less than %d minutes."), ADMIN_TEMPBAN_MAXTIME)
        return PLUGIN_HANDLED
      }

      if(reason[0])
        formatex(freason, 127, _T("Banned: %s",player), reason)
      else
        copy(freason, 127, _T("Banned",player))

      found = 1
      --kicked

      if(get_cvarptr_num(g_cvar_sv_lan) || startarg == 3
      || equal("4294967295",authid)
      || equal("STEAM_ID_LAN",authid)
      || equali("VALVE_ID_LAN", authid)) {
        new address[32], address2[32]
        get_user_ip(player, address, 31, 1)
        for(i = 0; i < pnum; ++i) {
          if(players[i] != player) {
            get_user_ip(players[i], address2, 31, 1)
            if(equal(address, address2)) {
              #if !defined NO_STEAM
              server_cmd("kick #%d %s", get_user_userid(players[i]), freason)
              #else
              client_cmd(players[i], "echo ^"%s^";disconnect", freason)
              #endif
              ++kicked
            }
          }
        }
        #if !defined NO_STEAM
        server_cmd("kick #%d %s", get_user_userid(player), freason)
        #else
        client_cmd(player, "echo ^"%s^";disconnect", freason)
        #endif
        server_cmd("addip %d ^"%s^";wait;writeip", minutes, address)
      }
      else {
        new authid2[32]
        get_user_authid(player, authid, 31)
        for(i = 0; i < pnum; ++i) {
          if(players[i] != player) {
            get_user_authid(players[i], authid2, 31)
            if(equal(authid, authid2)) {
              server_cmd("kick #%d %s", get_user_userid(players[i]), freason)
              ++kicked
            }
          }
        }
        server_cmd("kick #%d %s", get_user_userid(player), freason)
        server_cmd("banid %d %s;writeid", minutes, authid)
      }

      if(found) {
        log_amx("Ban: ^"%s<%d><%s><%s>^" ban and kick ^"%s<%d><%s><%s>^" (minutes ^"%d^") (reason ^"%s^")",
          myname,get_user_userid(id),myauthid,myipaddress,arg,player,authid,ipaddress,minutes,reason)
        console_print(id, _T("Player ^"%s^" kicked and banned."), arg)
        if(kicked)
          console_print(id, _T("%d other player(s) with same %s kicked and banned as well."), kicked,
            (get_cvarptr_num(g_cvar_sv_lan) || startarg == 3) ? _T("IP") : _T("AuthID") )
      }
    }
  }
  else {
    strtoupper(arg)
    if(contain(arg, ".") == -1) {
      startarg = 2
      if(len == 7 && equali(arg, "STEAM_", 6)) {
        new str[16]
        read_argv(3, str, 15)
        add(arg, 31, ":")
        add(arg, 31, str)
        read_argv(5, str, 15)
        add(arg, 31, ":")
        len = add(arg, 31, str)
        startarg = 6
      }
      if(len > 10 && equali(arg, "STEAM_", 6) && arg[7] == ':' && arg[9] == ':' && str_to_num(arg[10])) {
        if((isMaster == 0) && (hasImmunity(arg, 1) == 1)) {
          console_print(id, _T("Player ^"%s^" has immunity."), arg)
          return PLUGIN_HANDLED
        }

        new authid[32]
        getBanArgs(startarg, minutes, reason, 127)

        if((minutes == 0 || minutes > ADMIN_TEMPBAN_MAXTIME) && ((flags & ADMIN_PERMBAN) == 0)) {
          console_print(id, _T("You can only ban for less than %d minutes."), ADMIN_TEMPBAN_MAXTIME)
          return PLUGIN_HANDLED
        }

        found = 1

        for(i = 0; i < pnum; ++i) {
          player = players[i]
          get_user_authid(player, authid, 31)
          if(equal(authid, arg)) {
            if((player != id) && (isMaster == 0) && (get_user_flags(player) & ADMIN_IMMUNITY)) {
              console_print(id, _T("Player ^"%s^" has immunity."), arg)
              return PLUGIN_HANDLED
            }
            freason[0] = '^0'
            if(reason[0])
              formatex(freason, 127, _T("Banned: %s",player), reason)
            else
              copy(freason, 127, _T("Banned",player))
            server_cmd("kick #%d %s", get_user_userid(player), freason)
            ++kicked
          }
        }
        
        log_amx("Ban: ^"%s<%d><%s><%s>^" ban ^"%s^" (minutes ^"%d^") (reason ^"%s^")",
          myname,get_user_userid(id),myauthid,myipaddress,arg,minutes,reason)

        server_cmd("banid %d %s;writeid", minutes, arg)

        console_print(id, _T("AuthID ^"%s^" added to ban list."), arg)
        if(kicked)
          console_print(id, _T("%d player(s) with this AuthID kicked."), kicked)
      }
    }
    else {
      new dots = 0
      while(isdigit(arg[i]) || arg[i] == '.')
        if(arg[i++] == '.')
          ++dots
      if(i == len && dots == 3) {
        if((isMaster == 0) && (hasImmunity(arg, 0) == 1)) {
          console_print(id, _T("Player ^"%s^" has immunity."), arg)
          return PLUGIN_HANDLED
        }
        
        new address[32]
        getBanArgs(2, minutes, reason, 127)
        
        if((minutes == 0 || minutes > ADMIN_TEMPBAN_MAXTIME) && ((flags & ADMIN_PERMBAN) == 0)) {
          console_print(id, _T("You can only ban for less than %d minutes."), ADMIN_TEMPBAN_MAXTIME)
          return PLUGIN_HANDLED
        }
      
        found = 1

        for(i = 0; i < pnum; ++i) {
          player = players[i]
          get_user_ip(player, address, 31, 1)
          if(equal(address, arg)) {
            if((player != id) && (isMaster == 0) && (get_user_flags(player) & ADMIN_IMMUNITY)) {
              console_print(id, _T("Player ^"%s^" has immunity."), arg)
              return PLUGIN_HANDLED
            }
            #if !defined NO_STEAM
            server_cmd("kick #%d %s", get_user_userid(player), freason)
            #else
            client_cmd(player, "echo ^"%s^";disconnect", freason)
            #endif
            ++kicked
          }
        }
        
        log_amx("Ban: ^"%s<%d><%s><%s>^" ban ^"%s^" (minutes ^"%d^") (reason ^"%s^")",
          myname,get_user_userid(id),myauthid,myipaddress,arg,minutes,reason)

        server_cmd("addip %d ^"%s^";wait;writeip", minutes, arg)

        console_print(id, _T("IP ^"%s^" added to ban list."), arg)
        if(kicked)
          console_print(id, _T("%d player(s) with this IP kicked."), kicked)
      }
    }
  }

  if(!found) {
    console_print(id, _T("Could not find specified player/AuthID/IP."))
    console_print(id, _T("Usage: amx_ban <name|#userid|authid|ip> [ip] <minutes> [reason]"))
  }
  else {
    if(minutes)
      #if !defined COLORED_ACTIVITY
      show_activity(id, myname, _T("ban ^"%s^" for %d min."), arg, minutes)
      #else
      show_activity_color(id, myname, _T("ban ^"%s^" for %d min."), arg, minutes)
      #endif
    else
      #if !defined COLORED_ACTIVITY
      show_activity(id, myname, _T("ban ^"%s^" permanently."), arg)
      #else
      show_activity_color(id, myname, _T("ban ^"%s^" permanently."), arg)
      #endif
  }

  return PLUGIN_HANDLED
}

public cmdUnban(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32], authid[32], name[32], ipaddress[24]
  read_args(arg, 31)
  trim_imp(arg)
  if(contain(arg, ".") != -1) {
    server_cmd("removeip ^"%s^";writeip", arg)
    console_print(id, _T("IP ^"%s^" removed from ban list."), arg)
  }
  else {
    server_cmd("removeid %s;writeid", arg)
    console_print(id, _T("AuthID ^"%s^" removed from ban list."), arg)
  }

  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_authid(id, authid, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" unban ^"%s^"",
    name,get_user_userid(id),authid,ipaddress,arg)

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("unban ^"%s^"."), arg)
  #else
  show_activity_color(id, name, _T("unban ^"%s^"."), arg)
  #endif

  return PLUGIN_HANDLED
}

public cmdSlay(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 7)
  if(!player) return PLUGIN_HANDLED
  user_kill(player, 1)
  new authid[32], name[32], authid2[32], name2[32], ipaddress[24], ipaddress2[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  get_user_authid(player, authid2, 31)
  get_user_name(player, name2, 31)
  get_user_ip(player, ipaddress2, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" slay ^"%s<%d><%s><%s>^"",
    name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2)

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("slay %s."), name2)
  #else
  show_activity_color(id, name, _T("slay %s."), name2)
  #endif

  console_print(id, _T("Player ^"%s^" slayed."), name2)
  return PLUGIN_HANDLED
}

public cmdSlap(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 7)
  if(!player) return PLUGIN_HANDLED
  new spower[32], authid[32], name[32], authid2[32], name2[32], ipaddress[24], ipaddress2[24]
  read_argv(2, spower, 31)
  new damage = str_to_num(spower)
  user_slap(player, damage)
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  get_user_authid(player, authid2, 31)
  get_user_name(player, name2, 31)
  get_user_ip(player, ipaddress2, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" slap with %d damage ^"%s<%d><%s><%s>^"",
    name,get_user_userid(id),authid,ipaddress,damage,name2,get_user_userid(player),authid2,ipaddress2)

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("slap %s with %d damage."), name2, damage)
  #else
  show_activity_color(id, name, _T("slap %s with %d damage."), name2, damage)
  #endif

  console_print(id, _T("Player ^"%s^" slapped with %d damage."), name2, damage)
  return PLUGIN_HANDLED
}

public chMap(const map[]) {
  server_cmd("changelevel %s", map)
}

public cmdMap(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32]
  new arglen = read_argv(1, arg, 31)
  if(!is_map_valid(arg)) {
    console_print(id, _T("Map with that name not found or map is invalid."))
    return PLUGIN_HANDLED
  }
  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("changelevel ^"%s^"."), arg)
  #else
  show_activity_color(id, name, _T("changelevel ^"%s^"."), arg)
  #endif
  new modName[8]
  get_modname(modName, 7)
  if(!equal(modName, "zp")) {
    message_begin(MSG_ALL, SVC_INTERMISSION)
    message_end()
  }
  log_amx("Cmd: ^"%s<%d><%s><%s>^" changelevel ^"%s^"",name,get_user_userid(id),authid,ipaddress,arg)
  set_task(2.0, "chMap", 55119944, arg, arglen+1)
  return PLUGIN_HANDLED
}

public cmdExtendMap(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new mns = str_to_num(arg)
  if(mns <= 0) {
    return PLUGIN_HANDLED
  }
  new mp_timelimit = get_cvarptr_num(g_cvar_mp_timelimit)
  if(mp_timelimit == 0) {
    console_print(id, _T("You can't extend map when the map timelimit is infinite (^"mp_timelimit^" cvar to 0)."))
    return PLUGIN_HANDLED
  }

  new mapname[32]
  get_mapname(mapname, 31)
  set_cvarptr_num(g_cvar_mp_timelimit, mp_timelimit + mns)

  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("ExtendMap: ^"%s<%d><%s><%s>^" extended map ^"%s^" for %d minutes.",
    name,get_user_userid(id),authid,ipaddress,mapname,mns)

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("extend map for %d minutes."), mns)
  #else
  show_activity_color(id, name, _T("extend map for %d minutes."), mns)
  #endif

  console_print(id, _T("Map ^"%s^" has been extended for %d minutes."), mapname, mns)
  return PLUGIN_HANDLED
}

onlyRcon(const name[]) {
  for(new a = 0; a < g_cvarRconNum; ++a)
    if(equali(g_cvarRcon[a], name))
      return 1
  return 0
}

public cmdCvar(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[32], arg2[64], cvar_pointer
  new hasRcon = (get_user_flags(id) & ADMIN_RCON) ? 1 : 0
  read_argv(1, arg, 31)
  read_argv(2, arg2, 63)
  if(equal(arg, "add") && (hasRcon == 1)) {
    if((cvar_pointer = get_cvar_pointer(arg2)) != 0) {
      if(g_cvarRconNum < MAXRCONCVARS) {
        copy(g_cvarRcon[g_cvarRconNum++], 31, arg2)
        if(id != 0)
          console_print(id, _T("Cvar ^"%s^" added to rcon access."), arg2)
      }
      else
        console_print(id, _T("Can't add more cvars for rcon access!"))
    }
    return PLUGIN_HANDLED
  }
  if((cvar_pointer = get_cvar_pointer(arg)) == 0) {
    console_print(id, _T("Unknown cvar: %s"), arg)
    return PLUGIN_HANDLED
  }
  if(onlyRcon(arg) && (hasRcon == 0)) {
    console_print(id, _T("You have no access to that cvar."))
    return PLUGIN_HANDLED
  }
  else if(equal(arg, "sv_password") && !(get_user_flags(id) & ADMIN_PASSWORD)) {
    console_print(id, _T("You have no access to that cvar."))
    return PLUGIN_HANDLED
  }
  if(read_argc() < 3) {
    get_cvarptr_string(cvar_pointer, arg2, 63)
    console_print(id, _T("Cvar ^"%s^" is ^"%s^"."), arg, arg2)
    return PLUGIN_HANDLED
  }
  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" set cvar (name ^"%s^") (value ^"%s^")",
    name,get_user_userid(id),authid,ipaddress,arg,arg2)
  set_cvar_string(arg, arg2)

  if(equal(arg, "rcon_password") || equal(arg, "sv_password"))
    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("set cvar ^"%s^" to ^"%s^"."), arg, _T("*** PROTECTED ***"))
    #else
    show_activity_color(id, name, _T("set cvar ^"%s^" to ^"%s^"."), arg, _T("*** PROTECTED ***"))
    #endif
  else
    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("set cvar ^"%s^" to ^"%s^"."), arg, arg2)
    #else
    show_activity_color(id, name, _T("set cvar ^"%s^" to ^"%s^"."), arg, arg2)
    #endif

  console_print(id, _T("Cvar ^"%s^" changed to ^"%s^"."), arg, arg2)
  return PLUGIN_HANDLED
}

public cmdCfg(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[128]
  read_argv(1, arg, 127)
  if(!file_exists(arg)) {
    console_print(id, _T("File ^"%s^" not found."), arg)
    return PLUGIN_HANDLED
  }
  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" execute cfg (file ^"%s^")",
    name,get_user_userid(id),authid,ipaddress,arg)
  console_print(id, _T("Executing file ^"%s^"."), arg)
  server_cmd("exec %s", arg)
  server_exec()

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("execute config ^"%s^"."), arg)
  #else
  show_activity_color(id, name, _T("execute config ^"%s^"."), arg)
  #endif

  return PLUGIN_HANDLED
}

public cmdLBack() {
  if(!g_CanPause)
    return PLUGIN_CONTINUE
  g_CanPause = false
  set_cvar_float("pausable", g_pausAble)
  console_print(g_pauseCon, _T("Server %s."), g_Paused ? _T("unpaused") : _T("paused"))
  g_Paused = !g_Paused
  return PLUGIN_HANDLED
}

public cmdPause(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new authid[32], name[32], ipaddress[24], slayer = id
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  g_pausAble = get_cvarptr_float(g_cvar_pausable)
  if(!slayer) slayer = find_player("h")
  if(!slayer) {
    console_print(id, _T("Server was unable to pause the game. Real players on server are needed."))
    return PLUGIN_HANDLED
  }
  set_cvar_float("pausable", 1.0)
  g_CanPause = true
  client_cmd(slayer, "pause;pauseAck")
  log_amx("Cmd: ^"%s<%d><%s><%s>^" %s server",name,get_user_userid(id),authid,ipaddress,g_Paused ? "unpause" : "pause")
  console_print(id, _T("Server proceed %s."), g_Paused ? _T("unpausing") : _T("pausing"))

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("%s server."), g_Paused ? _T("unpause") : _T("pause"))
  #else
  show_activity_color(id, name, _T("%s server."), g_Paused ? _T("unpause") : _T("pause"))
  #endif

  g_pauseCon = id
  return PLUGIN_HANDLED
}

hasTag(const name[],const tags[4][32],tagsNum) {
  for(new a = 0; a < tagsNum; ++a)
    if(contain(name, tags[a]) != -1)
      return a
  return -1
}

public cmdLeave(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new argnum = read_argc()
  new ltags[4][32]
  new ltagsnum = 0
  for(new a = 1; a < 5; ++a) {
    if(a < argnum)
      read_argv(a, ltags[ltagsnum++], 31)
    else
      ltags[ltagsnum++][0] = '^0'
  }
  new nick[32], ires, pnum = get_maxplayers(), count = 0
  for(new b = 1; b <= pnum; ++b) {
    if(!is_user_connected(b) && !is_user_connecting(b)) continue
    if((b == id) && is_user_connected(id)) continue
    get_user_name(b, nick, 31)
    ires = hasTag(nick, ltags, ltagsnum)
    if(ires != -1) {
      console_print(id, _T("Skipping %s (matching ^"%s^")."), nick, ltags[ires])
      continue
    }
    if( (get_user_flags(b) & ADMIN_IMMUNITY) && (b != id) && !(get_user_flags(id) & ADMIN_SUPREME) ) {
      console_print(id, _T("Skipping %s (immunity)."), nick)
      continue
    }
    console_print(id, _T("Kicking %s."), nick)
    if(is_user_bot(b))
      server_cmd("kick #%d", get_user_userid(b))
    else {
      #if !defined NO_STEAM
      server_cmd("kick #%d %s", get_user_userid(b), _T("You have been dropped because admin has left only specified group of players."))
      #else
      client_cmd(b, "echo ^"%s^";disconnect", _T("You have been dropped because admin has left only specified group of players."))
      #endif
    }
    ++count
  }
  console_print(id, _T("Kicked %d players."), count)
  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Kick: ^"%s<%d><%s><%s>^" leave some group (tag1 ^"%s^") (tag2 ^"%s^") (tag3 ^"%s^") (tag4 ^"%s^")",
    name,get_user_userid(id),authid,ipaddress,ltags[0],ltags[1],ltags[2],ltags[3])

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("leave ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), ltags[0], ltags[1], ltags[2], ltags[3])
  #else
  show_activity_color(id, name, _T("leave ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), ltags[0], ltags[1], ltags[2], ltags[3])
  #endif

  return PLUGIN_HANDLED
}

public cmdName(id,level,cid) {
  if(!cmd_access(id,level,cid,3))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 3)
  if(!player) return PLUGIN_HANDLED
  new newname[70], authid[32], name[32], authid2[32], name2[32], ipaddress[24], ipaddress2[24]
  read_args(newname, 69)
  new length = strlen(arg) + 1
  replace_all(newname[length], 69-length, "^"", "")
  replace_all(newname[length], 69-length, " ", "%")
  set_user_info(player, "name", newname[length])
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  get_user_authid(player, authid2, 31)
  get_user_name(player, name2, 31)
  get_user_ip(player, ipaddress2, 23, 1)
  replace_all(newname[length], 69-length, "%", " ")
  log_amx("Cmd: ^"%s<%d><%s><%s>^" change name of ^"%s<%d><%s><%s>^" to ^"%s^"",
    name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,newname[length])

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("change name of %s to ^"%s^"."), name2, newname[length])
  #else
  show_activity_color(id, name, _T("change name of %s to ^"%s^"."), name2, newname[length])
  #endif

  console_print(id, _T("Changed name of %s to ^"%s^"."), name2, newname[length])
  return PLUGIN_HANDLED
}

public cmdClCmd(id,level,cid) {
  if(!cmd_access(id,level,cid,3))
    return PLUGIN_HANDLED

  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 11)
  if(!player) return PLUGIN_HANDLED
  new args[128], authid[32], name[32], authid2[32], name2[32], ipaddress[24], ipaddress2[24]
  read_args(args, 127)
  new start = strlen(arg) + 1
  trim_imp(args[start], _, true)
  replace(args[start], 127-start, " ", "^t")
  client_cmd(player, args[start])
  replace(args[start], 127-start, "^t", " ")
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  get_user_authid(player, authid2, 31)
  get_user_name(player, name2, 31)
  get_user_ip(player, ipaddress2, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" execute client command ^"%s^" to ^"%s<%d><%s><%s>^"",
    name,get_user_userid(id),authid,ipaddress,args[start],name2,get_user_userid(player),authid2,ipaddress2)

  new amx_show_clcmds = get_cvarptr_num(g_cvar_amx_show_clcmds)

  if(amx_show_clcmds == 0 || amx_show_clcmds == 1 && bIsProtectedCmd(args[start]))
    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("execute client command ^"%s^" to %s."), _T("*** PROTECTED ***"), name2)
    #else
    show_activity_color(id, name, _T("execute client command ^"%s^" to %s."), _T("*** PROTECTED ***"), name2)
    #endif
  else
    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("execute client command ^"%s^" to %s."), args[start], name2)
    #else
    show_activity_color(id, name, _T("execute client command ^"%s^" to %s."), args[start], name2)
    #endif

  console_print(id, _T("Client command ^"%s^" executed to %s."), args[start], name2)
  return PLUGIN_HANDLED
}

public cmdWho(id) {
  new players[32], inum, player, authid[32], name[32], ipaddress[24], flags, sflags[32]
  get_players(players, inum)
  if(is_user_admin(id) || id == (is_dedicated_server() ? 0 : 1)) {
    console_print(id, "^n%s:^n #  %-26.25s %-20.19s %-8s %-6.5s %-6.5s %s",
      _T("Players on server"), _T("nick"), _T("authid"), _T("userid"), _T("imm"), _T("res"), _T("access"))
    for(new a = 0; a < inum; ++a) {
      player = players[a]
      get_user_authid(player, authid, 31)
      get_user_name(player, name, 31)
      flags = get_user_flags(player)
      get_flags(flags, sflags, 31)
      console_print(id, "%2d  %-26.25s %-20.19s %-8d %-6.5s %-6.5s %s", player, name, authid,
        get_user_userid(player), (flags&ADMIN_IMMUNITY) ? _T("yes") : _T("no"),
        (flags & ADMIN_RESERVATION) ? _T("yes") : _T("no"), sflags)
    }
  }
  else {
    console_print(id, "^n%s:^n #  %-26.25s %-20.19s %-8s",
      _T("Players on server"), _T("nick"), _T("authid"), _T("userid"))
    for(new a = 0; a < inum; ++a) {
      player = players[a]
      get_user_authid(player, authid, 31)
      get_user_name(player, name, 31)
      console_print(id, "%2d  %-26.25s %-20.19s %-8d", player, name, authid, get_user_userid(player))
    }
  }
  console_print(id, _T("Total %d"), inum)
  if(id) {
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)
    log_amx("Cmd: ^"%s<%d><%s><%s>^" ask for players list",name,get_user_userid(id),authid,ipaddress)
  }
  return PLUGIN_HANDLED
}

public cmdRcon(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new arg[128], authid[32], name[32], ipaddress[24]
  read_args(arg, 127)

  if(containi(arg, "rcon_password") != -1) {
    console_print(id, _T("You have no access to that cvar."))
    return PLUGIN_HANDLED
  }

  get_user_authid(id, authid, 31)
  get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" server console (cmdline ^"%s^")",
    name,get_user_userid(id),authid,ipaddress,arg)
  console_print(id, _T("Command line ^"%s^" sent to server console."), arg)
  server_cmd("%s", arg)
  return PLUGIN_HANDLED
}

public cmdShowRcon(id, level, cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new password[64]
  get_cvarptr_string(g_cvar_rcon_password, password, 63)
  if(!password[0]) {
    cmdRcon(id, level, cid)
  }
  else {
    new args[128]
    read_args(args, 127)
    client_cmd(id, "rcon_password %s", password)
    client_cmd(id, "rcon %s", args)
  }
  return PLUGIN_HANDLED
}

public cmdShowIp(id, level, cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED
    
  new player = 0
  if(read_argc() == 2) {
    new arg[32]
    read_argv(1, arg, 31)
    player = cmd_target(id, arg, 11)
    if(!player) return PLUGIN_HANDLED
  }
  new name[32], ipaddress[24]
  if(player) {
    console_print(id, "^n #  %-33.32s %-26.25s", _T("nick"), _T("ip"))
    get_user_ip(player, ipaddress, 23)
    get_user_name(player, name, 31)
    console_print(id, "%2d  %-33.32s %-26.25s", player, name, ipaddress)
  }
  else {
    new players[32], inum
    get_players(players, inum, "c")
    console_print(id, "^n #  %-33.32s %-26.25s", _T("nick"), _T("ip"))
    for(new a = 0; a < inum; ++a) {
      player = players[a]
      get_user_ip(player, ipaddress, 23)
      get_user_name(player, name, 31)
      console_print(id, "%2d  %-33.32s %-26.25s", player, name, ipaddress)
    }
  }
  if(id) {
    new authid[32]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)
    log_amx("Cmd: ^"%s<%d><%s><%s>^" ask for players' IP list",name,get_user_userid(id),authid,ipaddress)
  }
  return PLUGIN_HANDLED
}
