/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Vote commands (+ examples):
 *
 * I. Vote for changing the level:
 * amx_votemap <map> [map] [map] [map]
 *
 * Examples:
 * amx_votemap "de_dust"
 * amx_votemap "de_dust" "de_dust2" "cs_italy"
 *
 * II. Vote for kicking the player:
 * amx_votekick <name|#userid|authid>
 *
 * Example:
 * amx_votekick "Player"
 *
 * III. Vote for banning and kicking the player for 30 minutes:
 * amx_voteban <name|#userid|authid>
 *
 * Example:
 * amx_voteban "Player"
 *
 * IV. Custom voting:
 * amx_vote <question> <answer 1> <answer 2> [answer 3] [answer 4]
 *
 * Examples:
 * amx_vote "sv_restart" "5" "0" 
 * // this will change the value of the cvar if you accept the result
 * // (You need the "g" admin flag)
 *
 * amx_vote "mp_timelimit" "20" "30" "40" "50"
 * // this will change the value of the cvar if you accept the result
 * // (You need the "g" admin flag)
 *
 * amx_vote "csdm_enable" "1" "0"
 * // this will enable/disable CSDM if you accept the result
 * // (You need the admin flag required by csdm_enable)
 *
 * amx_vote "are you hungry?" "yes" "no" "maybe"
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_VOTEMAP    ADMIN_VOTE
#define FLAG_AMX_VOTEKICK   ADMIN_VOTE
#define FLAG_AMX_VOTEBAN    ADMIN_VOTE
#define FLAG_AMX_VOTE       ADMIN_VOTE
#define FLAG_AMX_CANCELVOTE ADMIN_VOTE

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const Float:CHECK_MENU_TIME = 0.5

new g_Answer[128]
const OPTIONNAME_EXTEND = 16 // Len to add for customvote quest/answer(s)...
new g_optionName[4][32+OPTIONNAME_EXTEND]
new g_voteCount[4]
new g_validMaps
new g_yesNoVote
new g_coloredMenus
new g_voteCaller
new g_bHasVoted
new g_Execute[256]
new g_execLen
new g_menukeys
new g_menumsg[33][256]
new g_menutitle[16]

new const g_alreadyVoting[] = "There is already one voting..."
new const g_notAllowed[]    = "Voting not allowed at this time."
new const g_votingStarted[] = "Voting has started..."
new const g_resultRef[]     = "Result refused."
new const g_resultAcpt[]    = "Result accepted."
new const g_votingFailed[]  = "Voting failed"
new const g_votingSuccess[] = "Voting successful"

new bool:g_execResult
new Float:g_voteRatio
new g_maxplayers

new menuid_changemap, menuid_choosemap, menuid_kick, menuid_ban, menuid_vote

new g_cvar_amx_votekick_ratio
new g_cvar_amx_voteban_ratio
new g_cvar_amx_votemap_ratio
new g_cvar_amx_vote_ratio
new g_cvar_amx_vote_time
new g_cvar_amx_vote_delay
new g_cvar_amx_vote_answers
new g_cvar_amx_last_voting

const TASK_check_menu = 1467851
const TASK_checkVotes = 99889988
const TASK_autoRefuse = 4545454

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("adminvote")
  register_plugin(_T("Admin Votes"), AMX_VERSION, "AMX Mod Team")

  menuid_changemap = register_menuid("Change map to ")
  menuid_choosemap = register_menuid("Choose map: ")
  menuid_kick = register_menuid("Kick ")
  menuid_ban = register_menuid("Ban ")
  menuid_vote = register_menuid("Vote: ")
  register_menucmd(menuid_changemap,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_choosemap,(1<<0)|(1<<1)|(1<<2)|(1<<3)|(1<<9),"voteCount")
  register_menucmd(menuid_kick,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_ban,(1<<0)|(1<<1),"voteCount")
  register_menucmd(menuid_vote,(1<<0)|(1<<1)|(1<<2)|(1<<3),"voteCount")
  register_menucmd(register_menuid("The result: "),(1<<0)|(1<<1),"actionResult")

  register_concmd("amx_votemap","cmdVoteMap",FLAG_AMX_VOTEMAP,_T("<map> [map] [map] [map] - starts a votemap"))
  register_concmd("amx_votekick","cmdVoteKickBan",FLAG_AMX_VOTEKICK,_T("<name|#userid|authid> - starts a votekick"))
  register_concmd("amx_voteban","cmdVoteKickBan",FLAG_AMX_VOTEBAN,_T("<name|#userid|authid> [ip] - starts a voteban"))
  register_concmd("amx_vote","cmdVote",FLAG_AMX_VOTE,_T("<question> <answer #1> <answer #2> [answer #3] [answer #4] - starts a custom vote"))
  register_concmd("amx_cancelvote","cmdCancelVote",FLAG_AMX_CANCELVOTE,_T("- cancels last vote"))

  g_cvar_amx_votekick_ratio = get_cvar_pointer("amx_votekick_ratio")
  g_cvar_amx_voteban_ratio = get_cvar_pointer("amx_voteban_ratio")
  g_cvar_amx_votemap_ratio = get_cvar_pointer("amx_votemap_ratio")
  g_cvar_amx_vote_ratio = get_cvar_pointer("amx_vote_ratio")
  g_cvar_amx_vote_time = get_cvar_pointer("amx_vote_time")
  g_cvar_amx_vote_delay = get_cvar_pointer("amx_vote_delay")
  g_cvar_amx_vote_answers = get_cvar_pointer("amx_vote_answers")
  g_cvar_amx_last_voting = get_cvar_pointer("amx_last_voting")

  g_coloredMenus = colored_menus()
  g_maxplayers = get_maxplayers()
}

canChangeCvar(id, const cvar[]) {
  if(equali(cvar, "amx_version")
  || equali(cvar, "amx_last_voting"))
    return 0
  if(!(get_user_flags(id) & ADMIN_RCON) &&
  (  equali(cvar, "sv_cheats")
  || equali(cvar, "sv_password")
  || equali(cvar, "rcon_password")
  || equali(cvar, "amx_mode")
  || equali(cvar, "amx_password_field")
  || equali(cvar, "amx_default_access")
  || equali(cvar, "amx_realadmin_flags")
  || equali(cvar, "amx_show_activity")
  || equali(cvar, "amx_show_clcmds")
  || equali(cvar, "amx_reservation")
  || equali(cvar, "amx_log_to_hl_logs")
  || equali(cvar, "amx_debug")
  || equali(cvar, "amx_langdebug")
  || equali(cvar, "csstats_rank")
  || equali(cvar, "csstats_rankbots")
  || equali(cvar, "csstats_pause")
  || equali(cvar, "csstats_maxsize")
  || equali(cvar, "csstats_reset")
  || equali(cvar, "dodstats_rank")
  || equali(cvar, "dodstats_rankbots")
  || equali(cvar, "dodstats_pause")
  || equali(cvar, "dodstats_maxsize")
  || equali(cvar, "dodstats_reset")))
    return 0
  return 1
}

public check_menu(param[]) {
  new iter = param[0]
  new Float:vote_time = float(param[2]) + 2.0 - (iter * CHECK_MENU_TIME)
  new menutime = floatround(vote_time, floatround_floor)
  if(menutime > 0) {
    new players[32], numplayers
    new menuid, keys, player
    get_players(players, numplayers, "c")
    for(numplayers--; numplayers >= 0; numplayers--) {
      player = players[numplayers]
      if(g_bHasVoted & (1<<(player&31))) continue
      get_user_menu(player, menuid, keys)
      if(keys == 0 || menuid < 0) {
        show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
      }
      else {
        if(menuid != param[1]) {
          if(menutime <= 6) {
            show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
          }
        }
        else if(player == g_voteCaller) {
          show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
        }
      }
    }
    ++param[0]
    set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 3)
  }
}

public cmdCancelVote(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  if(task_exists(TASK_checkVotes, 1)) {
    if(is_user_connected(id)) {
      new authid[32], name[32], ipaddress[24]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)
      get_user_ip(id, ipaddress, 23, 1)

      log_amx("Vote: ^"%s<%d><%s><%s>^" cancel vote session",name,get_user_userid(id),authid,ipaddress)
      #if !defined COLORED_ACTIVITY
      show_activity(id, name, _T("cancel vote."))
      #else
      show_activity_color(id, name, _T("cancel vote."))
      #endif
    }

    console_print(id, _T("Voting canceled."))
    client_print(0, print_chat, _T("Voting canceled."))
    remove_task(TASK_checkVotes, 1)
    set_cvarptr_float(g_cvar_amx_last_voting, get_gametime())

    remove_task(TASK_check_menu)
    new menuid, keys
    for(new i = 1; i <= g_maxplayers; ++i) {
      g_bHasVoted |= (1<<(i&31))
      if(!is_user_connected(i)) continue
      menuid = 0
      keys = 0
      get_user_menu(i, menuid, keys)
      if(menuid == menuid_changemap
      || menuid == menuid_choosemap
      || menuid == menuid_kick
      || menuid == menuid_ban
      || menuid == menuid_vote) {
        show_menu(i, 0, "NULL", 0, "NULL")
      }
    }
  }
  else {
    console_print(id, _T("There is no voting to cancel or the vote session can't be canceled with that command."))
  }
  return PLUGIN_HANDLED
}

public delayedExec(const cmd[]) {
  server_cmd("%s", cmd)
}

public autoRefuse() {
  log_amx("Vote: %s",g_resultRef)
  client_print(0, print_chat, _T(g_resultRef))
}

public actionResult(id,key) {
  remove_task(TASK_autoRefuse)

  switch(key) {
    case 0: {
      set_task(2.0, "delayedExec", 55119977, g_Execute, g_execLen)
      log_amx("Vote: %s", g_resultAcpt) 
      client_print(0, print_chat, _T(g_resultAcpt))
    }
    case 1:
      autoRefuse()
  }
  return PLUGIN_HANDLED
}

public checkVotes() {
  remove_task(TASK_check_menu)
  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted |= (1<<(i&31))
  }

  new best = 0
  if(!g_yesNoVote) {
    for(new a = 0; a < 4; ++a)
      if(g_voteCount[a] > g_voteCount[best])
        best = a
  }

  new votesNum = g_voteCount[0] + g_voteCount[1] + g_voteCount[2] + g_voteCount[3]
  new iRatio = votesNum ? floatround(g_voteRatio * float(votesNum), floatround_ceil) : 1
  new iResult = g_voteCount[best]

  if(iResult < iRatio) {
    if(g_yesNoVote)
      client_print(0, print_chat, _T("%s (yes ^"%d^") (no ^"%d^") (needed ^"%d^")."), _T(g_votingFailed), g_voteCount[0], g_voteCount[1], iRatio)
    else
      client_print(0, print_chat, _T("%s (got ^"%d^") (needed ^"%d^")."), _T(g_votingFailed), iResult, iRatio)
    log_amx("Vote: %s (got ^"%d^") (needed ^"%d^")",g_votingFailed,iResult,iRatio)
    return
  }
  g_execLen = formatex(g_Execute, 255, g_Answer, g_optionName[best]) + 1

  if(g_execResult) {
    g_execResult = false
    if(is_user_connected(g_voteCaller)) {
      new menuBody[512]
      new len = formatex(menuBody, 511, g_coloredMenus ? _T("\yThe result: \w%s^n^n",g_voteCaller) : _T("The result: %s^n^n",g_voteCaller), g_Execute)
      len += copy(menuBody[len], 511 - len, g_coloredMenus ? _T("\yDo you want to continue?^n\w",g_voteCaller) : _T("Do you want to continue?^n",g_voteCaller))
      copy(menuBody[len], 511 - len, _T("^n1. Yes^n2. No",g_voteCaller))
      show_menu(g_voteCaller, 0x03, menuBody, 10, "The result: ")
      set_task(10.0, "autoRefuse", TASK_autoRefuse)
    }
    else {
      if(equal(g_Execute, "changelevel", 11)) {
        new modName[8]
        get_modname(modName, 7)
        if(!equal(modName, "zp")) {
          message_begin(MSG_ALL, SVC_INTERMISSION)
          message_end()
        }
      }
      set_task(1.0, "delayedExec", 55119966, g_Execute, g_execLen)
    }
  }

  client_print(0, print_chat, _T("%s (got ^"%d^") (needed ^"%d^"). The result: %s."), _T(g_votingSuccess), iResult, iRatio, g_Execute)
  log_amx("Vote: %s (got ^"%d^") (needed ^"%d^") (result ^"%s^")",g_votingSuccess,iResult,iRatio,g_Execute)
}

public voteCount(id,key) {
  if(g_bHasVoted & (1<<(id&31))) {
    return PLUGIN_HANDLED
  }
  g_bHasVoted |= (1<<(id&31))
  if(key == 9) {
    return PLUGIN_HANDLED
  }
  if(get_cvarptr_num(g_cvar_amx_vote_answers)) {
    new name[32]
    get_user_name(id, name, 31)
    if(g_yesNoVote)
      client_print(0, print_chat, _T("%s voted %s."), name, key ? _T("against") : _T("for"))
    else
      client_print(0, print_chat, _T("%s voted for ^"%s^"."), name, g_optionName[key])
  }
  ++g_voteCount[key]
  return PLUGIN_HANDLED
}

public cmdVoteMap(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new Float:voting = get_cvarptr_float(g_cvar_amx_last_voting)
  new Float:gametime = get_gametime()
  if(voting > gametime) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }
  if(voting && voting + get_cvarptr_float(g_cvar_amx_vote_delay) > gametime) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new argc = read_argc()
  //if(argc > 5)
    //argc = 5

  g_validMaps = 0
  g_optionName[0][0] = '^0'
  g_optionName[1][0] = '^0'
  g_optionName[2][0] = '^0'
  g_optionName[3][0] = '^0'

  for(new i = 1; i < argc && g_validMaps < 4; ++i) {
    read_argv(i, g_optionName[g_validMaps], 31)
    if(is_map_valid(g_optionName[g_validMaps])) {
      replace_all(g_optionName[g_validMaps], 31, "<", "")
      replace_all(g_optionName[g_validMaps], 31, ">", "")
      ++g_validMaps
    }
  }
  if(g_validMaps == 0) {
    console_print(id, (argc == 2) ? _T("Given map is not valid.") : _T("Given maps are not valid."))
    return PLUGIN_HANDLED
  }

  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted &= ~(1<<(i&31))
  }

  g_voteCaller = id
  
  new menu_msg[256]
  new keys = 0
  new players[32], player
  new numplayers
  new Float:vote_time = get_cvarptr_float(g_cvar_amx_vote_time) + 2.0

  get_players(players, numplayers, "c")
  if(g_validMaps > 1) {
    new temp[128]
    new menuid, tempkeys
    for(numplayers--; numplayers >= 0; numplayers--) {
      player = players[numplayers]
      keys = (1<<9)
      copy(menu_msg, 255, g_coloredMenus ? _T("\yChoose map: \w^n^n",player) : _T("Choose map: ^n^n",player))
      temp[0] = '^0'
      for(new a = 0; a < g_validMaps; ++a) {
        formatex(temp, 127, "%d. %s^n", a+1, g_optionName[a])
        add(menu_msg, 255, temp)
        keys |= (1<<a)
      }
      add(menu_msg, 255, _T("^n0. None",player))
      get_user_menu(player, menuid, tempkeys)
      if(tempkeys == 0 || menuid < 0) {
        show_menu(player, keys, menu_msg, floatround(vote_time), "Choose map: ")
      }
      g_menukeys = keys
      g_menumsg[player][0] = '^0'
      copy(g_menumsg[player], 255, menu_msg)
    }
    g_yesNoVote = 0
    g_menutitle[0] = '^0'
    copy(g_menutitle, 15, "Choose map: ")
    remove_task(TASK_check_menu)
    new param[3]
    param[0] = 1
    param[1] = menuid_choosemap
    param[2] = get_cvarptr_num(g_cvar_amx_vote_time)
    set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 3)
  }
  else {
    new menuid, tempkeys
    for(numplayers--; numplayers >= 0; numplayers--) {
      player = players[numplayers]
      formatex(menu_msg, 255, g_coloredMenus ? _T("\yChange map to ^"%s^"?\w^n^n1. Yes^n2. No",player) : _T("Change map to ^"%s^"?^n^n1. Yes^n2. No",player), g_optionName[0])
      keys = (1<<0)|(1<<1)
      get_user_menu(player, menuid, tempkeys)
      if(tempkeys == 0 || menuid < 0) {
        show_menu(player, keys, menu_msg, floatround(vote_time), "Change map to ")
      }
      g_menukeys = keys
      g_menumsg[player][0] = '^0'
      copy(g_menumsg[player], 255, menu_msg)
    }
    g_yesNoVote = 1
    g_menutitle[0] = '^0'
    copy(g_menutitle, 15, "Change map to ")
    remove_task(TASK_check_menu)
    new param[3]
    param[0] = 1
    param[1] = menuid_changemap
    param[2] = get_cvarptr_num(g_cvar_amx_vote_time)
    set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 3)
  }

  if(is_user_connected(id)) {
    new authid[32], name[32], ipaddress[24]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)

    if(argc == 2)
      log_amx("Vote: ^"%s<%d><%s><%s>^" vote map (map ^"%s^")",name,get_user_userid(id),authid,ipaddress,g_optionName[0])
    else
      log_amx("Vote: ^"%s<%d><%s><%s>^" vote maps (map#1 ^"%s^") (map#2 ^"%s^") (map#3 ^"%s^") (map#4 ^"%s^")",name,get_user_userid(id),authid,ipaddress,g_optionName[0],g_optionName[1],g_optionName[2],g_optionName[3])

    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("vote map(s)."))
    #else
    show_activity_color(id, name, _T("vote map(s)."))
    #endif
  }

  g_execResult = true
  set_cvarptr_float(g_cvar_amx_last_voting, gametime + vote_time)
  g_voteRatio = get_cvarptr_float(g_cvar_amx_votemap_ratio)
  g_Answer = "changelevel %s"
  set_task(vote_time, "checkVotes", TASK_checkVotes)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  return PLUGIN_HANDLED
}

public cmdVote(id,level,cid) {
  if(!cmd_access(id,level,cid,4))
    return PLUGIN_HANDLED

  new Float:voting = get_cvarptr_float(g_cvar_amx_last_voting)
  new Float:gametime = get_gametime()
  if(voting > gametime) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }
  if(voting && voting + get_cvarptr_float(g_cvar_amx_vote_delay) > gametime) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new quest[48+OPTIONNAME_EXTEND]
  read_argv(1, quest, 47+OPTIONNAME_EXTEND)
  if((contain(quest, "sv_password") != -1) || (contain(quest, "rcon_password") != -1) || 
    (contain(quest, "kick") != -1) || (contain(quest, "addip") != -1) || (contain(quest, "ban") != -1)) {
    console_print(id, _T("Voting for that has been forbidden."))
    return PLUGIN_HANDLED
  }

  g_optionName[0][0] = 0
  g_optionName[1][0] = 0
  g_optionName[2][0] = 0
  g_optionName[3][0] = 0

  read_argv(2, g_optionName[0], 31+OPTIONNAME_EXTEND)
  read_argv(3, g_optionName[1], 31+OPTIONNAME_EXTEND)
  read_argv(4, g_optionName[2], 31+OPTIONNAME_EXTEND)
  read_argv(5, g_optionName[3], 31+OPTIONNAME_EXTEND)

  if(is_user_connected(id)) {
    new authid[32], name[32], ipaddress[24]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)

    log_amx("Vote: ^"%s<%d><%s><%s>^" vote custom (question ^"%s^") (option#1 ^"%s^") (option#2 ^"%s^") (option#3 ^"%s^") (option#4 ^"%s^")",name,get_user_userid(id),authid,ipaddress,quest,g_optionName[0],g_optionName[1],g_optionName[2],g_optionName[3])

    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("vote custom."))
    #else
    show_activity_color(id, name, _T("vote custom."))
    #endif
  }

  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted &= ~(1<<(i&31))
  }

  g_voteCaller = id
  
  new menu_msg[256]
  new keys = (1<<0)|(1<<1)
  new players[32], player
  new numplayers
  new Float:vote_time = get_cvarptr_float(g_cvar_amx_vote_time) + 2.0

  new cmdaccess[32], cmdflags, playerflags, command = 0
  if(get_cmdaccess(quest, cmdaccess, 31)) {
    cmdflags = read_flags(cmdaccess)
    playerflags = get_user_flags(id)
    command = 1
  }
  new yesNoVote = (command && read_argc() == 4 && ((g_optionName[0][0] == '0' && g_optionName[1][0] == '1') || (g_optionName[0][0] == '1' && g_optionName[1][0] == '0')))

  new len
  new menuid, tempkeys
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    player = players[numplayers]
    if(yesNoVote) {
      len = formatex(menu_msg, 255, g_coloredMenus ? _T("\yVote: %s\w^n^n1. %s^n2. %s",player) : _T("Vote: %s^n^n1. %s^n2. %s",player), quest,
                  (g_optionName[0][0] == '0') ? _T("No",player) : _T("Yes",player),
                  (g_optionName[1][0] == '0') ? _T("No",player) : _T("Yes",player))
      g_menumsg[player][0] = '^0'
      copy(g_menumsg[player], 255, menu_msg)
    }
    else {
      len = formatex(menu_msg, 255, g_coloredMenus ? _T("\yVote: %s\w^n^n1. %s^n2. %s",player) : _T("Vote: %s^n^n1. %s^n2. %s",player),
                   quest, g_optionName[0], g_optionName[1])
      if(g_optionName[2][0]) {
        len += formatex(menu_msg[len], 255-len, "^n3. %s", g_optionName[2])
        keys |= (1<<2)
      }
      if(g_optionName[3][0]) {
        len += formatex(menu_msg[len], 255-len, "^n4. %s", g_optionName[3])
        keys |= (1<<3)
      }
      g_menumsg[player][0] = '^0'
      copy(g_menumsg[player], 255, menu_msg)
    }
    get_user_menu(player, menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      show_menu(player, keys, menu_msg, floatround(vote_time), "Vote: ")
    }
    g_menukeys = keys
  }

  g_menutitle[0] = '^0'
  copy(g_menutitle, 15, "Vote: ")
  remove_task(TASK_check_menu)
  new param[3]
  param[0] = 1
  param[1] = menuid_vote
  param[2] = get_cvarptr_num(g_cvar_amx_vote_time)
  set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 3)

  if(cvar_exists(quest) && (get_user_flags(id) & ADMIN_CVAR) && canChangeCvar(id, quest)) {
    g_execResult = true
    formatex(g_Answer, 127, "%s %%s", quest)
  }
  else if(command && ((playerflags & cmdflags) == cmdflags)) {
    g_execResult = true
    formatex(g_Answer, 127, "%s %%s", quest)
  }
  else {
    g_execResult = false
    formatex(g_Answer, 127, "%s - %%s", quest)
  }
  set_cvarptr_float(g_cvar_amx_last_voting, gametime + vote_time)
  g_voteRatio = get_cvarptr_float(g_cvar_amx_vote_ratio)
  
  set_task(vote_time, "checkVotes", TASK_checkVotes)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  g_yesNoVote = 0
  return PLUGIN_HANDLED
}

public cmdVoteKickBan(id,level,cid) {
  if(!cmd_access(id,level,cid,2))
    return PLUGIN_HANDLED

  new Float:voting = get_cvarptr_float(g_cvar_amx_last_voting)
  new Float:gametime = get_gametime()
  if(voting > gametime) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }

  if(voting && voting + get_cvarptr_float(g_cvar_amx_vote_delay) > gametime) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  new cmd[32]
  read_argv(0, cmd, 31)
  new voteban = equal(cmd, "amx_voteban")
  new arg[32]
  read_argv(1, arg, 31)
  new player = cmd_target(id, arg, 1)
  if(!player) return PLUGIN_HANDLED

  if(voteban && is_user_bot(player)) {
    new imname[32]
    get_user_name(player, imname, 31)
    console_print(id, _T("That action can't be performed on bot ^"%s^"."), imname)
    return PLUGIN_HANDLED
  }

  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted &= ~(1<<(i&31))
  }

  g_voteCaller = id
  
  new keys = (1<<0)|(1<<1)
  new menu_msg[256]
  new players[32], playerid
  new numplayers
  new Float:vote_time = get_cvarptr_float(g_cvar_amx_vote_time) + 2.0

  get_user_name(player, arg, 31)
  new menuid, tempkeys
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    playerid = players[numplayers]
    formatex(menu_msg, 255, g_coloredMenus ? _T("\y%s %s?\w^n^n1. Yes^n2. No", playerid) :  _T("%s %s?^n^n1. Yes^n2. No", playerid),
      voteban ? _T("Ban", playerid) : _T("Kick", playerid), arg)
    get_user_menu(playerid, menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      show_menu(playerid, keys, menu_msg, floatround(vote_time), voteban ? "Ban " : "Kick ")
    }
    g_menumsg[playerid][0] = '^0'
    copy(g_menumsg[playerid], 255, menu_msg)
  }

  g_menukeys = keys
  g_menutitle[0] = '^0'
  copy(g_menutitle, 15, voteban ? "Ban " : "Kick ")
  remove_task(TASK_check_menu)
  new param[3]
  param[0] = 1
  param[1] = voteban ? menuid_ban : menuid_kick
  param[2] = get_cvarptr_num(g_cvar_amx_vote_time)
  set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 3)

  g_yesNoVote = 1

  new banByIp = 0
  new arg2[32]
  read_argv(2, arg2, 31)
  if(equali(arg2, "ip"))
    banByIp = 1
  
  if(voteban) {
    if(banByIp)
      get_user_ip(player, g_optionName[0], 31, 1)
    else
      get_user_authid(player, g_optionName[0], 31)
  }
  else {
    num_to_str(get_user_userid(player), g_optionName[0], 31)
  }

  if(is_user_connected(id)) {
    new authid[32], name[32], ipaddress[24]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)
    log_amx("Vote: ^"%s<%d><%s><%s>^" vote %s (target ^"%s^")",name,get_user_userid(id),authid,ipaddress,voteban ? "ban" : "kick",arg)

    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("vote %s for %s."), voteban ? _T("ban") : _T("kick"), arg)
    #else
    show_activity_color(id, name, _T("vote %s for %s."), voteban ? _T("ban") : _T("kick"), arg)
    #endif
  }

  g_execResult = true
  set_cvarptr_float(g_cvar_amx_last_voting, gametime + vote_time)
  g_voteRatio = get_cvarptr_float(voteban ? g_cvar_amx_voteban_ratio : g_cvar_amx_votekick_ratio)
  if(voteban) {
    if(banByIp)
      g_Answer = "addip 30.0 ^"%s^";wait;writeip"
    else
      g_Answer = "banid 30.0 %s kick"
  }
  else {
    g_Answer = "kick #%s"
  }
  set_task(vote_time, "checkVotes", TASK_checkVotes)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0, 0, 0}
  return PLUGIN_HANDLED
}
