/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/* 
 * Cvar:
 * amx_flood_time <time in seconds> - sets frequency of chating
 *
 */

// Waiting delay before rewriting
// Note: delay added when a player's chat message has been blocked.
// This time is added to the amx_flood_time, so the player have to
// wait for the end before to use the "say" command normally.
const Float:FLOOD_INTERVAL = 3.0

#include <translator>
#include <amxmod>
#include <amxmisc>

new Float:g_fFlooding[33] = {0.0, ...}
new g_bHasFlooded
new g_cvar_amx_flood_time

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("antiflood")
  register_plugin(_T("Anti Flood"), AMX_VERSION, "AMX Mod Team")
  register_clcmd("say", "chkFlood")
  register_clcmd("say_team", "chkFlood")
  g_cvar_amx_flood_time = register_cvar("amx_flood_time", "0.75")
}

public chkFlood(id) {
  if(is_user_realadmin(id))
    return PLUGIN_CONTINUE

  new Float:fMaxChat = get_cvarptr_float(g_cvar_amx_flood_time)

  if(fMaxChat > 0.0) {
    new Float:fNextTime = get_gametime()
    if(g_fFlooding[id] > fNextTime) {
      if(!(g_bHasFlooded & (1<<(id&31)))) {
        client_print(id, print_notify, _T("** Stop flooding the server!"))
        g_fFlooding[id] = fNextTime + fMaxChat + FLOOD_INTERVAL
        g_bHasFlooded |= (1<<(id&31))
      }
      return PLUGIN_HANDLED
    }
    g_bHasFlooded &= ~(1<<(id&31))
    g_fFlooding[id] = fNextTime + fMaxChat
  }
  return PLUGIN_CONTINUE
}
