/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Admin commands:
 * amx_ff <0/1> - disables/enables Friendly Fire
 * amx_voteff - starts a vote to enable/disable Friendly Fire
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_FF      ADMIN_CVAR
#define FLAG_AMX_VOTEFF  ADMIN_VOTE

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const Float:CHECK_MENU_TIME = 0.5

new g_voteCount[2]
new g_coloredMenus
new g_maxplayers
new g_voteCaller
new g_bHasVoted
new g_menukeys
new g_menumsg[33][256]
new g_menutitle[16]

new const g_alreadyVoting[] = "There is already one voting..."
new const g_notAllowed[]    = "Voting not allowed at this time."
new const g_votingStarted[] = "Voting has started..."
new const g_votingFailed[]  = "Voting failed"
new const g_votingSuccess[] = "Voting successful"

new Float:g_voteRatio

new menuid_ff

new g_cvar_amx_vote_ratio
new g_cvar_amx_vote_time
new g_cvar_amx_vote_delay
new g_cvar_amx_vote_answers
new g_cvar_amx_last_voting

new g_cvar_mp_friendlyfire

const TASK_check_menu = 1467854

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("ff_manager")
  register_plugin(_T("FF Manager"),AMX_VERSION,"AMX Mod Team")
  register_concmd("amx_ff","cmdFF",FLAG_AMX_FF,_T("<0/1> - disables/enables Friendly Fire"))
  register_concmd("amx_voteff","cmdVoteFF",FLAG_AMX_VOTEFF,_T("- starts a vote to enable/disable Friendly Fire"))
  menuid_ff = register_menuid("Friendly Fire?")
  register_menucmd(menuid_ff,(1<<0)|(1<<1),"voteCount")

  g_coloredMenus = colored_menus()
  g_maxplayers = get_maxplayers()
}

public plugin_cfg() {
  g_cvar_amx_vote_ratio = get_cvar_pointer("amx_vote_ratio")
  g_cvar_amx_vote_time = get_cvar_pointer("amx_vote_time")
  g_cvar_amx_vote_delay = get_cvar_pointer("amx_vote_delay")
  g_cvar_amx_vote_answers = get_cvar_pointer("amx_vote_answers")
  g_cvar_amx_last_voting = get_cvar_pointer("amx_last_voting")
  g_cvar_mp_friendlyfire = get_cvar_pointer("mp_friendlyfire")
}

public cmdFF(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new mp_friendlyfire = get_cvarptr_num(g_cvar_mp_friendlyfire) ? 1 : 0

  if(read_argc() < 2) {
    console_print(id, _T("Friendly Fire is %s."), mp_friendlyfire ? _T("enabled") : _T("disabled"))
    return PLUGIN_HANDLED
  }

  new arg[8]
  read_argv(1, arg, 7)
  new ff = str_to_num(arg) ? 1 : 0
  if(ff == mp_friendlyfire) {
    console_print(id, _T("Friendly Fire is already %s."), mp_friendlyfire ? _T("enabled") : _T("disabled"))
    return PLUGIN_HANDLED
  }

  set_cvarptr_num(g_cvar_mp_friendlyfire, ff)
  showFFStatus()

  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("Cmd: ^"%s<%d><%s><%s>^" %s Friendly Fire",
    name,get_user_userid(id),authid,ipaddress,ff ? "enabled" : "disabled")

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("%s Friendly Fire."), ff ? _T("enable") : _T("disable"))
  #else
  show_activity_color(id, name, _T("%s Friendly Fire."), ff ? _T("enable") : _T("disable"))
  #endif

  return PLUGIN_HANDLED
}

public check_menu(param[]) {
  new iter = param[0]
  new Float:vote_time = float(param[1]) + 2.0 - (iter * CHECK_MENU_TIME)
  new menutime = floatround(vote_time, floatround_floor)
  if(menutime > 0) {
    new players[32], numplayers, player
    new menuid, keys
    get_players(players, numplayers, "c")
    for(numplayers--; numplayers >= 0; numplayers--) {
      player = players[numplayers]
      if(g_bHasVoted & (1<<(player&31))) continue
      get_user_menu(player, menuid, keys)
      if(keys == 0 || menuid < 0) {
        show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
      }
      else {
        if(menuid != menuid_ff) {
          if(menutime <= 6) {
            show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
          }
        }
        else if(player == g_voteCaller) {
          show_menu(player, g_menukeys, g_menumsg[player], menutime, g_menutitle)
        }
      }
    }
    ++param[0]
    set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 2)
  }
}

public checkVotes() {
  remove_task(TASK_check_menu)
  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted |= (1<<(i&31))
  }
  new votesNum = g_voteCount[0] + g_voteCount[1]
  new iRatio = votesNum ? floatround(g_voteRatio * float(votesNum), floatround_ceil) : 1

  new mp_friendlyfire = get_cvarptr_num(g_cvar_mp_friendlyfire)

  if(g_voteCount[0] <  iRatio) {
    client_print(0, print_chat, _T("%s (yes ^"%d^") (no ^"%d^") (needed ^"%d^")."), _T(g_votingFailed), g_voteCount[0], g_voteCount[1], iRatio)
    log_amx("Vote %s FF: %s (got ^"%d^") (needed ^"%d^")",mp_friendlyfire ? "Disable" : "Enable",g_votingFailed,g_voteCount[0],iRatio)
    return PLUGIN_CONTINUE
  }

  client_print(0, print_chat, _T("%s (got ^"%d^") (needed ^"%d^"). The result: Friendly Fire %s."), _T(g_votingSuccess), g_voteCount[0], iRatio, mp_friendlyfire ? _T("disabled") : _T("enabled"))
  log_amx("Vote %s FF: %s (got ^"%d^") (needed ^"%d^") (result ^"FF %s^")",mp_friendlyfire ? "Disable" : "Enable",g_votingSuccess,g_voteCount[0],iRatio,mp_friendlyfire ? _T("disabled") : _T("enabled"))

  set_cvarptr_num(g_cvar_mp_friendlyfire, mp_friendlyfire ? 0 : 1)
  showFFStatus()

  return PLUGIN_CONTINUE
}

public voteCount(id,key) {
  if(g_bHasVoted & (1<<(id&31))) {
    return PLUGIN_HANDLED
  }
  g_bHasVoted |= (1<<(id&31))
  if(get_cvarptr_num(g_cvar_amx_vote_answers)) {
    new name[32]
    get_user_name(id, name, 31)
    client_print(0, print_chat, _T("%s voted %s."), name, key ? _T("against") : _T("for"))
  }
  ++g_voteCount[key]
  return PLUGIN_HANDLED
}

public cmdVoteFF(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new Float:voting = get_cvarptr_float(g_cvar_amx_last_voting)
  if(voting > get_gametime()) {
    console_print(id, _T(g_alreadyVoting))
    return PLUGIN_HANDLED
  }
  if(voting && voting + get_cvarptr_float(g_cvar_amx_vote_delay) > get_gametime()) {
    console_print(id, _T(g_notAllowed))
    return PLUGIN_HANDLED
  }

  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted &= ~(1<<(i&31))
  }

  g_voteCaller = id

  new menu_msg[256]
  new keys = (1<<0)|(1<<1)
  new players[32], player
  new numplayers
  new Float:vote_time = get_cvarptr_float(g_cvar_amx_vote_time) + 2.0
  new mp_friendlyfire = get_cvarptr_num(g_cvar_mp_friendlyfire)

  new menuid, tempkeys
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    player = players[numplayers]
    if(g_coloredMenus) {
      formatex(menu_msg, 255, _T("\y%s Friendly Fire?\w^n^n1. Yes^n2. No",player),
        mp_friendlyfire ? _T("Disable",player) : _T("Enable",player))
    }
    else {
      formatex(menu_msg, 255, _T("%s Friendly Fire?^n^n1. Yes^n2. No",player),
        mp_friendlyfire ? _T("Disable",player) : _T("Enable",player))
    }
    get_user_menu(player, menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      show_menu(player, keys, menu_msg, floatround(vote_time), "Friendly Fire?")
    }
    g_menumsg[player][0] = '^0'
    copy(g_menumsg[player], 255, menu_msg)
  }

  g_menukeys = keys
  g_menutitle[0] = '^0'
  copy(g_menutitle, 15, "Friendly Fire?")
  remove_task(TASK_check_menu)
  new param[2]
  param[0] = 1
  param[1] = get_cvarptr_num(g_cvar_amx_vote_time)
  set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 2)

  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)

  log_amx("Vote: ^"%s<%d><%s><%s>^" vote to %s Friendly Fire",name,get_user_userid(id),authid,ipaddress,mp_friendlyfire ? "disable" : "enable")

  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("vote to %s Friendly Fire."), mp_friendlyfire ? _T("disable") : _T("enable"))
  #else
  show_activity_color(id, name, _T("vote to %s Friendly Fire."), mp_friendlyfire ? _T("disable") : _T("enable"))
  #endif

  set_cvarptr_float(g_cvar_amx_last_voting, get_gametime() + vote_time)
  g_voteRatio = get_cvarptr_float(g_cvar_amx_vote_ratio)
  set_task(vote_time, "checkVotes", 99889988)
  console_print(id, _T(g_votingStarted))
  g_voteCount = {0, 0}
  return PLUGIN_HANDLED
}

public showFFStatus() {
  new ff = get_cvarptr_num(g_cvar_mp_friendlyfire)
  set_hudmessage(255, 255, 255, -1.0, 0.65, 0, 6.0, 6.0, 0.5, 0.15, 2)
  show_hudmessage(0, _T("Friendly Fire has been %s..."), ff ? _T("enabled") : _T("disabled"))
  console_print(0, _T("Friendly Fire has been %s..."), ff ? _T("enabled") : _T("disabled"))
  client_cmd(0, "spk ^"vox/team kill is now %s^"", ff ? "on _period do not shoot your team" : "deactivated")
}
