/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Server command:
 * amx_imessage <message> <color in RRRGGGBBB format>
 *
 * You can also use "?" or "white", "red", "green", "blue", "yellow", "magenta", "cyan", "orange", "amx", "amxmod"
 * for the color of each message.
 *
 * Note: with this plugin you can use the Translations.
 * This feature is enabled by default, so please read the imessage.txt for more informations (addons/amx/lang).
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

const MAX_MESSAGES    = 10
const Float:X_POS     = -1.0
const Float:Y_POS     = 0.30
const Float:HOLD_TIME = 12.0

// Uncomment the following line to get localized messages.
// You'll have to translate them in the translations file.
// Use "\n" with the amx_imessage command and use "^n" in the translations file
// (for both english message and translations)
#define USE_TRANSLATIONS

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const MAX_CLR = 10

new const g_ColorsNames[MAX_CLR][] = {"white", "red", "green", "blue", "yellow", "magenta", "cyan", "orange", "amx", "amxmod"}
new const g_ColorsValues[MAX_CLR][] = {
  {255,255,255}, // white
  {255,0,0}, // red
  {0,255,0}, // green
  {0,0,255}, // blue
  {255,255,0}, // yellow
  {255,0,255}, // magenta
  {0,255,255}, // cyan
  {255,100,0}, // orange
  {0,100,255}, // AMX Mod original info color!
  {0,100,255} // AMX Mod original info color!
}

const IMSG_SIZE = 256

new g_Values[MAX_MESSAGES][3]
new g_Messages[MAX_MESSAGES][IMSG_SIZE]
new g_MessagesNum
new g_tempMessage[IMSG_SIZE]
new g_Current
new g_iColorNum
#if defined USE_TRANSLATIONS
new g_maxplayers
#endif

new g_cvar_amx_freq_imessage
new g_cvar_hostname

const TASK_infoMessage = 12345

isColorValue(const text[]) {
  for(new a = 0; a < MAX_CLR; ++a) {
    if(equali(text, g_ColorsNames[a])) {
      g_iColorNum = a
      return 1
    }
  }
  return 0
}

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("imessage")
  register_plugin(_T("Information Messages"), AMX_VERSION, "AMX Mod Team")
  register_srvcmd("amx_imessage", "setMessage")
  g_cvar_amx_freq_imessage = register_cvar("amx_freq_imessage", "60")
  g_cvar_hostname = get_cvar_pointer("hostname")
  new lastinfo[8]
  get_localinfo("lastinfomsg", lastinfo, charsmax(lastinfo))
  g_Current = str_to_num(lastinfo)
  set_localinfo("lastinfomsg", "")
#if defined USE_TRANSLATIONS
  g_maxplayers = get_maxplayers()
  server_print(_T("Note: You have activated the Translations in the ^"Information Messages^" plugin.^nPlease make sure your translations file (imessage.txt) contains the messages^nyou have defined in your AMX configuration file (amx.cfg)."))
#endif
}

public infoMessage() {
  if(g_Current >= g_MessagesNum) {
    g_Current = 0
  }
  set_hudmessage(g_Values[g_Current][0], g_Values[g_Current][1], g_Values[g_Current][2],  X_POS, Y_POS, 0, 0.5, HOLD_TIME, 2.0, 2.0, 1)
  copy(g_tempMessage, IMSG_SIZE-1, g_Messages[g_Current])
  new hostname[64]
  get_cvarptr_string(g_cvar_hostname, hostname, charsmax(hostname))
#if defined USE_TRANSLATIONS
  static tempMessage[IMSG_SIZE]
  for(new id = 1; id <= g_maxplayers; ++id) {
    if(!is_user_connected(id) || is_user_bot(id)) continue
    if(is_translated(g_tempMessage)) copy(tempMessage, IMSG_SIZE-1, _T(g_tempMessage, id))
    replace(tempMessage, IMSG_SIZE-1, "%hostname%", hostname)
    show_hudmessage(id, tempMessage)
    client_print(id, print_console, tempMessage)
  }
#else
  replace(g_tempMessage, IMSG_SIZE-1, "%hostname%", hostname)
  show_hudmessage(0, g_tempMessage)
  client_print(0, print_console, g_tempMessage)
#endif
  ++g_Current
  new Float:freq_im = get_cvarptr_float(g_cvar_amx_freq_imessage)
  if(freq_im > 0.0) set_task(freq_im, "infoMessage", TASK_infoMessage)
}

public setMessage() {
  if(read_argc() < 3)
    return PLUGIN_HANDLED

  if(g_MessagesNum >= MAX_MESSAGES) {
    server_print(_T("Information Messages limit reached!"))
    return PLUGIN_HANDLED
  }
  remove_task(TASK_infoMessage)
  read_argv(1, g_Messages[g_MessagesNum], IMSG_SIZE-1)
  replace_all(g_Messages[g_MessagesNum], IMSG_SIZE-1, "\n", "^n")
  new szColor[12]
  read_argv(2, szColor, charsmax(szColor)) // RRRGGGBBB
  trim(szColor) // Clean the text
  if(isdigit(szColor[0])) {
    g_Values[g_MessagesNum][2] = str_to_num(szColor[6])
    szColor[6] = 0
    g_Values[g_MessagesNum][1] = str_to_num(szColor[3])
    szColor[3] = 0
    g_Values[g_MessagesNum][0] = str_to_num(szColor[0])
  }
  else if(isColorValue(szColor)) {
    g_Values[g_MessagesNum][2] = g_ColorsValues[g_iColorNum][2]
    g_Values[g_MessagesNum][1] = g_ColorsValues[g_iColorNum][1]
    g_Values[g_MessagesNum][0] = g_ColorsValues[g_iColorNum][0]
  }
  else {
    g_Values[g_MessagesNum][2] = random_num(0, 255)
    g_Values[g_MessagesNum][1] = random_num(0, 255)
    g_Values[g_MessagesNum][0] = random_num(0, 255)
  }
  ++g_MessagesNum
  new Float:freq_im = get_cvarptr_float(g_cvar_amx_freq_imessage)
  if(freq_im > 0.0) set_task(freq_im, "infoMessage", TASK_infoMessage)
  return PLUGIN_HANDLED
}

public plugin_end() {
  new lastinfo[8]
  num_to_str(g_Current, lastinfo, charsmax(lastinfo))
  set_localinfo("lastinfomsg", lastinfo)
}
