/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * On two minutes before the timelimit, plugin
 * displays menu with 5 random maps to select.
 * One of the options is also map extending.
 * Map with most votes becomes the nextmap.
 * Last 5 maps played won't appear in the vote.
 *
 * Maps to select are in config/maps.ini file, mapcycle file or maps folder.
 *
 * If amx_mapchooser_type is set to 1, players can nominate maps for the vote.
 * They have to type "nominate mapname" or "vote mapname" or "mapname" in the chat.
 * They can type "nominations" in the chat to see the list of nominated maps.
 * They can type "amx_listmaps" in console to see the maps that can be nominated if
 * amx_nominfromfile is set to 1.
 *
 * Commands:
 * amx_votenextmap [time] - the map will be changed [time] seconds after the end of the vote
 * amx_listmaps - lists maps that can be nominated
 *
 * Cvars:
 * amx_mapchooser_type <0|1|2> - 0: random maps from amx_mapchooser_mapsfile/mapcycle/maps folder
 *                               1: nominations
 *                               2: the nextmap is a random map chosen from amx_mapchooser_mapsloc (no vote)
 * amx_mapchooser_mapsloc <0|1|2> - 0: amx_mapchooser_mapsfile
 *                                  1: mapcycle
 *                                  2: maps folder
 * amx_mapchooser_mapsfile "maps.ini" - File used when amx_mapchooser_mapsloc is set to 0
 *                                      The file must be in amx/config/ folder.
 * amx_nominfromfile <0/1> - 0: players can nominate all the maps from the map folder
 *                           1: players can only nominate maps from amx_mapchooser_mapsfile/mapcycle
 * amx_maxnominperplayer <num> - how many maps each player can nominate
 * amx_extendmap_max <num> - number of times the map can be extended in time
 * amx_extendmap_step <time in mins.> - with what time the map will be extended
 * amx_extendround_max <num> - number of times the map can be extended in rounds
 * amx_extendround_step <num> - with what number of rounds the map will be extended
 * amx_extendwin_max <num> - number of times the map can be extended in wins
 * amx_extendwin_step <num> - with what number of wins the map will be extended
 *
 *
 * NOTE: Nextmap plugin is required for proper working of this plugin.
 *
 * Some parts of the nominations code are taken from Deagles's map management plugin.
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_VOTENEXTMAP ADMIN_VOTE
#define FLAG_AMX_LISTMAPS    ADMIN_ALL

const MAX_MAPS            = 256 // maximum number of maps read from the amx_mapchooser_mapsfile/mapcycle files or from the maps folder
const SELECTMAPS          = 5   // maximum number of maps displayed in the vote menu
const MAP_HISTORY         = 5   // number of maps kept in the history
const NOMINATIONS_HELPMSG = 12  // a help message telling people they can nominate maps will be displayed
                                // every 15 * value seconds (ie: 15 * 12 = 3mns)
                                // if amx_mapchooser_type is set to 1

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const VOTE_TIME = 15
const Float:CHECK_MENU_TIME = 0.5

new g_hist_mapName[MAP_HISTORY][32]

new g_mapName[MAX_MAPS][32]
new g_mapsNum = 0

const LISTMAPAMOUNT = 10
new g_nominMapName[SELECTMAPS][32]
new g_nominMapsNum = 0
new g_nominated[33]
new g_whoNominMapNum[SELECTMAPS]

new g_mapVoteName[SELECTMAPS][32]
new g_voteCount[SELECTMAPS+2]
new g_mapVoteNum = 0
new g_teamScore[2]
new g_lastMap[32]

new g_bHasVoted

new g_coloredMenus
new g_maxplayers
new bool:g_selected = false

new g_extendCount
new bool:g_extend
new bool:g_forceVote
new g_forceVoteTime

new g_nominationsHelpMsgIter
new g_mapsOnServerNum

new g_cstrikeRunning

new g_cvar_amx_mapchooser_type
new g_cvar_amx_mapchooser_mapsloc
new g_cvar_amx_mapchooser_mapsfile
new g_cvar_amx_nominfromfile
new g_cvar_amx_maxnominperplayer
new g_cvar_amx_extendmap_max
new g_cvar_amx_extendmap_step
new g_cvar_amx_extendround_max
new g_cvar_amx_extendround_step
new g_cvar_amx_extendwin_max
new g_cvar_amx_extendwin_step

new g_cvar_amx_vote_answers
new g_cvar_amx_nextmap

new g_cvar_mapcyclefile
new g_cvar_mp_timelimit
new g_cvar_mp_maxrounds
new g_cvar_mp_winlimit

new menuid_choosenextmap

const TASK_setXvars    = 64973122
const TASK_voteNextmap = 987456
const TASK_check_menu  = 1467853

public NEXTMAP_MSG
public NEXTMAP_ROUNDCOUNT
public NEXTMAP_WINCOUNT
public NEXTMAP_TIMECOUNT

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("mapchooser")
  register_plugin(_T("Nextmap Chooser"),AMX_VERSION,"AMX Mod Team")
  register_concmd("amx_votenextmap","cmdVoteNextMap",FLAG_AMX_VOTENEXTMAP,_T("[time] - the map will be changed [time] seconds after the end of the vote"))
  register_clcmd("say","handleSay")
  register_clcmd("amx_listmaps","cmdListMaps",FLAG_AMX_LISTMAPS,_T("- lists maps that can be nominated"))
  menuid_choosenextmap = register_menuid("AMX Choose nextmap:")
  register_menucmd(menuid_choosenextmap,(-1^(-1<<(SELECTMAPS+2))),"countVote")
  g_cvar_amx_mapchooser_type = register_cvar("amx_mapchooser_type","0")
  g_cvar_amx_mapchooser_mapsloc = register_cvar("amx_mapchooser_mapsloc", "0")
  g_cvar_amx_mapchooser_mapsfile = register_cvar("amx_mapchooser_mapsfile", "maps.ini")
  g_cvar_amx_nominfromfile = register_cvar("amx_nominfromfile","0")
  g_cvar_amx_maxnominperplayer = register_cvar("amx_maxnominperplayer","1")
  g_cvar_amx_extendmap_max = register_cvar("amx_extendmap_max","3")
  g_cvar_amx_extendmap_step = register_cvar("amx_extendmap_step","15")
  g_cvar_amx_extendround_max = register_cvar("amx_extendround_max","3")
  g_cvar_amx_extendround_step = register_cvar("amx_extendround_step","3")
  g_cvar_amx_extendwin_max = register_cvar("amx_extendwin_max","3")
  g_cvar_amx_extendwin_step = register_cvar("amx_extendwin_step","3")

  g_cstrikeRunning = (is_running("cstrike") || is_running("czero"))
  g_coloredMenus = colored_menus()
  g_maxplayers = get_maxplayers()
  if(g_cstrikeRunning) {
    register_event("TeamScore","team_score","a")
  }

  get_localinfo("lastMap", g_lastMap, 31)
  set_localinfo("lastMap", "")
  g_nominationsHelpMsgIter = 0
}

public plugin_cfg() {
  g_cvar_amx_vote_answers = get_cvar_pointer("amx_vote_answers")
  g_cvar_amx_nextmap = get_cvar_pointer("amx_nextmap")
  g_cvar_mapcyclefile = get_cvar_pointer("mapcyclefile")
  g_cvar_mp_timelimit = get_cvar_pointer("mp_timelimit")
  if(g_cstrikeRunning) {
    g_cvar_mp_maxrounds = get_cvar_pointer("mp_maxrounds")
    g_cvar_mp_winlimit = get_cvar_pointer("mp_winlimit")
  }

  new filename[64]
  build_path(filename, 63, "$configdir/maphist.ini")
  loadHistory(filename)
  getMapsOnServerNum()

  new amx_mapchooser_mapsloc = get_cvarptr_num(g_cvar_amx_mapchooser_mapsloc)
  if(amx_mapchooser_mapsloc == 0) {
    new mapslocfile[32]
    get_cvarptr_string(g_cvar_amx_mapchooser_mapsfile, mapslocfile, 31)
    build_path(filename, 63, "$configdir/%s", mapslocfile)
    if(loadSettings(filename)) {
      log_amx(_T("Nextmap Chooser: ^"%s^" succesfully loaded."),mapslocfile)
      set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
      set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
      set_task(6.9, "setXvars", TASK_setXvars)
    }
    else {
      log_amx(_T("Nextmap Chooser: failed to load ^"%s^" or no maps inside the file. Trying to load mapcycle."),mapslocfile)
      get_cvarptr_string(g_cvar_mapcyclefile, filename, 63)
      if(loadSettings(filename)) {
        log_amx(_T("Nextmap Chooser: mapcycle succesfully loaded."))
        set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
        set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
        set_task(6.9, "setXvars", TASK_setXvars)
      }
      else {
        log_amx(_T("Nextmap Chooser: failed to load mapcycle or no maps inside the file. Trying to load maps folder."))
        set_cvarptr_num(g_cvar_amx_nominfromfile, 0)
        if(loadMapsFolder()) {
          log_amx(_T("Nextmap Chooser: maps folder succesfully loaded."))
          set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
          set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
          set_task(6.9, "setXvars", TASK_setXvars)
        }
        else {
          log_amx(_T("Nextmap Chooser: failed to load maps folder. No maps loaded."))
        }
      }
    }
  }
  else if(amx_mapchooser_mapsloc == 1) {
    get_cvarptr_string(g_cvar_mapcyclefile, filename, 63)
    if(loadSettings(filename)) {
      log_amx(_T("Nextmap Chooser: mapcycle succesfully loaded."))
      set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
      set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
      set_task(6.9, "setXvars", TASK_setXvars)
    }
    else {
      log_amx(_T("Nextmap Chooser: failed to load mapcycle or no maps inside the file. Trying to load maps folder."))
      set_cvarptr_num(g_cvar_amx_nominfromfile, 0)
      if(loadMapsFolder()) {
        log_amx(_T("Nextmap Chooser: maps folder succesfully loaded."))
        set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
        set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
        set_task(6.9, "setXvars", TASK_setXvars)
      }
      else {
        log_amx(_T("Nextmap Chooser: failed to load maps folder. No maps loaded."))
      }
    }
  }
  else if(amx_mapchooser_mapsloc == 2) {
    set_cvarptr_num(g_cvar_amx_nominfromfile, 0)
    if(loadMapsFolder()) {
      log_amx(_T("Nextmap Chooser: maps folder succesfully loaded."))
      set_task(15.0, "voteNextmap", TASK_voteNextmap, "", 0, "b")
      set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
      set_task(6.9, "setXvars", TASK_setXvars)
    }
    else {
      log_amx(_T("Nextmap Chooser: failed to load maps folder. No maps loaded."))
    }
  }
}

public setXvars() {
  if(get_cvarptr_num(g_cvar_amx_mapchooser_type) == 2) {
    remove_task(TASK_voteNextmap)
    set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 3)
    new a, i, randNum = random_num(1,10)
    for(i = 0; i < randNum; ++i) {
      a = random_num(0,g_mapsNum-1)
    }
    set_cvarptr_string(g_cvar_amx_nextmap, g_mapName[a])
    return
  }

  new timeleft = get_timeleft()
  new count = 1
  while((timeleft -= 15) > 129) ++count
  set_xvar_num(get_xvar_id("NEXTMAP_TIMECOUNT"), count*15)
  set_xvar_num(get_xvar_id("NEXTMAP_ROUNDCOUNT"), (g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_maxrounds) : 0) - 2)
  set_xvar_num(get_xvar_id("NEXTMAP_WINCOUNT"), (g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_winlimit) : 0) - 2)
}

public check_menu(param[]) {
  new iter = param[0]
  new Float:vote_time = float(VOTE_TIME) - (iter * CHECK_MENU_TIME)
  new menutime = floatround(vote_time, floatround_floor)
  if(menutime > 0) {
    new mapsmenu[512], mkeys, pos
    mkeys = g_extend ? ((1<<SELECTMAPS)|(1<<SELECTMAPS+1)) : (1<<SELECTMAPS+1)
    pos = 0
    for(new i = 0; i < g_mapVoteNum; ++i) {
      pos += formatex(mapsmenu[pos], 511, "%d. %s^n", i+1, g_mapVoteName[i])
      mkeys |= (1<<i)
    }
    mapsmenu[pos++] = '^n'
    pos = 0
    new mapname[32]
    get_mapname(mapname, 31)
    new nextMap[32]
    get_cvarptr_string(g_cvar_amx_nextmap, nextMap, 31)
    new players[32], numplayers, player
    new menuid, keys
    new menu[512]
    get_players(players, numplayers, "c")
    for(numplayers--; numplayers >= 0; numplayers--) {
      player = players[numplayers]
      if(g_bHasVoted & (1<<(player&31))) continue
      get_user_menu(player, menuid, keys)
      if(keys == 0 || menuid < 0) {
        menu[0] = '^0'
        if(g_coloredMenus) {
          pos = copy(menu, 511, _T("\yAMX Choose Nextmap\w^n^n",player))
        }
        else {
          pos = copy(menu, 511, _T("AMX Choose Nextmap^n",player))
        }
        pos += copy(menu[pos], 511-pos, mapsmenu)
        if(g_extend) {
          pos += formatex(menu[pos], 511-pos, _T("%d. Extend map ^"%s^"^n",player), SELECTMAPS+1, mapname)
        }
        formatex(menu[pos], 511-pos, _T("%d. Keep current nextmap: %s",player), SELECTMAPS+2, nextMap)
        show_menu(player, mkeys, menu, menutime, "AMX Choose nextmap:")
      }
      else if(menuid != menuid_choosenextmap) {
        if(menutime <= 6) {
          menu[0] = '^0'
          if(g_coloredMenus) {
            pos = copy(menu, 511, _T("\yAMX Choose Nextmap\w^n^n",player))
          }
          else {
            pos = copy(menu, 511, _T("AMX Choose Nextmap^n",player))
          }
          pos += copy(menu[pos], 511-pos, mapsmenu)
          if(g_extend) {
            pos += formatex(menu[pos], 511-pos, _T("%d. Extend map ^"%s^"^n",player), SELECTMAPS+1, mapname)
          }
          formatex(menu[pos], 511-pos, _T("%d. Keep current nextmap: %s",player), SELECTMAPS+2, nextMap)
          show_menu(player, mkeys, menu, menutime, "AMX Choose nextmap:")
        }
      }
    }
    param[0] += 1
    set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 1)
  }
}

public checkVotes() {
  remove_task(TASK_check_menu)
  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted |= (1<<(i&31))
  }

  new b = 0
  new Float:timelimit = get_cvarptr_float(g_cvar_mp_timelimit)
  new maxrounds = g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_maxrounds) : 0
  new winlimit = g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_winlimit) : 0

  for(new a = 0; a < g_mapVoteNum; ++a) {
    if(g_voteCount[b] < g_voteCount[a]) {
      b = a
    }
  }

  if(g_voteCount[SELECTMAPS] > g_voteCount[b]) {
    set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
    g_forceVote = false
    g_selected = false
    if(timelimit > 0) {
      new mapname[32], text[10]
      get_mapname(mapname, 31)
      new Float:steptime = get_cvarptr_float(g_cvar_amx_extendmap_step)
      formatex(text, 9, "%.2f", timelimit + steptime)
      set_cvarptr_string(g_cvar_mp_timelimit, text)
      client_print(0, print_chat, _T("Choosing finished. Current map will be extended to next %.0f minutes."), steptime)
      log_amx("Vote: Voting for the nextmap finished. Map ^"%s^" will be extended to next %.0f minutes.",
        mapname,steptime)
      ++g_extendCount
      new timeleft = get_timeleft()
      new count = 1
      while((timeleft -= 15) > 129) ++count
      set_xvar_num(get_xvar_id("NEXTMAP_TIMECOUNT"), count*15)
      return
    }
    if(maxrounds > 0) {
      new mapname[32]
      get_mapname(mapname, 31)
      new stepround = get_cvarptr_num(g_cvar_amx_extendround_step)
      set_cvarptr_num(g_cvar_mp_maxrounds, maxrounds + stepround)
      client_print(0, print_chat, _T("Choosing finished. Current map will be extended to next %d rounds."), stepround)
      log_amx("Vote: Voting for the nextmap finished. Map ^"%s^" will be extended to next %d rounds.",
        mapname,stepround)
      ++g_extendCount
      set_xvar_num(get_xvar_id("NEXTMAP_ROUNDCOUNT"), (get_cvarptr_num(g_cvar_mp_maxrounds) - 2) - (g_teamScore[0] + g_teamScore[1]))
      return
    }
    if(winlimit > 0) {
      new mapname[32]
      get_mapname(mapname, 31)
      new stepwin = get_cvarptr_num(g_cvar_amx_extendwin_step)
      set_cvarptr_num(g_cvar_mp_winlimit, winlimit + stepwin)
      client_print(0, print_chat, _T("Choosing finished. Current map will be extended to next %d wins."), stepwin)
      log_amx("Vote: Voting for the nextmap finished. Map ^"%s^" will be extended to next %d wins.",
        mapname,stepwin)
      ++g_extendCount
      new c = get_cvarptr_num(g_cvar_mp_winlimit) - 2
      set_xvar_num(get_xvar_id("NEXTMAP_WINCOUNT"), min(c-g_teamScore[0],c-g_teamScore[1]))
      return
    }
  }

  if(g_voteCount[b] && g_voteCount[SELECTMAPS+1] <= g_voteCount[b]) {
    set_cvarptr_string(g_cvar_amx_nextmap, g_mapVoteName[b])
  }
  new smap[32]
  get_cvarptr_string(g_cvar_amx_nextmap, smap, 31)
  log_amx("Vote: Voting for the nextmap finished. The nextmap will be ^"%s^".",smap)
  set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 3)
  if(g_forceVote == true) {
    set_task(float(g_forceVoteTime), "doMapChange", 6482257)
    client_print(0, print_chat, _T("Choosing finished. The nextmap will be ^"%s^", and it will be loaded in %d seconds."), smap, g_forceVoteTime)
  }
  else {
    client_print(0, print_chat, _T("Choosing finished. The nextmap will be ^"%s^"."), smap)
  }
}

public countVote(id,key) {
  if(g_bHasVoted & (1<<(id&31))) {
    return PLUGIN_HANDLED
  }
  g_bHasVoted |= (1<<(id&31))
  if(get_cvarptr_float(g_cvar_amx_vote_answers)) {
    new name[32]
    get_user_name(id, name, 31)
    if(key == SELECTMAPS+1) {
      new nextMap[32]
      get_cvarptr_string(g_cvar_amx_nextmap, nextMap, 31)
      client_print(0, print_chat, _T("%s chose to keep the current nextmap (%s)."), name, nextMap)
    }
    else if(key == SELECTMAPS) {
      client_print(0, print_chat, _T("%s chose map extending."), name)
    }
    else if(key < SELECTMAPS) {
      client_print(0, print_chat, _T("%s chose ^"%s^"."), name, g_mapVoteName[key])
    }
  }
  ++g_voteCount[key]
  return PLUGIN_HANDLED
}

bool:isInMenu(const map[]) {
  for(new a = 0; a < g_mapVoteNum; ++a) {
    if(equal(g_mapVoteName[a], map)) {
      return true
    }
  }
  return false
}

public voteNextmap() {
  new maxrounds = g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_maxrounds) : 0
  new winlimit = g_cstrikeRunning ? get_cvarptr_num(g_cvar_mp_winlimit) : 0
  new Float:timelimit = get_cvarptr_float(g_cvar_mp_timelimit)
  
  if(g_forceVote == false) {
    new amx_mapchooser_type = get_cvarptr_num(g_cvar_amx_mapchooser_type)
    if(maxrounds > 0) {
      if((maxrounds - 2) > (g_teamScore[0] + g_teamScore[1])) {
        set_xvar_num(get_xvar_id("NEXTMAP_ROUNDCOUNT"), (maxrounds - 2)-(g_teamScore[0] + g_teamScore[1]))
        g_selected = false
        if(amx_mapchooser_type == 1 && ++g_nominationsHelpMsgIter >= NOMINATIONS_HELPMSG) {
          g_nominationsHelpMsgIter = 0
          set_hudmessage(255, 255, 255, -1.0, 0.70, 0, 1.0, 10.0, 0.1, 0.2, 4)
          show_hudmessage(0, _T("Say ^"nominate map_name^" to nominate a map."))
        }
        return
      }
    }
    else if(winlimit > 0) {
      new c = winlimit - 2
      if((c > g_teamScore[0]) && (c > g_teamScore[1])) {
        set_xvar_num(get_xvar_id("NEXTMAP_WINCOUNT"), min(c-g_teamScore[0],c-g_teamScore[1]))
        g_selected = false
        if(amx_mapchooser_type == 1 && ++g_nominationsHelpMsgIter >= NOMINATIONS_HELPMSG) {
          g_nominationsHelpMsgIter = 0
          set_hudmessage(255, 255, 255, -1.0, 0.70, 0, 1.0, 10.0, 0.1, 0.2, 4)
          show_hudmessage(0, _T("Say ^"nominate map_name^" to nominate a map."))
        }
        return
      }
    }
    else {
      new timeleft = get_timeleft()
      if(timeleft < 1 || timeleft > 129) {
        if(timeleft > 0) {
          set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
          new count = 1
          while((timeleft -= 15) > 129) ++count
          set_xvar_num(get_xvar_id("NEXTMAP_TIMECOUNT"), count*15)
        }
        g_selected = false
        if(amx_mapchooser_type == 1 && ++g_nominationsHelpMsgIter >= NOMINATIONS_HELPMSG) {
          g_nominationsHelpMsgIter = 0
          set_hudmessage(255, 255, 255, -1.0, 0.70, 0, 1.0, 10.0, 0.1, 0.2, 4)
          show_hudmessage(0, _T("Say ^"nominate map_name^" to nominate a map."))
        }
        return
      }
    }
  }
  if(g_selected)
    return

  g_selected = true
  new menu[512], mapsmenu[512], a, mkeys, pos
  new dmax = ((g_mapsNum+g_nominMapsNum) > SELECTMAPS) ? SELECTMAPS : (g_mapsNum+g_nominMapsNum)
  new players[32], numplayers
  g_voteCount[SELECTMAPS] = 0
  g_voteCount[SELECTMAPS+1] = 0
  
  new mapname[32]
  get_mapname(mapname, 31)
  new nextMap[32]
  get_cvarptr_string(g_cvar_amx_nextmap, nextMap, 31)

  // Build the maps entries and the valid keys (same for all players)
  mkeys = (1<<SELECTMAPS+1)
  mapsmenu[0] = '^0'
  pos = 0
  for(g_mapVoteNum = 0; g_mapVoteNum < dmax; ++g_mapVoteNum) {
    if(g_nominMapsNum > 0 && g_mapVoteNum < g_nominMapsNum) {
      copy(g_mapVoteName[g_mapVoteNum], 31, g_nominMapName[g_mapVoteNum])
    }
    else {
      a = random_num(0,g_mapsNum-1)
      while(equal(g_mapName[a], nextMap) || isInMenu(g_mapName[a])) {
        if(++a >= g_mapsNum)
          a = 0
      }
      copy(g_mapVoteName[g_mapVoteNum], 31, g_mapName[a])
    }
    pos += formatex(mapsmenu[pos], 511-pos, "%d. %s^n", g_mapVoteNum+1, g_mapVoteName[g_mapVoteNum])
    mkeys |= (1<<g_mapVoteNum)
    g_voteCount[g_mapVoteNum] = 0
  }
  mapsmenu[pos++] = '^n'

  g_extend = false
  if((winlimit + maxrounds) == 0 && timelimit > 0 && g_extendCount < get_cvarptr_num(g_cvar_amx_extendmap_max)) {
    mkeys |= (1<<SELECTMAPS)
    g_extend = true
  }
  if(timelimit == 0 && maxrounds > 0 && g_extendCount < get_cvarptr_num(g_cvar_amx_extendround_max)) {
    mkeys |= (1<<SELECTMAPS)
    g_extend = true
  }
  if(timelimit == 0 && winlimit > 0 && g_extendCount < get_cvarptr_num(g_cvar_amx_extendwin_max)) {
    mkeys |= (1<<SELECTMAPS)
    g_extend = true
  }

  for(new i = 1; i <= g_maxplayers; ++i) {
    g_bHasVoted &= ~(1<<(i&31))
  }

  // Now build (translated) menu for each player and send it
  new menuid, tempkeys, player
  get_players(players, numplayers, "c")
  for(numplayers--; numplayers >= 0; numplayers--) {
    player = players[numplayers]
    get_user_menu(player, menuid, tempkeys)
    if(tempkeys == 0 || menuid < 0) {
      pos = copy(menu, 511, g_coloredMenus ? _T("\yAMX Choose Nextmap\w^n^n",player) : _T("AMX Choose Nextmap^n",player))
      pos += copy(menu[pos], 511-pos, mapsmenu)
      if(g_extend) {
        pos += formatex(menu[pos], 511-pos, _T("%d. Extend map ^"%s^"^n",player), SELECTMAPS+1, mapname)
      }
      formatex(menu[pos], 511-pos, _T("%d. Keep current nextmap: %s",player), SELECTMAPS+2, nextMap)
      show_menu(player, mkeys, menu, VOTE_TIME, "AMX Choose nextmap:")
    }
  }

  remove_task(TASK_check_menu)
  new param[1]
  param[0] = 1
  set_task(CHECK_MENU_TIME, "check_menu", TASK_check_menu, param, 1)
    
  set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 2)
  set_task(float(VOTE_TIME), "checkVotes")
  client_print(0, print_chat, _T("It's time to choose the nextmap..."))
  client_cmd(0, "spk Gman/Gman_Choose2")
  log_amx("Vote: Voting for the nextmap started")
}

public cmdVoteNextMap(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED
    
  if(get_xvar_num(get_xvar_id("NEXTMAP_MSG")) != 1 || g_forceVote == true) {
    console_print(id, _T("[AMX] You cannot start the vote for the next map now."))
    return PLUGIN_HANDLED
  }
  
  new arg[32]
  read_argv(1, arg, 31)
  g_forceVoteTime = strtonum(arg)
  if(g_forceVoteTime < 20) g_forceVoteTime = 20
  
  g_forceVote = true
  voteNextmap()
  
  new authid[32], name[32], ipaddress[24]
  get_user_authid(id, authid, 31)
  if(id == 0) name = "SERVER"
  else get_user_name(id, name, 31)
  get_user_ip(id, ipaddress, 23, 1)
  log_amx("VoteNextMap: ^"%s<%d><%s><%s>^" start the vote for the next map",name,get_user_userid(id),authid,ipaddress)
  #if !defined COLORED_ACTIVITY
  show_activity(id, name, _T("start the vote for the next map."))
  #else
  show_activity_color(id, name, _T("start the vote for the next map."))
  #endif
  return PLUGIN_HANDLED
}

public delayedChange(param[])
  server_cmd("changelevel %s", param)
  
public doMapChange() {
  new string[32]
  new len = get_cvarptr_string(g_cvar_amx_nextmap, string, 31)
  new modName[8]
  get_modname(modName, 7)
  if(!equal(modName, "zp")) {
    message_begin(MSG_ALL, SVC_INTERMISSION)
    message_end()
  }
  set_task(1.0, "delayedChange", 0, string, len)
}

isLastMaps(const map[]) {
  for(new i = 0; i < MAP_HISTORY; ++i) {
    if(equali(map, g_hist_mapName[i])) {
      return 1
    }
  }
  return 0
}

public cmdListMaps(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  if(get_cvarptr_num(g_cvar_amx_mapchooser_type) == 1) {
    new arg1[8]
    new start = read_argv(1, arg1, 7) ? str_to_num(arg1) : 1
    if(--start < 0) start = 0
    if(get_cvar_num("amx_nominfromfile") > 0) {
      if(start >= g_mapsNum) start = g_mapsNum - 1
      console_print(id, _T("^n----- AMX Nextmap Chooser: Maps that can be nominated -----"))
      new end = start + LISTMAPAMOUNT
      if(end > g_mapsNum) end = g_mapsNum
      for(new i = start; i < end; ++i) {
        console_print(id, "%3d: %s", i+1, g_mapName[i])
      }
      console_print(id, _T("----- Maps %d - %d of %d -----"), start+1, end, g_mapsNum)
      if(end < g_mapsNum)
        console_print(id, _T("----- Use ^"amx_listmaps %d^" for more -----"), end+1)
      else
        console_print(id, _T("----- Use ^"amx_listmaps 1^" for begin -----"))
    }
    else {
      if(start >= g_mapsOnServerNum) start = g_mapsOnServerNum - 1
      console_print(id, _T("^n----- AMX Nextmap Chooser: Maps that can be nominated -----"))
      new end = start + LISTMAPAMOUNT
      if(end > g_mapsOnServerNum) end = g_mapsOnServerNum
      new len, pos = 2, iter = 0, text[32]
      new currentMap[32]
      get_mapname(currentMap, 31)
      while((pos = read_dir("maps", pos, text, 31, len)) && iter < end) {
        if(len <= 4 || (len > 4 && !equali(text[len-4], ".bsp", 4))) continue
        text[len-4] = '^0'
        if(is_map_valid(text) && !isLastMaps(text) && !equali(text, currentMap)) {
          if(iter >= start) {
            console_print(id, "%3d: %s", iter+1, text)
          }
          ++iter
        }
      }
      console_print(id, _T("----- Maps %d - %d of %d -----"), start+1, end, g_mapsOnServerNum)
      if(end < g_mapsOnServerNum)
        console_print(id, _T("----- Use ^"amx_listmaps %d^" for more -----"), end+1)
      else
        console_print(id, _T("----- Use ^"amx_listmaps 1^" for begin -----"))
    }
  }
  else {
    console_print(id, _T("Nomination system is not available at the moment."))
  }
  return PLUGIN_HANDLED
}

public listNominations(id) {
  if(get_cvarptr_num(g_cvar_amx_mapchooser_type) == 1) {
    new a = 0, message[512], len = 0
    if(g_nominMapsNum > 0) {
      len = copy(message, 511, _T("Maps that have been nominated for the next map vote:",id))
      new name[24]
      while(a < g_nominMapsNum) {
        name[0] = '^0'
        get_user_name(g_whoNominMapNum[a], name, 23)
        len += formatex(message[len], 511-len, _T("^n^"%s^" by %s",id), g_nominMapName[a], name)
        ++a
      }
      set_hudmessage(0, 150, 255, 0.01, 0.18, 0, 15.0, 12.0, 1.5, 3.75, 2)
      show_hudmessage(id, message)
    }
  }
  return PLUGIN_HANDLED
}

public handleSay(id) {
  if(get_cvarptr_num(g_cvar_amx_mapchooser_type) == 1) {
    new message[256]
    read_args(message, 255)
    remove_quotes(message)
    if(equali(message, "nominations", 11)) {
      if(get_xvar_num(get_xvar_id("NEXTMAP_MSG")) == 2) {
        client_print(id, print_chat, _T("Voting in progress..."))
      }
      else {
        if(!g_nominMapsNum)
          client_print(id, print_chat, _T("No maps have been nominated so far, type ^"nominate map_name^" to nominate a map."))
        else
          listNominations(id)
      }
      return PLUGIN_CONTINUE
    }
    else if(equali(message, "nominate ", 9)) {
      handleNominate(id, message[9])
    }
    else if(equali(message, "vote ", 5)) {
      handleNominate(id, message[5])
    }
    else if(is_map_valid(message)) {
      nominateMap(id, message)
    }
    else {
      new mapname[32], saymap[29]
      read_args(saymap, 28)
      remove_quotes(saymap)
      formatex(mapname,31, "aim_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "as_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "awp_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "cs_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "de_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "fy_%s", saymap)
      if(is_map_valid(mapname)){
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "he_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "ka_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "kz_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
      formatex(mapname, 31, "dod_%s", saymap)
      if(is_map_valid(mapname)) {
        nominateMap(id, mapname)
        return PLUGIN_CONTINUE
      }
    }
  }
  return PLUGIN_CONTINUE
}

public handleNominate(id,map[]) {
  if(is_map_valid(map)) {
    nominateMap(id, map)
  }
  else {
    new mapname[32]
    formatex(mapname, 31, "aim_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "as_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "awp_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "cs_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "de_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "fy_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "he_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "ka_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "kz_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
    formatex(mapname, 31, "dod_%s", map)
    if(is_map_valid(mapname)) {
      nominateMap(id, mapname)
      return PLUGIN_HANDLED
    }
  }
  return PLUGIN_HANDLED
}

public nominateMap(id,map[]) {
  strtolower(map)
  new current_map[32]
  new n = 0, i, done = 0, isreplacement = 0
  get_mapname(current_map, 31)
  new temp_nominMapNums = g_nominMapsNum
  if(get_xvar_num(get_xvar_id("NEXTMAP_MSG")) == 2) {
    client_print(id, print_chat, _T("Voting in progress..."))
    return PLUGIN_HANDLED
  }
  if(get_xvar_num(get_xvar_id("NEXTMAP_MSG")) == 3) {
    new nextmap[32]
    get_cvarptr_string(g_cvar_amx_nextmap, nextmap, 31)
    client_print(id, print_chat, _T("Voting is now over, the next map will be ^"%s^"."), nextmap)
    return PLUGIN_HANDLED
  }
  if(!is_map_valid(map)) {
    client_print(id, print_chat, _T("Map ^"%s^" not found on this server, type ^"amx_listmaps^" in console for a list of maps."), map)
    return PLUGIN_HANDLED
  }
  if(isLastMaps(map) && !equali(map, current_map)) {
    client_print(id, print_chat, _T("You cannot nominate a map from the last %d maps played."), MAP_HISTORY)
    return PLUGIN_HANDLED
  }
  if(equali(map,current_map)) {
    client_print(id, print_chat, _T("^"%s^" is the current map, a vote will determine whether or not the map is extended."), map)
    return PLUGIN_HANDLED
  }
  new isinthelist = 0
  if(get_cvarptr_num(g_cvar_amx_nominfromfile) == 0) {
    new len, pos = 2, text[32]
    while((pos = read_dir("maps", pos, text, 31, len))) {
      if(len <= 4 || (len > 4 && !equali(text[len-4], ".bsp", 4))) continue
      text[len-4] = '^0'
      if(equali(map, text)) {
        isinthelist = 1
        break
      }
    }
  }
  else {
    for(i = 0; i < g_mapsNum; ++i) {
      if(equali(map, g_mapName[i])) {
        isinthelist = 1
      }
    }
  }
  if(!isinthelist) {
    client_print(id, print_chat, _T("You can only nominate certain maps. Type ^"amx_listmaps^" in console to see the listing."))
    return PLUGIN_HANDLED
  }
  new maxnom = get_cvarptr_num(g_cvar_amx_maxnominperplayer)
  if(g_nominMapsNum >= SELECTMAPS || g_nominated[id] >= maxnom) {
    if(g_nominated[id] > maxnom) {
      client_print(id, print_chat, _T("You already have nominated more than %d maps!"), maxnom)
      return PLUGIN_HANDLED
    }
    new plname[32]
    for(i = 0; i < g_nominMapsNum; ++i) {
      if(equali(map, g_nominMapName[i])) {
        get_user_name(g_whoNominMapNum[i], plname, 31)
        client_print(id, print_chat, _T("Map ^"%s^" has already been nominated by %s."), map, plname)
        return PLUGIN_HANDLED
      }
    }
    while(n < g_nominMapsNum && !done && g_nominated[id] > 1) {
      if(g_whoNominMapNum[n] == id) {
        g_nominated[id]--
        g_nominMapsNum = n
        done = 1
        isreplacement = 1
      }
      ++n
    }
    if(!done) {
      n = 0
      while(n < g_nominMapsNum && !done && g_nominated[id] > 0) {
        if(g_whoNominMapNum[n] == id) {
         g_nominated[id]--
         g_nominMapsNum = n
         done = 1
         isreplacement = 1
        }
        ++n
      }
    }
    if(!done) {
      client_print(id, print_chat, _T("Maximum number of nominations has been reached (%d)."), g_nominMapsNum)
      return PLUGIN_HANDLED
    }
  }
  new plname[32]
  for(i = 0; i < g_nominMapsNum; ++i) {
    if(equali(map, g_nominMapName[i])) {
      get_user_name(g_whoNominMapNum[i], plname, 31)
      client_print(id, print_chat, _T("Map ^"%s^" has already been nominated by %s."), map, plname)
      g_nominMapsNum = temp_nominMapNums
      return PLUGIN_HANDLED
    }
  }
  new name[32]
  get_user_name(id, name, 31)
  if(isreplacement == 1) {
    client_print(id, print_chat, _T("Your previous nomination of ^"%s^" has been replaced."), g_nominMapName[g_nominMapsNum])
  }
  else if(isreplacement == 2) {
    client_print(0, print_chat, _T("The previous nomination of ^"%s^" has been replaced."), g_nominMapName[g_nominMapsNum])
  }
  ++g_nominated[id]
  console_print(id, _T("Adding ^"%s^" to map nomination slot %d."), map, g_nominMapsNum+1)
  copy(g_nominMapName[g_nominMapsNum], 31, map)
  g_whoNominMapNum[g_nominMapsNum] = id
  if(isreplacement) {
    g_nominMapsNum = temp_nominMapNums
  }
  else {
    g_nominMapsNum = temp_nominMapNums + 1
  }
  client_print(0, print_chat, _T("%s has nominated map ^"%s^", say ^"nominations^" to see a list."), name, map)
  return PLUGIN_HANDLED
}

loadHistory(const filename[]) {
  if(!file_exists(filename))
    return 0
    
  new a
  for(new pos = 0; pos < MAP_HISTORY; ++pos) {
    read_file(filename, pos, g_hist_mapName[pos], 31, a)
  }

  return 1
}

loadSettings(const filename[]) {
  if(!file_exists(filename))
    return 0

  g_mapsNum = 0

  new szText[32]
  new a, pos = 0, i
  new currentMap[32]
  get_mapname(currentMap, 31)

  while(g_mapsNum < MAX_MAPS && (pos = read_file(filename, pos, szText, 31, a))) {
    if(!a || szText[0] == ';' || szText[0] == '/' || szText[0] == '#') continue

    parse(szText, g_mapName[g_mapsNum], 31)

    replace_all(g_mapName[g_mapsNum], 31, " ", "")
    replace_all(g_mapName[g_mapsNum], 31, ".bsp", "")

    if(is_map_valid(g_mapName[g_mapsNum])
    && !equali(g_mapName[g_mapsNum], g_lastMap)
    && !equali(g_mapName[g_mapsNum], currentMap)) {
      ++g_mapsNum
      for(i = 0; i < g_mapsNum-1; ++i) {
        if(equali(g_mapName[g_mapsNum-1], g_mapName[i])) {
          --g_mapsNum
          break
        }
      }
      for(i = 0; i < MAP_HISTORY; ++i) {
        if(equali(g_mapName[g_mapsNum-1], g_hist_mapName[i])) {
          --g_mapsNum
          break
        }
      }
    }
  }

  if(g_mapsNum <= SELECTMAPS)
    return 0

  return g_mapsNum
}

loadMapsFolder() {
  g_mapsNum = 0

  new len, pos = 2, i
  new currentMap[32]
  get_mapname(currentMap, 31)

  while(g_mapsNum < MAX_MAPS && (pos = read_dir("maps", pos, g_mapName[g_mapsNum], 31, len))) {
    if(len <= 4 || (len > 4 && !equali(g_mapName[g_mapsNum][len-4], ".bsp", 4))) continue

    g_mapName[g_mapsNum][len-4] = '^0'
    if(is_map_valid(g_mapName[g_mapsNum])
    && !equali(g_mapName[g_mapsNum], g_lastMap)
    && !equali(g_mapName[g_mapsNum], currentMap)) {
      ++g_mapsNum
      for(i = 0; i < MAP_HISTORY; ++i) {
        if(equali(g_mapName[g_mapsNum-1], g_hist_mapName[i])) {
          --g_mapsNum
          break
        }
      }
    }
  }

  if(g_mapsNum <= SELECTMAPS)
    return 0

  return g_mapsNum
}

getMapsOnServerNum() {
  new currentMap[32]
  get_mapname(currentMap, 31)
  new len, pos = 2, text[32]
  while((pos = read_dir("maps", pos, text, 31, len))) {
    if(len <= 4 || (len > 4 && !equali(text[len-4], ".bsp", 4))) continue
    text[len-4] = '^0'
    if(is_map_valid(text) && !isLastMaps(text) && !equali(text, currentMap)) {
      ++g_mapsOnServerNum
    }
  }
}

public team_score() {
  new team[2]
  read_data(1, team, 1)
  g_teamScore[(team[0]=='C') ? 0 : 1] = read_data(2)
  new maxrounds = get_cvarptr_num(g_cvar_mp_maxrounds)
  new winlimit = get_cvarptr_num(g_cvar_mp_winlimit)
  if(maxrounds > 0) {
    if((maxrounds - 2) > (g_teamScore[0] + g_teamScore[1])) {
      set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
      set_xvar_num(get_xvar_id("NEXTMAP_ROUNDCOUNT"), (maxrounds - 2) - (g_teamScore[0] + g_teamScore[1]))
    }
    else set_xvar_num(get_xvar_id("NEXTMAP_ROUNDCOUNT"), 0)
  }
  if(winlimit > 0) {
    new c = winlimit - 2
    if((c > g_teamScore[0]) && (c > g_teamScore[1])) {
      set_xvar_num(get_xvar_id("NEXTMAP_MSG"), 1)
      set_xvar_num(get_xvar_id("NEXTMAP_WINCOUNT"), min(c-g_teamScore[0],c-g_teamScore[1]))
    }
    else set_xvar_num(get_xvar_id("NEXTMAP_WINCOUNT"), 0)
  }
}

public plugin_end() {
  new current_map[32]
  get_mapname(current_map, 31)
  set_localinfo("lastMap", current_map)

  new filename[64]
  build_path(filename, 63, "$configdir/maphist.ini")
  if(file_exists(filename)) {
    new text[32]
    new a, pos
    for(pos = 0; pos < MAP_HISTORY; ++pos) {
      read_file(filename, pos+1, text, 31, a)
      write_file(filename, text, pos)
    }
  }
  write_file(filename, current_map, MAP_HISTORY-1)
}
