/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMXMODMENU       ADMIN_MENU
// The following flags are the flags required to use the menus from amxmodmenu (not from the console)
#define FLAG_AMX_KICKMENU     ADMIN_KICK
#define FLAG_AMX_BANMENU      ADMIN_BAN
#define FLAG_AMX_SLAPMENU     ADMIN_SLAY
#define FLAG_AMX_TEAMMENU     ADMIN_SLAY
#define FLAG_AMX_MAPMENU      ADMIN_MAP
#define FLAG_AMX_VOTEMAPMENU  ADMIN_VOTE
#define FLAG_AMX_SPEECHMENU   ADMIN_MENU
#define FLAG_AMX_CLCMDMENU    ADMIN_MENU
#define FLAG_AMX_LANGMENU     ADMIN_ALL
#define FLAG_AMX_CMDMENU      ADMIN_MENU
#define FLAG_AMX_CVARMENU     ADMIN_CVAR
#define FLAG_AMX_CFGMENU      ADMIN_MENU
#define FLAG_AMX_STATSCFGMENU ADMIN_CFG
#define FLAG_AMX_PAUSECFGMENU ADMIN_CFG
#define FLAG_AMX_RESTMENU     ADMIN_CFG
#define FLAG_AMX_TELEPORTMENU ADMIN_LEVEL_A
#define FLAG_AMX_PLUGCMDMENU  ADMIN_MENU
#define FLAG_AMX_PLUGCVARMENU ADMIN_CVAR

#define FLAG_AMX_ADDMENU      ADMIN_CFG
#define FLAG_AMX_ADDMENUITEM  ADMIN_CFG

#define AUTOMATIC_PRECACHING // add // in front of this line if you want to disable automatic precaching

const MAX_MENUS     = 16 // maximum number of menus you can create in the custom_menus.cfg file
const MAX_MENUITEMS = 32 // maximum number of items per menu

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

const NAME_LENGTH    = 48
const CMD_LENGTH     = 64
const TEAMCMD_LENGTH = 16

new g_menuName[MAX_MENUS][NAME_LENGTH]
new g_menuCmd[MAX_MENUS][CMD_LENGTH]
new g_menuAccess[MAX_MENUS]
new g_menuItemsPerPage[MAX_MENUS]
new g_menuItemName[MAX_MENUS][MAX_MENUITEMS][NAME_LENGTH]
new g_menuItemCmd[MAX_MENUS][MAX_MENUITEMS][CMD_LENGTH]
new g_menuItemMisc[MAX_MENUS][MAX_MENUITEMS][3]
new g_menuItemDisplayPlayers[MAX_MENUS][MAX_MENUITEMS]
new g_menuItemAll[MAX_MENUS][MAX_MENUITEMS][TEAMCMD_LENGTH]
new g_menuItemTeam1[MAX_MENUS][MAX_MENUITEMS][TEAMCMD_LENGTH]
new g_menuItemTeam2[MAX_MENUS][MAX_MENUITEMS][TEAMCMD_LENGTH]

new g_menuNum
new g_menuItemsNum[MAX_MENUS]

new g_menuIndex[33]
new g_menuItemsIndex[33]
new g_menuPosition[33]
new g_menuPlayers[33][32]
new g_menuPlayersNum[33]

new g_coloredMenus
new g_cstrikeRunning
new g_dodRunning

findmenuIndexByCmd(const menuCmd[]) {
  for(new i = 0; i < g_menuNum; ++i) {
    if(equali(menuCmd, g_menuCmd[i])) return i
  }
  return -1
}

findmenuIndexByName(const menuName[]) {
  for(new i = 0; i < g_menuNum; ++i) {
    if(equali(menuName, g_menuName[i])) return i
  }
  return -1
}

findMenuItemIndexByName(const menuNameCmd[], const menuItemName[]) {
  for(new i = 0; i < g_menuNum; ++i) {
    if(equali(menuNameCmd, g_menuName[i]) || equali(menuNameCmd, g_menuCmd[i])) {
      for(new j = 0; j < g_menuItemsNum[i]; ++j) {
        if(equali(menuItemName, g_menuItemName[i][j])) return j
      }
    }
  }
  return -1
}

addMenu(menuName[],const menuCmd[],access,menuInfo[],itemsPerPage) {
  if(g_menuNum < MAX_MENUS && findmenuIndexByName(menuName) == -1 && findmenuIndexByCmd(menuCmd) == -1) {
    copy(g_menuName[g_menuNum], NAME_LENGTH-1, menuName)
    copy(g_menuCmd[g_menuNum], CMD_LENGTH-1, menuCmd)
    g_menuAccess[g_menuNum] = access
    replace_all(menuName, NAME_LENGTH-1, " ", "")
    register_menucmd(register_menuid(menuName),1023,"actionMenu")
    if(is_translated(menuInfo)) {
      register_clcmd(menuCmd,"cmdMenu",access,_T(menuInfo))
    }
    else {
      register_clcmd(menuCmd,"cmdMenu",access,menuInfo)
    }
    g_menuItemsPerPage[g_menuNum] = itemsPerPage
    ++g_menuNum
    return 1
  }
  //log_amx(_T("Menus Front-End: failed creating menu: ^"%s^" ^"%s^" ^"%s^"."),menuName, menuCmd, menuInfo)
  return 0
}

//all mods: itemMod=0 ; only for CS: itemMod=1 ; only for CS and DoD: itemMod=2
addMenuItem(const menuNameCmd[],const plugin[],const itemName[],const itemCmd[],const itemFlags[],itemAccess,itemMod,displayPlayers=0,const All[]="",const Team1[]="",const Team2[]="") {
  new menuIndex = findmenuIndexByName(menuNameCmd)
  if(menuIndex == -1) menuIndex = findmenuIndexByCmd(menuNameCmd)
  if(menuIndex != -1 && g_menuItemsNum[g_menuNum] < MAX_MENUITEMS && (is_plugin_running(plugin) || !plugin[0])) {
    new menuItemsNum = g_menuItemsNum[menuIndex]
    copy(g_menuItemName[menuIndex][menuItemsNum], NAME_LENGTH-1, itemName)
    copy(g_menuItemCmd[menuIndex][menuItemsNum], CMD_LENGTH-1, itemCmd)
    g_menuItemMisc[menuIndex][menuItemsNum][0] = itemAccess
    g_menuItemMisc[menuIndex][menuItemsNum][1] = (read_flags(itemFlags) > 0) ? read_flags(itemFlags) : (1<<1)
    g_menuItemMisc[menuIndex][menuItemsNum][2] = itemMod
    g_menuItemDisplayPlayers[menuIndex][menuItemsNum] = displayPlayers
    copy(g_menuItemAll[menuIndex][menuItemsNum], TEAMCMD_LENGTH-1, All)
    copy(g_menuItemTeam1[menuIndex][menuItemsNum], TEAMCMD_LENGTH-1, Team1)
    copy(g_menuItemTeam2[menuIndex][menuItemsNum], TEAMCMD_LENGTH-1, Team2)
    ++g_menuItemsNum[menuIndex]
    return 1
  }
  //log_amx(_T("Menus Front-End: failed creating menu item: ^"%s^" ^"%s^" ^"%s^" ^"%s^"."),menuNameCmd,plugin,itemName,itemCmd)
  return 0
}

createDefaultMenus() {
  addMenu("AMX Mod Menu", "amxmodmenu", FLAG_AMXMODMENU, "- displays menus", 8)
  addMenuItem("AMX Mod Menu", "Players Menu", "Kick Player", "amx_kickmenu", "b", FLAG_AMX_KICKMENU, 0)
  addMenuItem("AMX Mod Menu", "Players Menu", "Ban Player", "amx_banmenu", "b", FLAG_AMX_BANMENU, 0)
  addMenuItem("AMX Mod Menu", "Players Menu", "Slap/Slay Player", "amx_slapmenu", "b", FLAG_AMX_SLAPMENU, 0)
  addMenuItem("AMX Mod Menu", "Players Menu", "Team Player", "amx_teammenu", "b", FLAG_AMX_TEAMMENU, 0)
  addMenuItem("AMX Mod Menu", "Maps Menu", "Changelevel", "amx_mapmenu", "b", FLAG_AMX_MAPMENU, 0)
  addMenuItem("AMX Mod Menu", "Maps Menu", "Vote for Maps", "amx_votemapmenu", "b", FLAG_AMX_VOTEMAPMENU, 0)
  addMenuItem("AMX Mod Menu", "Commands Menu", "Speech Stuff", "amx_speechmenu", "b", FLAG_AMX_SPEECHMENU, 0)
  addMenuItem("AMX Mod Menu", "Players Menu", "Client Commands", "amx_clcmdmenu", "b", FLAG_AMX_CLCMDMENU, 0)
  addMenuItem("AMX Mod Menu", "Language Manager", "Language Configuration", "amx_langmenu", "b", FLAG_AMX_LANGMENU, 0)
  addMenuItem("AMX Mod Menu", "Commands Menu", "Server Commands", "amx_cmdmenu", "b", FLAG_AMX_CMDMENU, 0)
  addMenuItem("AMX Mod Menu", "Commands Menu", "Cvars Settings", "amx_cvarmenu", "b", FLAG_AMX_CVARMENU, 0)
  addMenuItem("AMX Mod Menu", "Commands Menu", "Configuration", "amx_cfgmenu", "b", FLAG_AMX_CFGMENU, 0)
  if(g_cstrikeRunning || g_dodRunning) {
    addMenuItem("AMX Mod Menu", "Stats Configuration", "Stats Settings", "amx_statscfgmenu", "b", FLAG_AMX_STATSCFGMENU, 2)
  }
  addMenuItem("AMX Mod Menu", "Pause Plugins", "Pause Plugins", "amx_pausecfgmenu", "b", FLAG_AMX_PAUSECFGMENU, 0)
  if(g_cstrikeRunning) {
    addMenuItem("AMX Mod Menu", "Restrict Weapons", "Restrict Weapons", "amx_restmenu", "b", FLAG_AMX_RESTMENU, 1)
  }
  if(is_module_running("Fun")) {
    addMenuItem("AMX Mod Menu", "Teleport Menu", "Teleport Player", "amx_teleportmenu", "b", FLAG_AMX_TELEPORTMENU, 0)
  }
  addMenuItem("AMX Mod Menu", "Commands Menu", "Plugins Commands", "amx_plugcmdmenu", "b", FLAG_AMX_PLUGCMDMENU, 0)
  addMenuItem("AMX Mod Menu", "Commands Menu", "Plugins Cvars", "amx_plugcvarmenu", "b", FLAG_AMX_PLUGCVARMENU, 0)
}

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("menufront")
  register_plugin(_T("Menus Front-End"),AMX_VERSION,"AMX Mod Team")

  register_menucmd(register_menuid("TargetsMenu"),1023,"actionTargetsMenu")
  register_menucmd(register_menuid("PlayersMenu"),1023,"actionPlayersMenu")

  register_concmd("amx_addmenu","cmdAddMenu",FLAG_AMX_ADDMENU,_T("<menu name> <menu cmd> <menu access> <menu info> [number of items per page] - adds a menu"))
  register_concmd("amx_addmenuitem","cmdAddMenuItem",FLAG_AMX_ADDMENUITEM,_T("<menu name/cmd> <plugin> <item name> <item cmd> <flags> <access> [display players] [All arg] [Team1 arg] [Team2 arg]"))

  g_coloredMenus = colored_menus()
  g_cstrikeRunning = (is_running("cstrike") || is_running("czero"))
  g_dodRunning = is_running("dod")
}

public plugin_cfg() {
  g_menuNum = 0
  createDefaultMenus()

  new filename[64]
  build_path(filename, 63, "$configdir/custom_menus.cfg")
  server_cmd("exec %s", filename)
  server_exec()
}

#if defined AUTOMATIC_PRECACHING
public plugin_precache() {
  new szFilename[64]
  build_path(szFilename, 63, "$configdir/custom_menus.cfg")
  if(!file_exists(szFilename))
    return 0

  new text[256], szCommand[32], szMenuName[32], szPlugin[32], szItemName[32], szSound[128], szFlags[32], szAccess[32]
  new a, pos = 0
  new ext[5]
  new arg1[32], arg2[64], arg3[64]
  
  while((pos = read_file(szFilename, pos, text, 255, a))) {
    if(!a || text[0] == ';' || text[0] == '/' || text[0] == '#') continue

    if(parse(text, szCommand, 31, szMenuName, 31, szPlugin, 31, szItemName, 31, szSound, 127, szFlags, 31, szAccess, 31) > 6) {
      new parsedNums = parse(szSound, arg1, 31, arg2, 63, arg3, 63)
      if(parsedNums == 2 && equali(arg1, "spk")) {
        copy(szSound, 127, arg2)
        copy(ext, 4, ".wav")
      }
      else if(parsedNums == 3 && equali(arg1, "mp3") && (equali(arg2, "play") || equali(arg2, "loop"))) {
        copy(szSound, 127, arg3)
        copy(ext, 4, ".mp3")
      }
      else continue
     
      replace_all(szSound, 127, "\'", "")

      if(szSound[0] == '/') replace(szSound, 127, "/", "")

      if(ext[1] == 'm' || (!equali(szSound, "vox", 3) && !equali(szSound, "barney", 6) && !equali(szSound, "hgrunt", 6))) {
        if(strlen(szSound) > 4 && !equali(szSound[strlen(szSound)-4], ext)) add(szSound, 127, ext)
        if(ext[1] == 'w') format(szSound, 127, "sound/%s", szSound)
        if(file_exists(szSound)) {
          if(ext[1] == 'm') {
            precache_generic(szSound)
          }
          else {
            replace(szSound, 127, "sound/", "")
            precache_sound(szSound)
          }
        }
      }
    }
  }

  return 0
}
#endif

skiped_command(const cmd[]) {
  if(equali(cmd, "spk", 3)
  || equali(cmd, "amx", 3))
    return 1
  return 0
}

public cmdAddMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,5)) 
    return PLUGIN_HANDLED

  new menuName[NAME_LENGTH]
  read_argv(1, menuName, NAME_LENGTH-1)
  new menuCmd[CMD_LENGTH]
  read_argv(2, menuCmd, CMD_LENGTH-1)
  new menuAccess[32]
  read_argv(3, menuAccess, 31)
  new acc
  if(equal(menuAccess, "0")) acc = 0
  else acc = read_flags(menuAccess)
  new menuInfo[128]
  read_argv(4, menuInfo, 127)
  new szItemsPerPage[32]
  read_argv(5, szItemsPerPage, 31)
  new itemsPerPage = str_to_num(szItemsPerPage)
  if(itemsPerPage <= 0 || itemsPerPage > 8) itemsPerPage = 8
  
  if(addMenu(menuName, menuCmd, acc, menuInfo, itemsPerPage) && is_user_connected(id)) {
    new authid[32], name[32], ipaddress[24]
    get_user_authid(id, authid, 31)
    get_user_name(id, name, 31)
    get_user_ip(id, ipaddress, 23, 1)
    log_amx("Menu: ^"%s<%d><%s><%s>^" added menu ^"%s^" ^"%s^" ^"%s^" ^"%s^"",
      name,get_user_userid(id),authid,ipaddress,menuName,menuCmd,menuAccess,menuInfo)
    console_print(id, _T("Added menu ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuName, menuCmd, menuAccess, menuInfo)

    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("added menu ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuName, menuCmd, menuAccess, menuInfo)
    #else
    show_activity_color(id, name, _T("added menu ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuName, menuCmd, menuAccess, menuInfo)
    #endif
  }

  return PLUGIN_HANDLED
}

public cmdAddMenuItem(id,level,cid) {
  if(!cmd_access(id,level,cid,7))
    return PLUGIN_HANDLED

  new menuNameCmd[NAME_LENGTH]
  read_argv(1, menuNameCmd, NAME_LENGTH-1)
  new plugin[64]
  read_argv(2, plugin, 63)
  new itemName[NAME_LENGTH]
  read_argv(3, itemName, NAME_LENGTH-1)
  new itemCmd[CMD_LENGTH]
  read_argv(4, itemCmd, CMD_LENGTH-1)
  new itemFlags[32]
  read_argv(5, itemFlags, 31)
  new itemAccess[32]
  read_argv(6, itemAccess, 31)
  new acc
  if(equal(itemAccess, "0")) acc = 0
  else acc = read_flags(itemAccess)
  if(acc < 0) acc = 0
  new szDisplayPlayers[8]
  read_argv(7, szDisplayPlayers, 7)
  new displayPlayers = str_to_num(szDisplayPlayers)
  displayPlayers = (displayPlayers) ? 1 : 0
  new All[TEAMCMD_LENGTH]
  read_argv(8, All, TEAMCMD_LENGTH-1)
  replace_all(All, TEAMCMD_LENGTH-1, "^"", "")
  new Team1[TEAMCMD_LENGTH]
  read_argv(9, Team1, TEAMCMD_LENGTH-1)
  replace_all(Team1, TEAMCMD_LENGTH-1, "^"", "")
  new Team2[TEAMCMD_LENGTH]
  read_argv(10, Team2, TEAMCMD_LENGTH-1)
  replace_all(Team2, TEAMCMD_LENGTH-1, "^"", "")

  if(addMenuItem(menuNameCmd, plugin, itemName, itemCmd, itemFlags, acc, 0, displayPlayers, All, Team1, Team2) && is_user_connected(id)) {
    new authid[32], name[32], ipaddress[24]
    get_user_name(id, name, 31)
    get_user_authid(id, authid, 31)
    get_user_ip(id, ipaddress, 23, 1)
    log_amx("Menu: ^"%s<%d><%s><%s>^" added menu item ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^"",
      name,get_user_userid(id),authid,ipaddress,menuNameCmd,plugin,itemName,itemCmd,itemFlags,itemAccess)
    console_print(id, _T("Added menu item ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuNameCmd, plugin, itemName, itemCmd, itemFlags, itemAccess)

    #if !defined COLORED_ACTIVITY
    show_activity(id, name, _T("added menu item ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuNameCmd, plugin, itemName, itemCmd, itemFlags, itemAccess)
    #else
    show_activity_color(id, name, _T("added menu item ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^" ^"%s^"."), menuNameCmd, plugin, itemName, itemCmd, itemFlags, itemAccess)
    #endif
  }

  return PLUGIN_HANDLED
}

public cmdMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1)) 
    return PLUGIN_HANDLED

  new menuCmd[CMD_LENGTH]
  read_argv(0, menuCmd, CMD_LENGTH-1)
  new menuIndex = findmenuIndexByCmd(menuCmd)
  if(menuIndex != -1) {
    if(read_argc() > 1) {
      new args[128]
      read_args(args, 127)
      remove_quotes(args)
      replace_all(args, 127, "\'", "^"")
      new menuItemName[NAME_LENGTH]
      new teamCmd[TEAMCMD_LENGTH]
      parse(args, menuItemName, NAME_LENGTH-1, teamCmd, TEAMCMD_LENGTH-1)
      format(teamCmd, TEAMCMD_LENGTH-1, "^"%s^"", teamCmd)
      new menuItemIndex = findMenuItemIndexByName(g_menuName[menuIndex], menuItemName)
      if(menuItemIndex != -1) {
        new command[CMD_LENGTH+32]
        copy(command, CMD_LENGTH+31, g_menuItemCmd[menuIndex][menuItemIndex])
        if(containi(command, "%player%") != -1) {
          replace(command, CMD_LENGTH+31, "%player%", teamCmd)
        }
        replace_all(command, CMD_LENGTH+31, "\'", "^"")
        if(!skiped_command(command)) {
          replace(command, CMD_LENGTH+31, " ", "^t")
        }
        new flags = g_menuItemMisc[menuIndex][menuItemIndex][1]
        if(flags & 1)
          server_cmd("%s", command)
        else if(flags & 2) {
          new cmd[CMD_LENGTH], cmdaccess[32], playerflags, cmdflags, noaccess = 0
          playerflags = get_user_flags(id)
          parse(command, cmd, CMD_LENGTH-1)
          if(get_cmdaccess(cmd, cmdaccess, 31)) {
            cmdflags = read_flags(cmdaccess)
            if((playerflags & cmdflags) != cmdflags) noaccess = 1
            if(noaccess) {
              set_user_flags(id, cmdflags)
            }
            client_cmd(id, command)
            if(noaccess) {
              new param[3]
              param[0] = id
              param[1] = playerflags
              set_task(0.3, "setOldFlags", 1000+cmdflags+id, param, 3)
            }
          }
          else client_cmd(id, command)
        }
        else if(flags & 8)
          client_cmd(0, command)
        return PLUGIN_HANDLED
      }
    }
    displayMenu(id, g_menuIndex[id] = menuIndex, g_menuPosition[id] = 0)
  }
  return PLUGIN_HANDLED
}

public actionMenu(id,key) {
  switch(key) {
    case 8: displayMenu(id, g_menuIndex[id], ++g_menuPosition[id])
    case 9: displayMenu(id, g_menuIndex[id], --g_menuPosition[id])
    default: {
      new menuItemIndex = g_menuPosition[id] * g_menuItemsPerPage[g_menuIndex[id]] + key
      if(g_menuItemDisplayPlayers[g_menuIndex[id]][menuItemIndex]) {
        if(g_menuItemAll[g_menuIndex[id]][menuItemIndex][0] || g_menuItemTeam1[g_menuIndex[id]][menuItemIndex][0] || g_menuItemTeam2[g_menuIndex[id]][menuItemIndex][0]) {
          displayTargetsMenu(id, g_menuIndex[id], g_menuItemsIndex[id] = menuItemIndex, g_menuPosition[id] = 0)
        }
        else {
          g_menuItemsIndex[id] = menuItemIndex
          displayPlayersMenu(id, g_menuPosition[id] = 0)
        }
      }
      else {
        new command[CMD_LENGTH+32]
        copy(command, CMD_LENGTH+31, g_menuItemCmd[g_menuIndex[id]][g_menuPosition[id] * g_menuItemsPerPage[g_menuIndex[id]] + key])
        replace_all(command, CMD_LENGTH+31, "\'", "^"")
        if(!skiped_command(command)) {
          replace(command, CMD_LENGTH+31, " ", "^t")
        }
        new flags = g_menuItemMisc[g_menuIndex[id]][g_menuPosition[id] * g_menuItemsPerPage[g_menuIndex[id]] + key][1]
        if(flags & 1)
          server_cmd("%s", command)
        else if(flags & 2) {
          new cmd[CMD_LENGTH], cmdaccess[32], playerflags, cmdflags, noaccess = 0
          playerflags = get_user_flags(id)
          parse(command, cmd, CMD_LENGTH-1)
          if(get_cmdaccess(cmd, cmdaccess, 31)) {
            cmdflags = read_flags(cmdaccess)
            if((playerflags & cmdflags) != cmdflags) noaccess = 1
            if(noaccess) {
              set_user_flags(id, cmdflags)
            }
            client_cmd(id, command)
            if(noaccess) {
              new param[3]
              param[0] = id
              param[1] = playerflags
              set_task(0.3, "setOldFlags", 1200+cmdflags+id, param, 3)
            }
          }
          else client_cmd(id, command)
        }
        else if(flags & 8)
          client_cmd(0, command)
        if(flags & 16) displayMenu(id, g_menuIndex[id], g_menuPosition[id])
      }
    }
  }
  return PLUGIN_HANDLED
}

displayMenu(id,menuIndex,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0
  new start = pos * g_menuItemsPerPage[menuIndex]

  if(start >= g_menuItemsNum[menuIndex])
    start = pos = g_menuPosition[id] = 0

  new len
  if(is_translated(g_menuName[menuIndex])) {
    len = formatex(menuBody, 511, g_coloredMenus ? "\y%s\R%d/%d^n\w^n" : "%s %d/%d^n^n", _T(g_menuName[menuIndex],id),
      pos+1, (g_menuItemsNum[menuIndex] / g_menuItemsPerPage[menuIndex] + ((g_menuItemsNum[menuIndex] % g_menuItemsPerPage[menuIndex]) ? 1 : 0)))
  }
  else {
    len = formatex(menuBody, 511, g_coloredMenus ? "\y%s\R%d/%d^n\w^n" : "%s %d/%d^n^n", g_menuName[menuIndex],
      pos+1, (g_menuItemsNum[menuIndex] / g_menuItemsPerPage[menuIndex] + ((g_menuItemsNum[menuIndex] % g_menuItemsPerPage[menuIndex]) ? 1 : 0)))
  }

  new end = start + g_menuItemsPerPage[menuIndex]
  new keys = (1<<9)

  if(end > g_menuItemsNum[menuIndex])
    end = g_menuItemsNum[menuIndex]

  new flags = get_user_flags(id)

  for(new a = start; a < end; ++a) {
    new csdodRunning = (g_cstrikeRunning || g_dodRunning)
    if((!g_menuItemMisc[menuIndex][a][0] || (flags & g_menuItemMisc[menuIndex][a][0]) == g_menuItemMisc[menuIndex][a][0]) && (g_menuItemMisc[menuIndex][a][2] ? ((g_menuItemMisc[menuIndex][a][2] == 2) ? csdodRunning : g_cstrikeRunning) : 1)) {
      keys |= (1<<b)
      if(is_translated(g_menuItemName[menuIndex][a])) {
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, _T(g_menuItemName[menuIndex][a],id))
      }
      else {
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, g_menuItemName[menuIndex][a])
      }
    }
    else {
      ++b     
      if(g_coloredMenus) {
        if(is_translated(g_menuItemName[menuIndex][a])) {
          len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, _T(g_menuItemName[menuIndex][a],id))
        }
        else {
          len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, g_menuItemName[menuIndex][a])
        }
      }
      else {
        if(is_translated(g_menuItemName[menuIndex][a])) {
          len += formatex(menuBody[len], 511-len, "#. %s^n", _T(g_menuItemName[menuIndex][a],id))
        }
        else {
          len += formatex(menuBody[len], 511-len, "#. %s^n", g_menuItemName[menuIndex][a])
        }
      }
    }
  }

  if(end != g_menuItemsNum[menuIndex]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else formatex(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))

  new menuName[NAME_LENGTH]
  copy(menuName, NAME_LENGTH-1, g_menuName[menuIndex])
  replace_all(menuName, NAME_LENGTH-1, " ", "")
  show_menu(id, keys, menuBody, -1, menuName)
}

public setOldFlags(param[]) {
  new id = param[0]
  new playerflags = param[1]
  if(is_user_connected(id)) {
    remove_user_flags(id)
    set_user_flags(id, playerflags)
  }
}

public actionTargetsMenu(id,key) {
  switch(key) {
    case 0: displayPlayersMenu(id, g_menuPosition[id] = 0)
    case 1, 2, 3: {
      new command[CMD_LENGTH+32]
      copy(command, CMD_LENGTH+31, g_menuItemCmd[g_menuIndex[id]][g_menuItemsIndex[id]])
      switch(key) {
        case 1: replace(command, CMD_LENGTH+31, "%player%", g_menuItemTeam1[g_menuIndex[id]][g_menuItemsIndex[id]])
        case 2: replace(command, CMD_LENGTH+31, "%player%", g_menuItemTeam2[g_menuIndex[id]][g_menuItemsIndex[id]])
        case 3: replace(command, CMD_LENGTH+31, "%player%", g_menuItemAll[g_menuIndex[id]][g_menuItemsIndex[id]])
      }
      replace_all(command, CMD_LENGTH+31, "\'", "^"")

      new flags = g_menuItemMisc[g_menuIndex[id]][g_menuItemsIndex[id]][1]
      if(flags & 1)
        server_cmd("%s", command)
      else if(flags & 2) {
        new cmd[CMD_LENGTH], cmdaccess[32], playerflags, cmdflags, noaccess = 0
        playerflags = get_user_flags(id)
        parse(command, cmd, CMD_LENGTH-1)
        if(get_cmdaccess(cmd, cmdaccess, 31)) {
          cmdflags = read_flags(cmdaccess)
          if((playerflags & cmdflags) != cmdflags) noaccess = 1
          if(noaccess) {
            set_user_flags(id, cmdflags)
          }
          client_cmd(id, command)
          if(noaccess) {
            new param[3]
            param[0] = id
            param[1] = playerflags
            set_task(0.3, "setOldFlags", 1400+cmdflags+id, param, 3)
          }
        }
        else client_cmd(id, command)
      }
      else if(flags & 8)
        client_cmd(0, command)
      if(flags & 16) displayTargetsMenu(id, g_menuIndex[id], g_menuItemsIndex[id], g_menuPosition[id])

      /*new authid[32], name[32]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)          
      log_amx("%s: ^"%s<%d><%s><>^" executed ^"%s^"", 
          g_menuName[g_menuIndex[id]],name,get_user_userid(id),authid,command)

      #if !defined COLORED_ACTIVITY
      show_activity(id, name, "%s", command)
      #else
      show_activity_color(id, name, "%s", command)
      #endif*/
    }
    case 8: displayTargetsMenu(id, g_menuIndex[id], g_menuItemsIndex[id], ++g_menuPosition[id])
    case 9: displayTargetsMenu(id, g_menuIndex[id], g_menuItemsIndex[id], --g_menuPosition[id])
  }
  return PLUGIN_HANDLED
}

displayTargetsMenu(id,menuIndex,menuItemIndex,pos) {
  if(pos < 0)
    return

  new selNum = 4
  new menuBody[192]
  new b = 0
  new start = pos * 8

  if(start >= selNum)
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 191, g_coloredMenus ?
    _T("\yTargets Menu\R%d/%d^n\w^n",id) :
    _T("Targets Menu %d/%d^n^n",id),
    pos+1, (selNum / 8 + ((selNum % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > selNum)
    end = selNum

  keys |= (1<<b)
  len += formatex(menuBody[len], 191-len, _T("%d. Select player^n",id), ++b)

  if(!g_menuItemTeam1[menuIndex][menuItemIndex][0]) {
    ++b
    if(g_coloredMenus)
      len += formatex(menuBody[len], 191-len, "\d%d. %s^n\w", b, _T("Team 1",id))
    else
      len += formatex(menuBody[len], 191-len, "#. %s^n", _T("Team 1",id))
  }
  else {
    keys |= (1<<b)
    len += formatex(menuBody[len], 191-len, "%d. %s (%s)^n", ++b, _T("Team 1",id), g_menuItemTeam1[menuIndex][menuItemIndex])
  }

  if(!g_menuItemTeam2[menuIndex][menuItemIndex][0]) {
    ++b
    if(g_coloredMenus)
      len += formatex(menuBody[len], 191-len, "\d%d. %s^n\w", b, _T("Team 2",id))
    else
      len += formatex(menuBody[len], 191-len, "#. %s^n", _T("Team 2",id))
  }
  else {
    keys |= (1<<b)
    len += formatex(menuBody[len], 191-len, "%d. %s (%s)^n", ++b, _T("Team 2",id), g_menuItemTeam2[menuIndex][menuItemIndex])
  }

  if(!g_menuItemAll[menuIndex][menuItemIndex][0]) {
    ++b
    if(g_coloredMenus)
      len += formatex(menuBody[len], 191-len, "\d%d. %s^n\w", b, _T("All players",id))
    else
      len += formatex(menuBody[len], 191-len, "#. %s^n", _T("All players",id))
  }
  else {
    keys |= (1<<b)
    len += formatex(menuBody[len], 191-len, "%d. %s (%s)^n", ++b, _T("All players",id), g_menuItemAll[menuIndex][menuItemIndex])
  }

  if(end != selNum) {
    formatex(menuBody[len], 191-len,_T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 191-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "TargetsMenu")
}

public actionPlayersMenu(id,key) {
  switch(key) {
    case 8: displayPlayersMenu(id, ++g_menuPosition[id])
    case 9: displayPlayersMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 8 + key]
 
      new authid[32], authid2[32], name[32], name2[32]
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_name(id, name, 31)
      get_user_name(player, name2, 31)      
      new userid2 = get_user_userid(player)

      new command[CMD_LENGTH+32]
      copy(command, CMD_LENGTH+31, g_menuItemCmd[g_menuIndex[id]][g_menuItemsIndex[id]])
      new szUserId[32]
      formatex(szUserId, 31, "#%d", userid2)
      replace(command, CMD_LENGTH+31, "%player%", szUserId)
      replace_all(command, CMD_LENGTH+31, "\'", "^"")
      if(!skiped_command(command)) {
        replace(command, CMD_LENGTH+31, " ", "^t")
      }
      new flags = g_menuItemMisc[g_menuIndex[id]][g_menuItemsIndex[id]][1]
      if(flags & 1)
        server_cmd("%s", command)
      else if(flags & 2) {
        new cmd[CMD_LENGTH], cmdaccess[32], playerflags, cmdflags, noaccess = 0
        playerflags = get_user_flags(id)
        parse(command, cmd, CMD_LENGTH-1)
        if(get_cmdaccess(cmd, cmdaccess, 31)) {
          cmdflags = read_flags(cmdaccess)
          if((playerflags & cmdflags) != cmdflags) noaccess = 1
          if(noaccess) {
            set_user_flags(id, cmdflags)
          }
          client_cmd(id, command)
          if(noaccess) {
            new param[3]
            param[0] = id
            param[1] = playerflags
            set_task(0.3, "setOldFlags", 2000+cmdflags+id, param, 3)
          }
        }
        else client_cmd(id, command)
      }
      else if(flags & 4)
        client_cmd(player, command)
      else if(flags & 8)
        client_cmd(0, command)
      if(flags & 16) displayPlayersMenu(id, g_menuPosition[id])

      /*log_amx("%s: ^"%s<%d><%s><>^" executed ^"%s^" on ^"%s<%d><%s><>^"", 
          g_menuName[g_menuIndex[id]],name,get_user_userid(id),authid,command,name2,userid2,authid2)

      copy(command, CMD_LENGTH+31, g_menuItemCmd[g_menuIndex[id]][g_menuItemsIndex[id]])
      new quotedName[36]
      formatex(quotedName, 35, "^"%s^"", name2)
      replace(command, CMD_LENGTH+31, "%player%", quotedName)
      #if !defined COLORED_ACTIVITY
      show_activity(id, name, "%s", command)
      #else
      show_activity_color(id, name, "%s", command)
      #endif*/
    }
  }
  return PLUGIN_HANDLED
}

displayPlayersMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 8
  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yPlayers Menu\R%d/%d^n\w^n",id) :
    _T("Players Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 8 + ((g_menuPlayersNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if( (i != id) && (isSupreme == 0) && (get_user_flags(i) & ADMIN_IMMUNITY) ) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id), pos ? _T("Back",id) : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s", pos ? _T("Back",id) : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "PlayersMenu")
}
