/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Admin commands:
 * amx_kickmenu - displays kick menu
 * amx_banmenu - displays ban menu
 * amx_unbanmenu - displays unban menu
 * amx_slapmenu - displays slap/slay menu
 * amx_teammenu - displays team menu
 * amx_clcmdmenu - displays client commands menu
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_KICKMENU   ADMIN_KICK
#define FLAG_AMX_BANMENU    ADMIN_BAN
#define FLAG_AMX_UNBANMENU  ADMIN_UNBAN|ADMIN_PERMBAN // the | symbol means you must have both flags to use the command
#define FLAG_AMX_SLAPMENU   ADMIN_SLAY
#define FLAG_AMX_TEAMMENU   ADMIN_SLAY
#define FLAG_AMX_CLCMDMENU  ADMIN_MENU

const MAX_CLCMDS = 24 // maximum number of commands in the clcmds.ini file

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>
#include <fun>
#include <VexdUM>

new const g_menuTempBanTime[5] = {5, 15, 30, 60, 120}

new g_menuPosition[33]
new g_menuPlayers[33][32]
new g_menuPlayersNum[33]
new g_menuOption[33]
new g_menuSettings[33]
new g_menuBanSteamid[33]
new g_menuUnBanType[33]
new g_menuUnBanLine[33][2]
new g_menuUnBanText[33][8][32]
new const g_bannedCfgFile[2][] = {"banned.cfg", "listip.cfg"}

new g_menuSelect[33][64]
new g_menuSelectNum[33]

new g_clcmdName[MAX_CLCMDS][32]
new g_clcmdCmd[MAX_CLCMDS][64]
new g_clcmdMisc[MAX_CLCMDS][2]
new g_clcmdNum

new g_coloredMenus
new g_cstrikeRunning
new g_dodRunning
new g_funRunning
new g_vexdRunning

new g_cvar_allow_spectators
new g_cvar_mp_limitteams

public disable_natives_check() {
}

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("plmenu")
  register_plugin(_T("Players Menu"),AMX_VERSION,"AMX Mod Team")
  register_clcmd("amx_kickmenu","cmdKickMenu",FLAG_AMX_KICKMENU,_T("- displays kick menu"))
  register_clcmd("amx_banmenu","cmdBanMenu",FLAG_AMX_BANMENU,_T("- displays ban menu"))
  register_clcmd("amx_unbanmenu","cmdUnBanMenu",FLAG_AMX_UNBANMENU,_T("- displays unban menu"))
  register_clcmd("amx_slapmenu","cmdSlapMenu",FLAG_AMX_SLAPMENU,_T("- displays slap/slay menu"))
  register_clcmd("amx_teammenu","cmdTeamMenu",FLAG_AMX_TEAMMENU,_T("- displays team menu"))
  register_clcmd("amx_clcmdmenu","cmdClcmdMenu",FLAG_AMX_CLCMDMENU,_T("- displays client commands menu"))

  register_menucmd(register_menuid("Ban Menu"),1023,"actionBanMenu")
  register_menucmd(register_menuid("UnBan SteamID or IP?"),(1<<0|1<<1|1<<9),"actionUnBanMenuType")
  register_menucmd(register_menuid("UnBan  Menu"),1023,"actionUnBanMenu")
  register_menucmd(register_menuid("Kick Menu"),1023,"actionKickMenu")
  register_menucmd(register_menuid("Slap/Slay Menu"),1023,"actionSlapMenu")
  register_menucmd(register_menuid("Team Menu"),1023,"actionTeamMenu")
  register_menucmd(register_menuid("Client Commands Menu"),1023,"actionClcmdMenu")

  g_cvar_allow_spectators = get_cvar_pointer("allow_spectators")
  g_cvar_mp_limitteams = get_cvar_pointer("mp_limitteams")

  g_coloredMenus = colored_menus()
  g_cstrikeRunning = (is_running("cstrike") || is_running("czero"))
  g_dodRunning = is_running("dod")
  g_funRunning = is_module_running("Fun")
  g_vexdRunning = is_module_running("VexdUM")

  new filename[64]
  build_path(filename, 63, "$configdir/clcmds.ini")
  load_settings(filename)
}

/* Ban menu */
public actionBanMenu(id,key) {
  switch(key) {
    case 6: {
      g_menuBanSteamid[id] = !g_menuBanSteamid[id]
      displayBanMenu(id, g_menuPosition[id])
    }
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= 6

      switch(g_menuOption[id]) {
        case 0: {
          if(get_user_flags(id) & ADMIN_PERMBAN) {
            g_menuSettings[id] = 0
          }
          else {
            g_menuOption[id] = 1
            g_menuSettings[id] = g_menuTempBanTime[0]
          }
        }
        case 1: g_menuSettings[id] = g_menuTempBanTime[0]
        case 2: g_menuSettings[id] = g_menuTempBanTime[1]
        case 3: g_menuSettings[id] = g_menuTempBanTime[2]
        case 4: g_menuSettings[id] = g_menuTempBanTime[3]
        case 5: g_menuSettings[id] = g_menuTempBanTime[4]
      }

      displayBanMenu(id, g_menuPosition[id])
    }
    case 8: displayBanMenu(id, ++g_menuPosition[id])
    case 9: displayBanMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 6 + key]

      new name[32], name2[32], authid[32], authid2[32], ipaddress[24], ipaddress2[24]
      get_user_name(id, name, 31)
      get_user_name(player, name2, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_ip(id, ipaddress, 23, 1)
      get_user_ip(player, ipaddress2, 23, 1)
      new userid2 = get_user_userid(player)

      log_amx("Ban: ^"%s<%d><%s><%s>^" ban and kick ^"%s<%d><%s><%s>^" (minutes ^"%d^")",
        name,get_user_userid(id),authid,ipaddress,name2,userid2,authid2,ipaddress2,g_menuSettings[id])

      #if !defined COLORED_ACTIVITY
      show_activity(id, name, _T("ban %s."), name2)
      #else
      show_activity_color(id, name, _T("ban %s."), name2)
      #endif

      if(equal("4294967295",authid2)
      || equal("HLTV",authid2)
      || equal("STEAM_ID_LAN",authid2)
      || equali("VALVE_ID_LAN", authid2)
      || g_menuBanSteamid[id] == 0) {
        new ipa[32]
        get_user_ip(player, ipa, 31, 1)
        server_cmd("addip %d %s;writeip", g_menuSettings[id], ipa)
      }
      else
        server_cmd("banid %d #%d kick;writeid", g_menuSettings[id], userid2)

      server_exec()

      displayBanMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayBanMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 6

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yBan Menu\R%d/%d^n\w^n",id) :
    _T("Ban Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 6 + ((g_menuPlayersNum[id] % 6) ? 1 : 0)))

  new end = start + 6
  new keys = (1<<9)|(1<<7)|(1<<6)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  new isMaster = has_all_flags(id, "ly") ? 1 : 0

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(is_user_bot(i) || ((i != id) && (isMaster == 0) && (get_user_flags(i) & ADMIN_IMMUNITY)) ) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuBanSteamid[id])
    len += copy(menuBody[len], 511-len, _T("^n7. Ban by SteamID^n",id))
  else
    len += copy(menuBody[len], 511-len, _T("^n7. Ban by IP^n",id))
    
  if(g_menuSettings[id])
    len += formatex(menuBody[len], 511-len, _T("8. Ban for %d minutes^n",id), g_menuSettings[id])
  else
    len += copy(menuBody[len], 511-len, _T("8. Ban permanently^n",id))

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Ban Menu")
}

public cmdBanMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1
  g_menuSettings[id] = g_menuTempBanTime[0]
  g_menuBanSteamid[id] = 1
  displayBanMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* UnBan menu */

public actionUnBanMenu(id,key) {
  switch(key) {
    case 8: {
      displayUnBanMenu(id, ++g_menuPosition[id])
    }
    case 9: {
      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][0] - (8 + g_menuUnBanLine[id][1])
      if(g_menuUnBanLine[id][0] < 0) g_menuUnBanLine[id][0] = 0
      displayUnBanMenu(id, --g_menuPosition[id])
    }
    default: {
      new name[32], authid[32], ipaddress[24]
      get_user_authid(id, authid, 31)
      get_user_name(id, name, 31)
      get_user_ip(id, ipaddress, 23, 1)

      log_amx("UnBan: ^"%s<%d><%s><%s>^" unban ^"%s^"",
        name,get_user_userid(id),authid,ipaddress,g_menuUnBanText[id][key])

      #if !defined COLORED_ACTIVITY
      show_activity(id, name, _T("unban %s."), g_menuUnBanText[id][key])
      #else
      show_activity_color(id, name, _T("unban %s."), g_menuUnBanText[id][key])
      #endif

      if(g_menuUnBanType[id] == 1) {
        server_cmd("removeip ^"%s^";writeip", g_menuUnBanText[id][key])
        server_exec()
        console_print(id, _T("IP ^"%s^" removed from ban list."), g_menuUnBanText[id][key])
      }
      else {
        server_cmd("removeid %s;writeid", g_menuUnBanText[id][key])
        console_print(id, _T("AuthID ^"%s^" removed from ban list."), g_menuUnBanText[id][key])
      }

      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][1] = 0
      displayUnBanMenu(id, g_menuPosition[id] = 0)
    }
  }
  return PLUGIN_HANDLED
}

checkSTEAMID(const steamid[]) {
  new len = strlen(steamid)
  if(len > 10 && equali(steamid, "STEAM_", 6) && steamid[7] == ':' && steamid[9] == ':' && str_to_num(steamid[10])) {
    return 1
  }
  return 0
}

checkIP(ip[]) {
  new len = strlen(ip)
  new dots = 0, i = 0
  while(isdigit(ip[i]) || ip[i]=='.')
    if(ip[i++] == '.')
      ++dots
  if(i == len && dots == 3) {
    return 1
  }
  return 0
}

displayUnBanMenu(id,pos) {
  if(pos < 0)
    return

  new menuBody[512]
  new b = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yUnBan Menu\R%d^n\w^n",id) :
    _T("UnBan Menu %d^n^n",id),pos+1)

  new keys = (1<<9)
  new textlen, line
  new type = g_menuUnBanType[id]
  new temp[32], banTime[32], disable

  if(file_exists(g_bannedCfgFile[type])) {
    line = g_menuUnBanLine[id][0]
    while((line = read_file(g_bannedCfgFile[type], line, g_menuSelect[id], 63, textlen))) {
      temp[0] = '^0'
      banTime[0] = '^0'
      g_menuUnBanText[id][b][0] = '^0'
      disable = 0
      if(textlen < 9)
        disable = 1
      else if(parse(g_menuSelect[id], temp, 31, banTime, 31, g_menuUnBanText[id][b], 31) != 3)
        disable = 1
      else if((type == 1 && !checkIP(g_menuUnBanText[id][b])) || (type == 0 && !checkSTEAMID(g_menuUnBanText[id][b])))
        disable = 1
      if(disable == 0) {
        keys |= (1<<b)
        if(g_coloredMenus)
          len += formatex(menuBody[len], 511-len, "%d. %s\R\r%s^n\w", b, g_menuUnBanText[id][b++], banTime)
        else
          len += formatex(menuBody[len], 511-len, "%d. %s ( %s )^n", b, g_menuUnBanText[id][b++], banTime)
      }
      else {
        if(g_coloredMenus)
          len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, g_menuUnBanText[id][b++])
        else
          len += formatex(menuBody[len], 511-len, "#. %s^n", b, g_menuUnBanText[id][b++])
      }
      if(b == 8) break
    }

    if(b == 0) {
      if(type == 0)
        len += copy(menuBody[len], 511-len, _T("No SteamID found^n", id))
      else
        len += copy(menuBody[len], 511-len, _T("No IP found^n", id))
    }

    if(b == 8 && read_file(g_bannedCfgFile[type], line, g_menuSelect[id], 63, textlen) > 0) {
      formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
        pos ? _T("Back",id)
            : _T("Exit",id))
      keys |= (1<<8)
    }
    else
      formatex(menuBody[len], 511-len, "^n0. %s",
        pos ? _T("Back",id)
            : _T("Exit",id))

    g_menuUnBanLine[id][1] = line - g_menuUnBanLine[id][0]
    g_menuUnBanLine[id][0] = line
    show_menu(id, keys, menuBody, -1, "UnBan  Menu")
  }

  return
}

public actionUnBanMenuType(id,key) {
  switch(key) {
    case 9: return PLUGIN_HANDLED
    default: {
      g_menuUnBanType[id] = key // 0 = SteamID, 1 = IP
      g_menuUnBanLine[id][0] = g_menuUnBanLine[id][1] = 0
      displayUnBanMenu(id, g_menuPosition[id] = 0)
    }
  }
  return PLUGIN_HANDLED
}

displayUnBanMenuType(id) {
  new menuBody[512]

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yUnBan SteamID or IP?^n\w^n",id) :
    _T("UnBan SteamID or IP?^n^n",id))

  new keys = (1<<0)|(1<<1)|(1<<9)

  len += copy(menuBody[len], 511-len, _T("1. SteamID^n",id))
  len += copy(menuBody[len], 511-len, _T("2. IP^n",id))
  copy(menuBody[len], 511-len, _T("^n0. Exit",id))

  show_menu(id, keys, menuBody, -1, "UnBan SteamID or IP?")
}

public cmdUnBanMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuUnBanType[id] = -1
  displayUnBanMenuType(id)
  return PLUGIN_HANDLED
}

/* Slap/Slay */

public actionSlapMenu(id,key) {
  switch(key) {
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= 4
      switch(g_menuOption[id]) {
        case 1: g_menuSettings[id] = 0
        case 2: g_menuSettings[id] = 1
        case 3: g_menuSettings[id] = 5
      }
      displaySlapMenu(id, g_menuPosition[id])
    }
    case 8: displaySlapMenu(id, ++g_menuPosition[id])
    case 9: displaySlapMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]

      new name2[32]
      get_user_name(player, name2, 31)

      if(!is_user_alive(player)) {
        client_print(id, print_chat, _T("That action can't be performed on dead player ^"%s^"."), name2)
        displaySlapMenu(id, g_menuPosition[id])
        return PLUGIN_HANDLED
      }

      new authid[32], authid2[32], name[32], ipaddress[24], ipaddress2[24]

      get_user_name(id, name, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_ip(id, ipaddress, 23, 1)
      get_user_ip(player, ipaddress2, 23, 1)

      if(g_menuOption[id]) {
        log_amx("Cmd: ^"%s<%d><%s><%s>^" slap with %d damage ^"%s<%d><%s><%s>^"",
          name,get_user_userid(id),authid,ipaddress,g_menuSettings[id],name2,get_user_userid(player),authid2,ipaddress2)
        #if !defined COLORED_ACTIVITY
        show_activity(id, name, _T("slap %s with %d damage."), name2, g_menuSettings[id])
        #else
        show_activity_color(id, name, _T("slap %s with %d damage."), name2, g_menuSettings[id])
        #endif
      }
      else {
        log_amx("Cmd: ^"%s<%d><%s><%s>^" slay ^"%s<%d><%s><%s>^"",
          name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2)
        #if !defined COLORED_ACTIVITY
        show_activity(id, name, _T("slay %s."), name2)
        #else
        show_activity_color(id, name, _T("slay %s."), name2)
        #endif
      }

      if(g_menuOption[id])
        user_slap(player, (get_user_health(player) >  g_menuSettings[id]) ? g_menuSettings[id] : 0)
      else
        user_kill(player, 1)

      displaySlapMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displaySlapMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\ySlap/Slay Menu\R%d/%d^n\w^n",id) :
    _T("Slap/Slay Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0
  
  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(!is_user_alive(i) || ((i != id) && (isSupreme == 0) && (get_user_flags(i) & ADMIN_IMMUNITY))) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuOption[id])
    len += formatex(menuBody[len], 511-len, _T("^n8. Slap with %d damage^n",id), g_menuSettings[id])
  else
    len += copy(menuBody[len], 511-len, _T("^n8. Slay^n",id))

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len,_T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Slap/Slay Menu")
}

public cmdSlapMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1
  g_menuSettings[id] = 0

  displaySlapMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Kick */

public actionKickMenu(id,key) {
  switch(key) {
    case 8: displayKickMenu(id, ++g_menuPosition[id])
    case 9: displayKickMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 8 + key]

      new authid[32], authid2[32], name[32], name2[32], ipaddress[24], ipaddress2[24]
      get_user_name(id, name, 31)
      get_user_name(player, name2, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_ip(id, ipaddress, 23, 1)
      get_user_ip(player, ipaddress2, 23, 1)
      new userid2 = get_user_userid(player)

      log_amx("Kick: ^"%s<%d><%s><%s>^" kick ^"%s<%d><%s><%s>^"",
          name,get_user_userid(id),authid,ipaddress,name2,userid2,authid2,ipaddress2)

      #if !defined COLORED_ACTIVITY
      show_activity(id, name, _T("kick %s."), name2)
      #else
      show_activity_color(id, name, _T("kick %s."), name2)
      #endif

      #if !defined NO_STEAM
      server_cmd("kick #%d", userid2)
      #else
      client_cmd(player, "echo ^"Kicked.^";disconnect")
      #endif

      displayKickMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayKickMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 8

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yKick Menu\R%d/%d^n\w^n",id) :
    _T("Kick Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 8 + ((g_menuPlayersNum[id] % 8) ? 1 : 0)))

  new end = start + 8
  new keys = (1<<9)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0
  
  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if((i != id) && (isSupreme == 0) && (get_user_flags(i) & ADMIN_IMMUNITY)) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len,_T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Kick Menu")
}

public cmdKickMenu(id,level,cid) {
  if(cmd_access(id,level,cid,1))
    displayKickMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Team menu */
public actionTeamMenu(id,key) {
  switch(key) {
    case 7: {
      ++g_menuOption[id]
      if(g_menuOption[id] == 3 && g_cstrikeRunning && get_cvarptr_num(g_cvar_allow_spectators) == 0) g_menuOption[id] = 1
      if(g_menuOption[id] == 4) g_menuOption[id] = 1
      displayTeamMenu(id, g_menuPosition[id])
    }
    case 8: displayTeamMenu(id, ++g_menuPosition[id])
    case 9: displayTeamMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]
      new authid[32], authid2[32], name[32], name2[32], ipaddress[24], ipaddress2[24]
      get_user_name(id, name, 31)
      get_user_name(player, name2, 31)
      get_user_authid(id, authid, 31)
      get_user_authid(player, authid2, 31)
      get_user_ip(id, ipaddress, 23, 1)
      get_user_ip(player, ipaddress, 23, 1)

      switch(g_menuOption[id]) {
        case 1: {
          if(g_cstrikeRunning) {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"TERRORISTS")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("TERRORISTS"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("TERRORISTS"))
            #endif
          }
          else if(g_dodRunning) {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"ALLIES")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("ALLIES"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("ALLIES"))
            #endif
          }
          else {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"TEAM 1")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("TEAM 1"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("TEAM 1"))
            #endif
          }
        }
        case 2: {
          if(g_cstrikeRunning) {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"CTs")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("CTs"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("CTs"))
            #endif
          }
          else if(g_dodRunning) {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"AXIS")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("AXIS"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("AXIS"))
            #endif
          }
          else {
            log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
              name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"TEAM 2")
            #if !defined COLORED_ACTIVITY
            show_activity(id, name, _T("transfer %s to %s."), name2, _T("TEAM 2"))
            #else
            show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("TEAM 2"))
            #endif
          }
        }
        case 3: {
          log_amx("Cmd: ^"%s<%d><%s><%s>^" transfer ^"%s<%d><%s><%s>^" (to ^"%s^")",
            name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2,"SPECTATORS")
          #if !defined COLORED_ACTIVITY
          show_activity(id, name, _T("transfer %s to %s."), name2, _T("SPECTATORS"))
          #else
          show_activity_color(id, name, _T("transfer %s to %s."), name2, _T("SPECTATORS"))
          #endif
        }
      }

      if(!g_cstrikeRunning || !g_funRunning) {
        new limitt
        if(g_cstrikeRunning) {
          limitt = get_cvarptr_num(g_cvar_mp_limitteams)
          set_cvarptr_num(g_cvar_mp_limitteams, 0)
        }
        user_kill(player, 1)
        new szTeam[2]
        if(g_cstrikeRunning && g_menuOption[id] == 3) {
          num_to_str(6, szTeam, 1)
        }
        else {
          num_to_str(g_menuOption[id], szTeam, 1)
        }
        engclient_cmd(player, "jointeam", szTeam)
        if(g_menuOption[id] != 3) {
          if(g_cstrikeRunning) {
            engclient_cmd(player, "joinclass", "6")
          }
        }
        //show_menu(player, 0, "NULL", 0, "NULL")
        if(g_cstrikeRunning) {
          client_cmd(player, "slot5")
          set_cvarptr_num(g_cvar_mp_limitteams, limitt)
        }
      }
      else {
        set_user_team_cs(player, g_menuOption[id], "", (is_user_alive(player) && g_menuOption[id] != 3) ? 1 : 0, 1)
        if(g_menuOption[id] == 3) user_kill(player, 1)
      }

      displayTeamMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayTeamMenu(id, pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  static menuBody[512]
  new b = 0
  new i, iteam
  new name[32], team[7]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yTeam Menu\R%d/%d^n\w^n",id) :
    _T("Team Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]
    
  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0

  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)
    
    if(g_cstrikeRunning && (g_funRunning || g_vexdRunning)) {
      iteam = g_funRunning ? get_user_team_cs(i) : get_offset_int(i, 114)
      switch(iteam) {
        case 1: copy(team, 6, "TERROR")
        case 2: copy(team, 6, "CT")
        case 3: copy(team, 6, "SPECTA")
      }
    }
    else {
      iteam = get_user_team(i, team, 6)
      if(equali(team, "TERROR", 6) || equali(team, "Allies", 6)) iteam = 1
      if(equali(team, "CT", 2) || equali(team, "Axis", 4)) iteam = 2
      if(equali(team, "SPECTA", 6)) iteam = 3
    }
  
    if((iteam == g_menuOption[id]) || ((i != id) && (isSupreme == 0) && (get_user_flags(i) & ADMIN_IMMUNITY))) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s\R%s^n\w", b, name, team)
      else
        len += formatex(menuBody[len], 511-len, "#. %s   %s^n", name, team)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s\y\R%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name, team)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s   %s^n", ++b, name, team)
    }
  }

  switch(g_menuOption[id]) {
    case 1: {
      if(g_cstrikeRunning) {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TERRORISTS",id))
      }
      else if(g_dodRunning) {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("ALLIES",id))
      }
      else {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TEAM 1",id))
      }
    }
    case 2: {
      if(g_cstrikeRunning) {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("CTs",id))
      }
      else if(g_dodRunning) {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("AXIS",id))
      }
      else {
        len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("TEAM 2",id))
      }
    }
    case 3: {
      len += formatex(menuBody[len], 511-len, _T("^n8. Transfer to %s^n",id), _T("SPECTATORS",id))
    }
  }

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Team Menu")
}

public cmdTeamMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  g_menuOption[id] = 1

  displayTeamMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

/* Client cmds menu */
public actionClcmdMenu(id,key) {
  switch(key) {
    case 7: {
      ++g_menuOption[id]
      g_menuOption[id] %= g_menuSelectNum[id]
      displayClcmdMenu(id, g_menuPosition[id])
    }
    case 8: displayClcmdMenu(id, ++g_menuPosition[id])
    case 9: displayClcmdMenu(id, --g_menuPosition[id])
    default:{
      new player = g_menuPlayers[id][g_menuPosition[id] * 7 + key]
      new flags = g_clcmdMisc[g_menuSelect[id][g_menuOption[id]]][1]
      if(is_user_connected(player)) {
        new command[64], authid[32], name[32], userid[32]
        copy(command, 63, g_clcmdCmd[g_menuSelect[id][g_menuOption[id]]])
        get_user_authid(player, authid, 31)
        get_user_name(player, name, 31)
        num_to_str(get_user_userid(player), userid, 31)
        replace(command, 63, "%userid%", userid)
        replace(command, 63, "%authid%", authid)
        replace(command, 63, "%name%", name)
        if(flags & 1) {
          server_cmd("%s", command)
          server_exec()
        }
        else if(flags & 2) {
          new cmd[32], cmdaccess[32], playerflags, cmdflags, noaccess = 0
          playerflags = get_user_flags(id)
          parse(command, cmd, 31)
          if(get_cmdaccess(cmd, cmdaccess, 31)) {
            cmdflags = read_flags(cmdaccess)
            if((playerflags & cmdflags) != cmdflags) noaccess = 1
            if(noaccess) {
              set_user_flags(id, cmdflags)
            }
            client_cmd(id, command)
            if(noaccess) {
              new param[3]
              param[0] = id
              param[1] = playerflags
              set_task(0.3, "setOldFlags", 400+cmdflags+id, param, 3)
            }
          }
          else client_cmd(id, command)
        }
        else if(flags & 4)
          client_cmd(player, command)
      }
      if(flags & 8) displayClcmdMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

public setOldFlags(param[]) {
  new id = param[0]
  if(is_user_connected(id)) {
    remove_user_flags(id)
    set_user_flags(id, param[1])
  }
}

displayClcmdMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  static menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 7

  if(start >= g_menuPlayersNum[id])
    start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yClient Commands Menu\R%d/%d^n\w^n",id) :
    _T("Client Commands Menu %d/%d^n^n",id),
    pos+1, (g_menuPlayersNum[id] / 7 + ((g_menuPlayersNum[id] % 7) ? 1 : 0)))

  new end = start + 7
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0
  
  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(!g_menuSelectNum[id] || ((i != id) && (isSupreme == 0) && (get_user_flags(i) & ADMIN_IMMUNITY))) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuSelectNum[id]) {
    if(is_translated(g_clcmdName[g_menuSelect[id][g_menuOption[id]]])) {
      len += formatex(menuBody[len], 511-len, "^n8. %s^n", _T(g_clcmdName[g_menuSelect[id][g_menuOption[id]]],id))
    }
    else {
      len += formatex(menuBody[len], 511-len, "^n8. %s^n", g_clcmdName[g_menuSelect[id][g_menuOption[id]]])
    }
  }
  else
    len += copy(menuBody[len], 511-len, _T("^n8. No cmds available^n",id))

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
      pos ? _T("Back",id)
          : _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
      pos ? _T("Back",id)
          : _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Client Commands Menu")
}

public cmdClcmdMenu(id,level,cid) {
  if(!cmd_access(id,level,cid,1))
    return PLUGIN_HANDLED

  new flags = get_user_flags(id)

  g_menuSelectNum[id] = 0

  for(new a = 0; a < g_clcmdNum; ++a)
    if((g_clcmdMisc[a][0] & flags) == g_clcmdMisc[a][0])
      g_menuSelect[id][g_menuSelectNum[id]++] = a

  g_menuOption[id] = 0

  displayClcmdMenu(id, g_menuPosition[id] = 0)
  return PLUGIN_HANDLED
}

load_settings(const szFilename[]) {
  if(!file_exists(szFilename))
    return 0

  g_clcmdNum = 0

  new text[192], szFlags[32], szAccess[32]
  new a, pos = 0

  while(g_clcmdNum < MAX_CLCMDS && (pos = read_file(szFilename, pos, text, 191, a))) {
    if(!a || text[0] == ';' || text[0] == '/' || text[0] == '#') continue

    if(parse(text, g_clcmdName[g_clcmdNum], 31,
      g_clcmdCmd[g_clcmdNum], 63, szFlags, 31, szAccess, 31) > 3) {
      replace_all(g_clcmdCmd[g_clcmdNum], 63, "\'", "^"")

      g_clcmdMisc[g_clcmdNum][1] = read_flags(szFlags)
      g_clcmdMisc[g_clcmdNum][0] = read_flags(szAccess)
      ++g_clcmdNum
    }
  }

  return 1
}
