/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Server command:
 * amx_scrollmsg <message> <frequency in seconds> [color in RRRGGGBBB format or "?"]
 *
 * You can use "?" instead of the RRRGGGBBB format for a random color of on each map.
 *
 * Note: this plugin does NOT display localized messages, even if it
 *       uses the Translator. It would eat your resources at breakfast
 *       and we decided that the scrolling message would be global.
 *
 * You can use the imessage plugin to display localized messages.
 *
 */

#include <translator>
#include <amxmod>
#include <amxmisc>

const Float:SPEED    = 0.3
const SCROLLMSG_SIZE = 384

new g_startPos
new g_endPos
new g_oriScrollMsg[SCROLLMSG_SIZE]
new g_scrollMsg[SCROLLMSG_SIZE]
new g_displayMsg[SCROLLMSG_SIZE]
new Float:g_xPos
new g_Length
new g_Frequency
new bool:g_colorType
new g_values[3]

new g_cvar_hostname

const TASK_showMsg_msgInit = 123

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("scrollmsg")
  register_plugin(_T("Scrolling Message"),AMX_VERSION,"AMX Mod Team")
  register_srvcmd("amx_scrollmsg","setMessage")
  g_cvar_hostname = get_cvar_pointer("hostname")
}

public showMsg() {
  new a = g_startPos, i = 0

  while(a < g_endPos)
    g_displayMsg[i++] = g_scrollMsg[a++]

  g_displayMsg[i] = 0

  if(g_endPos < g_Length)
    ++g_endPos

  if(g_xPos > 0.35)
    g_xPos -= 0.0063
  else {
    ++g_startPos
    g_xPos = 0.35
  }

  if(g_colorType)
    set_hudmessage(g_values[0], g_values[1], g_values[2], g_xPos, 0.90, 0, SPEED, SPEED, 0.05, 0.05, 2)
  else
    set_hudmessage(200, 100, 0, g_xPos, 0.90, 0, SPEED, SPEED, 0.05, 0.05, 2)

  show_hudmessage(0, g_displayMsg)
}

public msgInit() {
  g_endPos = 1
  g_startPos = 0
  g_xPos = 0.65
  copy(g_scrollMsg, SCROLLMSG_SIZE-1, g_oriScrollMsg)
  new hostname[64]
  get_cvarptr_string(g_cvar_hostname, hostname, 63)
  replace(g_scrollMsg, SCROLLMSG_SIZE-1, "%hostname%", hostname)
  g_Length = strlen(g_scrollMsg)
  set_task(SPEED, "showMsg", TASK_showMsg_msgInit, "", 0, "a", g_Length + 48)
  client_print(0, print_console, "%s", g_scrollMsg)
}

public setMessage(id,level,cid) {
  if(read_argc() < 3)
    return PLUGIN_HANDLED

  remove_task(TASK_showMsg_msgInit)
  read_argv(1, g_oriScrollMsg, SCROLLMSG_SIZE-1)
  new mytime[32]
  read_argv(2, mytime, 31)
  g_Frequency = str_to_num(mytime)
  if(g_Frequency > 0) {
    new mycol[12]
    if(read_argv(3, mycol, 11)) { // RRRGGGBBB
      trim(mycol) // Clean the text
      new sred[4], sgreen[4], sblue[4]
      copy(sred, 3, mycol[0])
      copy(sgreen, 3, mycol[3])
      copy(sblue, 3, mycol[6])
      if(isdigit(mycol[0])) {
        new ired = str_to_num(sred)
        new igreen = str_to_num(sgreen)
        new iblue = str_to_num(sblue)
        if(ired < 0 || ired > 255)
          ired = random_num(32, 255)
        if(igreen < 0 || igreen > 255)
          igreen = random_num(32, 255)
        if(iblue < 0 || iblue > 255)
          iblue = random_num(32, 255)
        g_values[0] = ired
        g_values[1] = igreen
        g_values[2] = iblue
      }
      else {
        g_values[0] = random_num(32, 255)
        g_values[1] = random_num(32, 255)
        g_values[2] = random_num(32, 255)
      }
      g_colorType = true
    }
    else
      g_colorType = false
    new minimal = floatround((g_Length + 48) * (SPEED + 0.1))
    if(g_Frequency < minimal) {
      server_print( _T("Minimal frequency for this message is %d seconds."), minimal)
      g_Frequency = minimal
    }
    server_print(_T("Scrolling message displaying frequency: %d:%02d minutes."), g_Frequency / 60, g_Frequency % 60)
    set_task(float(g_Frequency), "msgInit", TASK_showMsg_msgInit, "", 0, "b")
  }
  else
    server_print(_T("Scrolling message disabled."))
  return PLUGIN_HANDLED
}
