/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Fun Module is required!
 * 
 * Admin command:
 * amx_teleportmenu - displays teleport menu
 *
 */
 
/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define FLAG_AMX_TELEPORTMENU ADMIN_LEVEL_A

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>
#include <fun>

new g_menuPosition[33]
new g_menuPlayers[33][32]
new g_menuPlayersNum[33]
new g_menuOption[33] = {-1, ...}
new g_menuOrigin[33][3]
new g_coloredMenus

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("telemenu")
  register_plugin(_T("Teleport Menu"),AMX_VERSION,"AMX Mod Team")
  register_clcmd("amx_teleportmenu","cmdTelMenu",FLAG_AMX_TELEPORTMENU,_T("- displays teleport menu"))
  register_menucmd(register_menuid("Teleport Menu"),1023,"actionTelMenu")
  g_coloredMenus = colored_menus()
}

public actionTelMenu(id,key) {
  switch(key) {
    case 6: {
      g_menuOption[id] = 1 - g_menuOption[id]
      displayTelMenu(id, g_menuPosition[id])
    }     
    case 7: {
      if(g_menuOption[id] < 0) /* unlocking position for the first time */
        g_menuOption[id] = 0
      get_user_origin(id, g_menuOrigin[id])
      g_menuOrigin[id][2] += 2 /* avoid player's blocking in a climb, keep this origin into menu */
      displayTelMenu(id, g_menuPosition[id])
    }
    case 8: displayTelMenu(id, ++g_menuPosition[id])
    case 9: displayTelMenu(id, --g_menuPosition[id])
    default: {
      new player = g_menuPlayers[id][g_menuPosition[id] * 6 + key]
      new name2[32]
      get_user_name(player, name2, 31)

      if(!is_user_alive(player)) {
        client_print(id, print_chat, _T("That action can't be performed on dead player ^"%s^"."), name2)
        displayTelMenu(id, g_menuPosition[id])
        return PLUGIN_HANDLED
      }

      if(g_menuOption[id] > 0) {
        set_user_origin(player, g_menuOrigin[id])
      }
      else {
        new origin[3]
        get_user_origin(id, origin)
        set_user_origin(player, origin)
      }

      new authid[32], authid2[32], ipaddress[24], ipaddress2[24], name[32]

      get_user_authid(id, authid, 31)
      get_user_ip(id, ipaddress, 23, 1)
      get_user_authid(player, authid2, 31)
      get_user_ip(player, ipaddress2, 23, 1)
      get_user_name(id, name, 31)

      log_amx("Cmd: ^"%s<%d><%s><%s>^" teleport ^"%s<%d><%s><%s>^"",
          name,get_user_userid(id),authid,ipaddress,name2,get_user_userid(player),authid2,ipaddress2)

      #if !defined COLORED_ACTIVITY
      show_activity(id, name, _T("teleport %s."), name2)
      #else
      show_activity_color(id, name, _T("teleport %s."), name2)
      #endif

      displayTelMenu(id, g_menuPosition[id])
    }
  }
  return PLUGIN_HANDLED
}

displayTelMenu(id,pos) {
  if(pos < 0)
    return

  get_players(g_menuPlayers[id], g_menuPlayersNum[id])

  new menuBody[512]
  new b = 0
  new i
  new name[32]
  new start = pos * 6
  new bool:blockMenu = (is_user_alive(id) && g_menuOption[id] < 1) ? true : false

  if(start >= g_menuPlayersNum[id])
      start = pos = g_menuPosition[id] = 0

  new len = formatex(menuBody, 511, g_coloredMenus ?
    _T("\yTeleport Menu\R%d/%d^n\w^n",id) :
    _T("Teleport Menu %d/%d^n^n",id),
      pos+1, (g_menuPlayersNum[id] / 6 + ((g_menuPlayersNum[id] % 6) ? 1 : 0)))

  new end = start + 6
  new keys = (1<<9)|(1<<7)

  if(end > g_menuPlayersNum[id])
    end = g_menuPlayersNum[id]

  new isSupreme = (get_user_flags(id) & ADMIN_SUPREME) ? 1 : 0
  
  for(new a = start; a < end; ++a) {
    i = g_menuPlayers[id][a]
    get_user_name(i, name, 31)

    if(blockMenu || !is_user_alive(i) || ((get_user_flags(i) & ADMIN_IMMUNITY) && (i != id) && (isSupreme == 0))) {
      ++b
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "\d%d. %s^n\w", b, name)
      else
        len += formatex(menuBody[len], 511-len, "#. %s^n", name)
    }
    else {
      keys |= (1<<b)
      if(g_coloredMenus)
        len += formatex(menuBody[len], 511-len, "%d. %s%s^n\w", ++b, is_user_realadmin(i, 1) ? "\r" : "", name)
      else
        len += formatex(menuBody[len], 511-len, "%d. %s^n", ++b, name)
    }
  }

  if(g_menuOption[id] > 0) { // 1
    keys |= (1<<6)
    len += formatex(menuBody[len], 511-len, _T("^n7. To location: %d %d %d^n",id),
        g_menuOrigin[id][0], g_menuOrigin[id][1], g_menuOrigin[id][2])
  }
  else if(g_menuOption[id]) { // -1
    if(g_coloredMenus)
      len += copy(menuBody[len], 511-len, _T("^n\d7. Current location^n\w",id))
    else
      len += copy(menuBody[len], 511-len, _T("^n#. Current location^n",id))
  }
  else { // 0
    keys |= (1<<6)
    len += copy(menuBody[len], 511-len, _T("^n7. Current location^n",id))
  }

  len += copy(menuBody[len], 511-len, _T("8. Save location^n",id))

  if(end != g_menuPlayersNum[id]) {
    formatex(menuBody[len], 511-len, _T("^n9. More...^n0. %s",id),
        pos ? _T("Back",id) :
              _T("Exit",id))
    keys |= (1<<8)
  }
  else
    formatex(menuBody[len], 511-len, "^n0. %s",
        pos ? _T("Back",id) :
              _T("Exit",id))

  show_menu(id, keys, menuBody, -1, "Teleport Menu")
}

public cmdTelMenu(id,level,cid) {
  if(cmd_access(id,level,cid,1))
    displayTelMenu(id, g_menuPosition[id] = 0)

  return PLUGIN_HANDLED
}
