/*
 * Copyright (c) 2002-2010 - Aleksander Naszko, Julien Laurent
 *
 *    This file is part of AMX Mod.
 *
 *    AMX Mod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    AMX Mod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with AMX Mod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

/*
 * Displays a console motd at the client connection
 *
 */

/******************************************************************************/
// If you change one of the following settings, do not forget to recompile
// the plugin and to install the new .amx file on your server.
// You can find the list of admin flags in the amx/examples/include/amxconst.inc file.

#define SHOW_MODS
#define READ_FROM_FILE
//#define SHOW_TIME_AND_IP

/******************************************************************************/

#include <translator>
#include <amxmod>
#include <amxmisc>

new g_cstrikeRunning
new g_dodRunning

#if defined READ_FROM_FILE
new g_motdFile[64]
#endif

new g_cvar_amx_nextmap
new g_cvar_hostname
new g_cvar_mp_timelimit
new g_cvar_mp_c4timer
new g_cvar_mp_friendlyfire
new g_cvar_sv_visiblemaxplayers

#if defined SHOW_MODS
enum {
  // MetaMod plugins cvars
  g_cvar_amx_version,
  g_cvar_DF_entversion,
  g_cvar_axn_version,
  g_cvar_chicken_version,
  g_cvar_pb_version,
  g_cvar_logmod_version,
  g_cvar_statsme_version,
  g_cvar_booster_version,
  g_cvar_hlguard_version,
  g_cvar_cdversion,
  g_cvar_cdrequired,
  g_cvar___sxei_clt_ver,
  g_cvar___sxei_required,
  // AMX Mod plugins cvars
  g_cvar_AntiRoxx,
  g_cvar_amx_ptb_version,
  g_cvar_badcamper_version,
  g_cvar_csdm_version,
  g_cvar_atac_version,
  g_cvar_atac_status
};

new const g_cvars[][] = {
  // MetaMod plugins cvars
  {g_cvar_amx_version, "amx_version"},
  {g_cvar_DF_entversion, "DF_entversion"},
  {g_cvar_axn_version, "axn_version"},
  {g_cvar_chicken_version, "chicken_version"},
  {g_cvar_pb_version, "pb_version"},
  {g_cvar_logmod_version, "logmod_version"},
  {g_cvar_statsme_version, "statsme_version"},
  {g_cvar_booster_version, "booster_version"},
  {g_cvar_hlguard_version, "hlguard_version"},
  {g_cvar_cdversion, "cdversion"},
  {g_cvar_cdrequired, "cdrequired"},
  {g_cvar___sxei_clt_ver, "__sxei_clt_ver"},
  {g_cvar___sxei_required, "__sxei_required"},
  // AMX Mod plugins cvars
  {g_cvar_AntiRoxx, "AntiRoxx"},
  {g_cvar_amx_ptb_version, "amx_ptb_version"},
  {g_cvar_badcamper_version, "badcamper_version"},
  {g_cvar_csdm_version, "csdm_version"},
  {g_cvar_atac_version, "atac_version"},
  {g_cvar_atac_status, "atac_status"}
}
#endif

public plugin_init() {
  if(cvar_exists("amxmodx_version")) load_module("error")
  load_translations("welcomemsg")
  register_plugin(_T("Welcome Message"),AMX_VERSION,"AMX Mod Team")
  g_cstrikeRunning = (is_running("cstrike") || is_running("czero"))
  g_dodRunning = is_running("dod")
#if defined READ_FROM_FILE
  build_path(g_motdFile, 63, "$configdir/conmotd.txt")
#endif
}

public plugin_cfg() {
  g_cvar_amx_nextmap = get_cvar_pointer("amx_nextmap")
  g_cvar_hostname = get_cvar_pointer("hostname")
  g_cvar_mp_timelimit = get_cvar_pointer("mp_timelimit")
  g_cvar_mp_c4timer = get_cvar_pointer("mp_c4timer")
  g_cvar_mp_friendlyfire = get_cvar_pointer("mp_friendlyfire")
  g_cvar_sv_visiblemaxplayers = get_cvar_pointer("sv_visiblemaxplayers")
#if defined SHOW_MODS
  new i = -1
  while(++i < sizeof(g_cvars)) {
    g_cvars[i][0] = get_cvar_pointer(g_cvars[i][1])
  }
#endif
}

new const g_Bar[] = "=============="

public client_connect(id) {
  if(is_user_bot(id)) return

  static name[32], hostname[64], nextmap[32], mapname[32]

  get_cvarptr_string(g_cvar_hostname, hostname, 63) 
  get_user_name(id, name, 31)

  get_mapname(mapname, 31)
  get_cvarptr_string(g_cvar_amx_nextmap, nextmap, 31)

  client_cmd(id, "echo ;echo %s%s%s%s", g_Bar, g_Bar, g_Bar, g_Bar)
  client_cmd(id, _T("echo ^"   Hello %s, welcome to %s^""), name, hostname)

#if defined SHOW_TIME_AND_IP  
  static stime[64], ip[32]
  get_time("%A %B %d, %Y - %H:%M:%S", stime, 63)
  get_user_ip(id,ip,31)
  client_cmd(id, _T("echo ^"   Today is %s^""), stime)
  client_cmd(id, _T("echo ^"   You are playing from: %s^""), ip)
#endif

  new maxplayers = get_cvarptr_num(g_cvar_sv_visiblemaxplayers)
  if(maxplayers < 0) maxplayers = get_maxplayers()
  client_cmd(id, _T("echo ^"   Players on server: %d/%d^""), get_playersnum(), maxplayers)
  client_cmd(id, _T("echo ^"   Current map: %s. Next map: %s^""), mapname, nextmap)

  // Time limit and time remaining
  new Float:mp_timelimit = get_cvarptr_float(g_cvar_mp_timelimit)
  if(mp_timelimit) {
    new timeleft = get_timeleft()
    if(timeleft > 0) {
      client_cmd(id, _T("echo ^"   Time left: %d:%02d of %.0f minutes^""), timeleft / 60, timeleft % 60, mp_timelimit)
    }
  }
  else {
    client_cmd(id, _T("echo ^"   No time limit^""))
  }

  // C4 and FF
  if(g_cstrikeRunning || g_dodRunning)
    client_cmd(id, _T("echo ^"   Friendly fire is %s^""), get_cvarptr_float(g_cvar_mp_friendlyfire) ? _T("ON") : _T("OFF"))
  if(g_cstrikeRunning)
    client_cmd(id, _T("echo ^"   C4 timer is set to %.0f seconds^""), get_cvarptr_float(g_cvar_mp_c4timer))

  // Server Mods
#if defined SHOW_MODS
  static mod_ver[32]
  client_cmd(id, _T("echo ;echo ^"   Server mods:^""))

  // Metamod plugins
  if(g_cvars[g_cvar_amx_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_amx_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o AMX Mod %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_DF_entversion][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_DF_entversion][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o Ent Mod %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_axn_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_axn_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o AXN %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_chicken_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_chicken_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o Chicken %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_pb_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_pb_version][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o POD-Bot %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_logmod_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_logmod_version][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o LogMod %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_statsme_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_statsme_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o StatsMe %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_booster_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_booster_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o HL-Booster %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_hlguard_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_hlguard_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o HLGuard %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_cdversion][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_cdversion][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, _T("echo ^"   o Cheating-Death %s in %s mode^""),
    mod_ver, get_cvarptr_num(g_cvars[g_cvar_cdrequired][0]) ? _T("required") : _T("optional"))
  }
  if(g_cvars[g_cvar___sxei_clt_ver][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar___sxei_clt_ver][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, _T("echo ^"   o sXe Injected %s in %s mode^""),
    mod_ver, get_cvarptr_num(g_cvars[g_cvar___sxei_required][0]) ? _T("required") : _T("optional"))
  }

  // AMX Mod plugins
  if(g_cvars[g_cvar_AntiRoxx][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_AntiRoxx][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o AntiRoxx %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_amx_ptb_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_amx_ptb_version][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o PTB Team Balancer %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_badcamper_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_badcamper_version][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o Bad Camper %s^"", mod_ver)
  }
  if(g_cvars[g_cvar_csdm_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_csdm_version][0], mod_ver, 31) // new!
    if(mod_ver[0]) client_cmd(id, "echo ^"   o CSDM %s%s^"", mod_ver, _T(" (AMX Mod special version)"))
  }
  if(g_cvars[g_cvar_atac_version][0] != 0) {
    get_cvarptr_string(g_cvars[g_cvar_atac_version][0], mod_ver, 31)
    if(mod_ver[0]) client_cmd(id, "echo ^"   o ATAC %s%s^"", mod_ver, get_cvarptr_num(g_cvars[g_cvar_atac_status][0]) 
    ? _T(" (setinfo atac_status_off 1 to disable Live Status)") : "")
  }
#endif

  // Info. from custom file
#if defined READ_FROM_FILE
  if(file_exists(g_motdFile)) {
    static message[192]
    new len, line = 0
    client_cmd(id, "echo %s%s%s%s", g_Bar, g_Bar, g_Bar, g_Bar)   
    while(read_file(g_motdFile, line++, message, 191, len)) {
      client_cmd(id, "echo ^"%s^"", message)
    }
  }
#endif
  client_cmd(id, "echo %s%s%s%s", g_Bar, g_Bar, g_Bar, g_Bar)
}
